Package: statTarget
Type: Package
Title: Statistical Analysis of Molecular Profiles
Version: 1.42.0
Author: Hemi Luan
Maintainer: Hemi Luan <hemi.luan@gmail.com>
Depends: R (>= 3.6.0)
Imports:
        randomForest,plyr,pdist,ROC,utils,grDevices,graphics,rrcov,stats,
        pls,impute
VignetteBuilder: knitr
Suggests: testthat, BiocStyle, knitr, rmarkdown
Description: A streamlined tool provides a graphical user interface for
    quality control based signal drift correction (QC-RFSC), integration of
    data from multi-batch MS-based experiments, and the
    comprehensive statistical analysis in metabolomics and proteomics.
License: LGPL (>= 3)
URL: https://stattarget.github.io
biocViews: ImmunoOncology, Metabolomics, Proteomics, Machine Learning,
        Lipidomics, MassSpectrometry, QualityControl, Normalization,
        QC-RFSC, ComBat, DifferentialExpression, BatchEffect,
        Visualization, MultipleComparison,Preprocessing, Software
RoxygenNote: 7.2.1
NeedsCompilation: no
BuildVignettes: true
git_url: https://git.bioconductor.org/packages/statTarget
git_branch: RELEASE_3_23
git_last_commit: 5012ea6
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
Packaged: 2026-04-28 23:40:20 UTC; biocbuild
Built: R 4.6.0; ; 2026-04-29 05:02:55 UTC; unix

# Generated by roxygen2: do not edit by hand

export(mdsPlot)
export(predict_RF)
export(pvimPlot)
export(rForest)
export(shiftCor)
export(shiftCor_dQC)
export(statAnalysis)
import(impute)
import(pdist)
import(pls)
importFrom(ROC,AUC)
importFrom(ROC,rocdemo.sca)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,colors)
importFrom(grDevices,dev.copy2pdf)
importFrom(grDevices,dev.new)
importFrom(grDevices,dev.off)
importFrom(grDevices,palette)
importFrom(grDevices,pdf)
importFrom(grDevices,rainbow)
importFrom(grDevices,rgb)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,barplot)
importFrom(graphics,box)
importFrom(graphics,boxplot)
importFrom(graphics,grid)
importFrom(graphics,hist)
importFrom(graphics,identify)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,segments)
importFrom(graphics,strheight)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(plyr,.)
importFrom(plyr,daply)
importFrom(plyr,rlply)
importFrom(randomForest,MDSplot)
importFrom(randomForest,importance)
importFrom(randomForest,randomForest)
importFrom(rrcov,plot)
importFrom(stats,aggregate)
importFrom(stats,binomial)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,cov.wt)
importFrom(stats,density)
importFrom(stats,ecdf)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,loess)
importFrom(stats,median)
importFrom(stats,p.adjust)
importFrom(stats,pf)
importFrom(stats,prcomp)
importFrom(stats,predict)
importFrom(stats,qf)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,resid)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,shapiro.test)
importFrom(stats,t.test)
importFrom(stats,update)
importFrom(stats,var)
importFrom(stats,wilcox.test)
importFrom(utils,capture.output)
importFrom(utils,install.packages)
importFrom(utils,read.csv)
importFrom(utils,read.delim)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
importFrom(utils,write.csv)
importFrom(utils,write.table)

# statTarget2

For details see https://stattarget.github.io/docs/


Package vignettes: [Vignettes](https://stattarget.github.io/docs/my-new-doc/) 


Reference manual: [Manual](https://github.com/13479776/Picture/blob/master/statTarget-manual.pdf)


Demo data: [Data](https://stattarget.github.io/docs/demo/)


Example reports: [Reports](https://stattarget.github.io/docs/demo/)


Binary Package: [statTarget2_WindowsOnly.zip](https://github.com/13479776/Picture/raw/master/statTarget_2.0.0.zip)


### System requirements
--------------------------------------------------------------------

> Dependent on R (>= 3.3.0)

> If you did not install the R software yet,you can download R >= 3.3.0  from https://www.r-project.org


### Installation
--------------------------------------------------------------------
     
> Install the earlier versions of “statTarget” at the Bioconductor

    if (!requireNamespace("BiocManager", quietly=TRUE))
        install.packages("BiocManager")

    BiocManager::install("statTarget")

    library(statTarget)  ## `Load statTarget`. 
    
    
> Install the latest version of "statTarget2" at Github. copy this code into R
    
    Updating, Coming Back Soon! 
   
    Binary Package for Windows PC is provided. If you want the source code, please contact us. Thanks.


    
> IMPORTANCE: for mac PC,  XQuartz instead of X11 support should be installed for the Graphical User Interface (GUI). Download it from https://www.xquartz.org. 


> RGTK2 is a binding for R to the GTK2 library and dependent libraries, and a multi-platform package for creating graphical user interfaces. If you have any problems about RGTK2 installation, see the related installation information for R and GTK on Windows/Mac OS at https://gist.github.com/sebkopf/9405675. 


> We recommend the R 3.3.0 and RGtk2 2.20.31 for mac OS paltform. `The R 3.3.0 and RGtk2 2.20.31 sailed through the test.` If you have multiple versions of R framework installed, RSwitch  - a small GUI that allows you to switch between R versions quickly. Download it from https://r.research.att.com

bStat = 
  function(x, ci = 0.95) 
  {   # A function implemented by Diethelm Wuertz
    
    # Description:
    #   Calculates Basic Statistics
    
    # Arguments:
    #   x - an object which can be transformed by the function
    #       as.matrix() into an object of class matrix. 
    #   ci - a numeric value setting the confidence interval.
    
    # Value:
    #   a two-column data frame, where the first column takes the 
    #   value of the statistics, and the second its name, e.g.
    #   "nobs", "NAs",  "Minimum", "Maximum", "1. Quartile",  
    #   "3. Quartile",  "Mean", "Median", "Sum",  "SE Mean", 
    #   "LCL Mean", "UCL Mean", "Variance", "Stdev", "Skewness", 
    #   "Kurtosis")
    
    # FUNCTION:
    
    # Univariate/Multivariate:
    y = as.matrix(x)
    
    
    # Handle Column Names:
    if (is.null(colnames(y))) {
      Dim = dim(y)[2]
      if (Dim == 1) {
        colnames(y) = paste(substitute(x), collapse = ".")
      } else if (Dim > 1) {
        colnames(y) = 
          paste(paste(substitute(x), collapse = ""), 1:Dim, sep = "")
      }
    }
    
    # Internal Function - CL Levels:    
    cl.vals = function(x, ci) {
      x = x[!is.na(x)]
      n = length(x)
      if(n <= 1) return(c(NA, NA))
      se.mean = sqrt(stats::var(x)/n)
      t.val = qt((1 - ci)/2, n - 1)
      mn = mean(x)
      lcl = mn + se.mean * t.val
      ucl = mn - se.mean * t.val
      c(lcl, ucl)
    }        
    
    # Basic Statistics:
    nColumns = dim(y)[2]
    ans = NULL
    for (i in 1:nColumns) {
      X = y[, i]     
      # Observations:
      X.length = length(X)
      X = X[!is.na(X)]
      X.na = X.length - length(X)
      # Basic Statistics:
      z = c(
        X.length, X.na, min(X), max(X),
        as.numeric(quantile(X, prob = 0.25, na.rm = TRUE)), 
        as.numeric(quantile(X, prob = 0.75, na.rm = TRUE)), 
        mean(X), median(X), sum(X), sqrt(stats::var(X)/length(X)), 
        cl.vals(X, ci)[1], cl.vals(X, ci)[2], stats::var(X), 
        sqrt(stats::var(X)))    
      # Row Names:
      znames = c(
        "nobs", "NAs",  "Minimum", "Maximum", 
        "1. Quartile",  "3. Quartile",  "Mean", "Median", 
        "Sum",  "SE Mean", "LCL Mean", "UCL Mean", 
        "Variance", "Stdev")   
      # Output as data.frame
      result = matrix(z, ncol = 1)
      row.names(result) = znames    
      ans = cbind(ans, result) 
    }
    
    # Column Names:
    colnames(ans) = colnames(y)
    
    # Return Value:
    data.frame(round(ans, digits = 6))
  }


################################################################################

### P values This function displyed the adjusted p values with BH method for each pairwise
### comparison of sample groups, by intergrating the p values from Student T test for normally
### distributed variables and from Wilcoxon Mann Whitney test for non-normally distributed
### variables.  file The connection to the data in the Univariate file.  fdr The false discovery
### rate for conceptualizing the rate of type I errors in null hypothesis testing when conducting
### multiple comparisons.  A matrix with p values
pvalues <- function(file, fdr) {
    pwdfile = paste(getwd(), "/Univariate/DataTable.csv", sep = "")
    file = pwdfile
    x <- read.csv(file, sep = ",", header = TRUE)
    x.x = x[, 3:ncol(x)]
    rownames(x.x) = x[, 2]
    k = matrix(x[, 1], ncol = 1)
    slink = paste(getwd(), "/DataPretreatment", "/slink.csv", sep = "")
    slink = read.csv(slink, header = TRUE)
    x.n = cbind(k, x.x)
    sorted = x.n[order(x.n[, 1]), ]
    sorted.x = sorted[, -1]
    g = c()
    for (i in 1:nrow(sorted)) {
        if (any(g == sorted[i, 1])) {
            g = g
        } else {
            g = matrix(c(g, sorted[i, 1]), ncol = 1)
        }
    }
    NoF = nrow(g)
    dirout.pv = paste(getwd(), "/Univariate/Pvalues/", sep = "")
    dir.create(dirout.pv)
    dirout.sign = paste(getwd(), "/Univariate/Significant_Variables/", sep = "")
    dir.create(dirout.sign)
    for (i in 1:NoF) {
        for (j in 1:NoF) {
            if (i < j) {
                shapi = paste("ShapiroTest.", ExcName(i, slink), ".csv", sep = "")
                shapj = paste("ShapiroTest.", ExcName(j, slink), ".csv", sep = "")
                shap.pwdi = paste(getwd(), "/Univariate/Shapiro_Tests/", shapi, sep = "")
                shap.pwdj = paste(getwd(), "/Univariate/Shapiro_Tests/", shapj, sep = "")
                Si = read.csv(shap.pwdi, header = TRUE)
                Sj = read.csv(shap.pwdj, header = TRUE)
                Si = matrix(Si[, -1], ncol = 1)
                Sj = matrix(Sj[, -1], ncol = 1)
                welchij = paste("WelchTest_", ExcName(i, slink), "vs", ExcName(j, slink), ".csv", 
                  sep = "")
                welch.pwdij = paste(getwd(), "/Univariate/WelchTest/", welchij, sep = "")
                Wij = read.csv(welch.pwdij, header = TRUE)
                Wij = matrix(Wij[, -1], ncol = 1)
                Wij[is.na(Wij)] <- 1
                wmwp = paste("WMWTest_pvalues_", ExcName(i, slink), "vs", ExcName(j, slink), ".csv", 
                  sep = "")
                wmw.pwd = paste(getwd(), "/Univariate/MannWhitneyTests/", wmwp, sep = "")
                WMWp = read.csv(wmw.pwd, header = TRUE)
                WMWp = matrix(WMWp[, -1], ncol = 1)
                WMWp[is.na(WMWp)] <- 1
                
                fin = ncol(sorted) - 1
                pvalues = matrix(rep(1, fin))
                for (q in 1:fin) {
                  if (Si[q, ] > 0.05 & Sj[q, ] > 0.05) {
                    pvalues[q, ] <- Wij[q, ]
                  } else {
                    pvalues[q, ] <- WMWp[q, ]
                  }
                }
                if (fdr) {
                  pval.corr = matrix(p.adjust(pvalues, method = c("BH"), n = nrow(pvalues)), ncol = 1)
                  rownames(pval.corr) = colnames(x.x)
                  colnames(pval.corr) <- c("adjusted.pvalue")
                  pvalfin = paste("Pvalues_", ExcName(i, slink), "vs", ExcName(j, slink), ".csv", 
                    sep = "")
                  assign(pvalfin, pval.corr)
                  write.csv(pval.corr, paste(dirout.pv, pvalfin, sep = ""))
                  sign = c()
                  for (q in 1:fin) {
                    if (pval.corr[q, ] < 0.05) {
                      sign = matrix(c(sign, colnames(sorted.x)[q]))
                    }
                  }
                  #colnames(sign) <- c("adjusted.pvalue")
                  signnam = paste("Significant_Variables_", ExcName(i, slink), "vs", ExcName(j, slink), 
                    ".csv", sep = "")
                  assign(signnam, sign)
                  write.csv(sign, paste(dirout.sign, signnam, sep = ""), row.names = FALSE)
                } else {
                  colnames(pvalues) <- c("pvalue")
                  rownames(pvalues) = colnames(x.x)
                  pvalfin = paste("Pvalues_", ExcName(i, slink), "vs", ExcName(j, slink), ".csv", 
                    sep = "")
                  assign(pvalfin, pvalues)
                  write.csv(pvalues, paste(dirout.pv, pvalfin, sep = ""))
                  sign = c()
                  for (q in 1:fin) {
                    if (pvalues[q, ] < 0.05) {
                      sign = matrix(c(sign, colnames(sorted.x)[q]))
                    }
                  }
                  #colnames(sign) <- c("pvalue")
                  signnam = paste("Significant_Variables_", ExcName(i, slink), "vs", ExcName(j, slink), 
                    ".csv", sep = "")
                  assign(signnam, sign)
                  write.csv(sign, paste(dirout.sign, signnam, sep = ""), row.names = FALSE)
                }
            }
        }
    }
}

explore_data_stat <- function(file, scaling, scal = TRUE, normalize = FALSE, imputation = FALSE, imput) {
    comp = read.csv(file, sep = ",", header = TRUE)
    comp.x = comp[, 3:ncol(comp)]
    comp.x = cbind(comp[, 2], comp[, 1], comp.x)
    x <- comp.x
    x.x <- x[, 3:ncol(x)]
    rownames(x.x) <- x[, 2]
    dirout = paste(getwd(), "/scaleData_", scaling, "/", sep = "")
    dir.create(dirout)
    x.x = cbind(comp[, 2], x.x)
    write.csv(x.x, paste(dirout, "CorrectedTable.csv", sep = ""))
    k = matrix(x[, 1], ncol = 1)
    K = paste(getwd(), "/scaleData_", scaling, "/class.csv", sep = "")
    write.csv(k, K)
    # scaling
    if (scal) {
        if (scaling == "Pareto" | scaling == "pareto" | scaling == "P" | scaling == "p") {
            pwd.n = paste(getwd(), "/scaleData_", scaling, "/CorrectedTable.csv", sep = "")
            x <- read.csv(pwd.n, sep = ",", header = TRUE)
            x.x <- x[, 3:ncol(x)]
            rownames(x.x) <- x[, 1]
            x.areanorm.tc <- scale(x.x, center = TRUE, scale = FALSE)
            # x.areanorm.tc[x.areanorm.tc == 0 ] <- runif(1, 0, 0.0000000001)
            
            all.sd <- matrix(apply(x.areanorm.tc, 2, sd), nrow = 1)
            all.sd[all.sd == 0] <- runif(1, 0, 1e-10)
            uni.exp.all = matrix(rep(1, nrow(x.areanorm.tc)), ncol = 1)
            all.sdm = uni.exp.all %*% all.sd
            all.sqsd = sqrt(all.sdm)
            all.pareto <- x.areanorm.tc/all.sqsd
            write.csv(all.pareto, paste(dirout, "/ProcessedTable.csv", sep = ""))
        } else if (scaling == "Auto" | scaling == "auto" | scaling == "A" | scaling == "a") {
            pwd.n = paste(getwd(), "/scaleData_", scaling, "/CorrectedTable.csv", sep = "")
            x <- read.csv(pwd.n, sep = ",", header = TRUE)
            x.x <- x[, 3:ncol(x)]
            rownames(x.x) <- x[, 1]
            x.areanorm.tc <- scale(x.x, center = TRUE, scale = FALSE)
            all.sd <- matrix(apply(x.areanorm.tc, 2, sd), nrow = 1)
            all.sd[all.sd == 0] <- runif(1, 0, 1e-10)
            uni.exp.all = matrix(rep(1, nrow(x.areanorm.tc)), ncol = 1)
            all.sdm = uni.exp.all %*% all.sd
            all.auto <- x.areanorm.tc/all.sdm
            write.csv(all.auto, paste(dirout, "/ProcessedTable.csv", sep = ""))
        } else if (scaling == "Vast" | scaling == "vast" | scaling == "V" | scaling == "v") {
            pwd.n = paste(getwd(), "/scaleData_", scaling, "/CorrectedTable.csv", sep = "")
            x <- read.csv(pwd.n, sep = ",", header = TRUE)
            x.x <- x[, 3:ncol(x)]
            rownames(x.x) <- x[, 1]
            x.areanorm.tc <- scale(x.x, center = TRUE, scale = FALSE)
            all.sd <- matrix(apply(x.areanorm.tc, 2, sd), nrow = 1)
            all.sd[all.sd == 0] <- runif(1, 0, 1e-10)
            uni.exp.all = matrix(rep(1, nrow(x.areanorm.tc)), ncol = 1)
            all.sdm = uni.exp.all %*% all.sd
            sdm2 = all.sdm^2
            colm = matrix(colMeans(x.x), nrow = 1)
            colm.m = uni.exp.all %*% colm
            num = x.areanorm.tc * colm.m
            vast = num/sdm2
            write.csv(vast, paste(dirout, "/ProcessedTable.csv", sep = ""))
        } else if (scaling == "Range" | scaling == "range" | scaling == "R" | scaling == "r") {
            pwd.n = paste(getwd(), "/scaleData_", scaling, "/CorrectedTable.csv", sep = "")
            x <- read.csv(pwd.n, sep = ",", header = TRUE)
            x.x <- x[, 3:ncol(x)]
            rownames(x.x) <- x[, 1]
            x.areanorm.tc <- scale(x.x, center = TRUE, scale = FALSE)
            range = c()
            for (i in 1:ncol(x.x)) {
                den = c()
                den = max(x.x[, i]) - min(x.x[, i])
                range = matrix(c(range, den), nrow = 1)
            }
            range[range == 0] <- runif(1, 0, 1e-10)
            uni.exp.all = matrix(rep(1, nrow(x.areanorm.tc)), ncol = 1)
            range.m = uni.exp.all %*% range
            all.range = x.areanorm.tc/range.m
            write.csv(all.range, paste(dirout, "/ProcessedTable.csv", sep = ""))
        } else if (scaling == "Median" | scaling == "median" | scaling == "M" | scaling == "m") {
            pwd.n = paste(getwd(), "/scaleData_", scaling, "/CorrectedTable.csv", sep = "")
            x <- read.csv(pwd.n, sep = ",", header = TRUE)
            x.x <- x[, 3:ncol(x)]
            rownames(x.x) <- x[, 1]
            x.areanorm.tc <- scale(x.x, center = TRUE, scale = FALSE)
            all.med <- matrix(apply(x.areanorm.tc, 2, median), nrow = 1)
            uni.exp.all = matrix(rep(1, nrow(x.areanorm.tc)), ncol = 1)
            all.sdm = uni.exp.all %*% all.med
            all.med <- x.areanorm.tc/all.sdm
            write.csv(all.med, paste(dirout, "/ProcessedTable.csv", sep = ""))
        } else if (scaling == "NONE" | scaling == "none" | scaling == "N" | scaling == "n") {
            pwd.n = paste(getwd(), "/scaleData_", scaling, "/CorrectedTable.csv", sep = "")
            x <- read.csv(pwd.n, sep = ",", header = TRUE)
            x.x <- x[, 3:ncol(x)]
            rownames(x.x) <- x[, 1]
            all.med <- x.x
            write.csv(all.med, paste(dirout, "/ProcessedTable.csv", sep = ""))
            
        } else if (scaling == "Center" | scaling == "center" | scaling == "C" | scaling == "c") {
            pwd.n = paste(getwd(), "/scaleData_", scaling, "/CorrectedTable.csv", sep = "")
            x <- read.csv(pwd.n, sep = ",", header = TRUE)
            x.x <- x[, 3:ncol(x)]
            rownames(x.x) <- x[, 1]
            x.c = scale(x.x, scale = FALSE)
            write.csv(x.c, paste(dirout, "/ProcessedTable.csv", sep = ""))
        }
    }
    pwd.scal = paste(getwd(), "/scaleData_", scaling, "/ProcessedTable.csv", sep = "")
    x <- read.csv(pwd.scal, sep = ",", header = TRUE)
    x.x <- x[, 2:ncol(x)]
    rownames(x.x) <- x[, 1]
    pc.all <- prcomp(x.x, center = FALSE, scale = FALSE)
    cat("\n", "PCA Model Summary")
    cat("\n", nrow(comp), "samples x", ncol(comp) - 2, "variables")
    cat("\n", "Variance Explained of PCA Model: ", "\n\n")
    summPca <- summary(pc.all)$importance
    if (dim(summPca)[2] < 5) {
        print(summPca)
    } else {
        print(summPca[, 1:5])
    }
    p.v <- matrix(((pc.all$sdev^2)/(sum(pc.all$sdev^2))), ncol = 1)
    p.i <- round(p.v * 100, 1)
    p.z <- matrix(1, nrow(p.i), 1)
    p.f <- cbind(p.i, p.z)
    dirout.pca = paste(getwd(), "/PCA_Data_", scaling, "/", sep = "")
    dir.create(dirout.pca)
    write.csv(p.f, paste(dirout.pca, "PCA_P", sep = ""))
    write.csv(pc.all$x, paste(dirout.pca, "PCA_ScoreMatrix.csv", sep = ""))
    
    write.csv(t(summPca), paste(dirout.pca, "PCA_explainVariance.csv", sep = ""))
    
    write.csv(pc.all$rotation, paste(dirout.pca, "PCA_LoadingsMatrix.csv", sep = ""))
    pwd.score = paste(getwd(), "/PCA_Data_", scaling, "/", "PCA_ScoreMatrix.csv", sep = "")
    Score <- read.csv(pwd.score, sep = ",", header = TRUE)
    Score.x <- Score[, 2:ncol(Score)]
    rownames(Score.x) <- Score[, 1]
    pwd.load = paste(getwd(), "/PCA_Data_", scaling, "/", "PCA_LoadingsMatrix.csv", sep = "")
    Loading <- read.csv(pwd.load, sep = ",", header = TRUE)
    Loading.x <- Loading[, 2:ncol(Loading)]
    rownames(Loading.x) <- Loading[, 1]
    pwd.pvar = paste(getwd(), "/PCA_Data_", scaling, "/", "PCA_P", sep = "")
    Pvar <- read.csv(pwd.pvar, sep = ",", header = TRUE)
    Pvar.x <- Pvar[, 2:ncol(Pvar)]
    rownames(Pvar.x) <- Pvar[, 1]
    scree = paste(dirout.pca, "Screeplot", scaling, ".pdf", sep = "")
    pdf(scree)
    barplot(Pvar.x[, 1], xlab = "Principal Components", ylab = "Proportion of Variance explained", 
        main = "Screeplot", ylim = c(0, 100))
    dev.off()
    pairs = paste(dirout.pca, "First_3_Components_", scaling, ".pdf", sep = "")
    pdf(pairs)
    tutticolors = matrix(c(1, 2, 3, 4, 5, 6, 7, 8, "rosybrown4", "green4", "navy", "purple2", "orange", 
        "pink", "chocolate2", "coral3", "khaki3", "thistle", "turquoise3", "palegreen1", "moccasin", 
        "olivedrab3", "azure4", "gold3", "deeppink"), ncol = 1)
    col = c()
    for (i in 1:nrow(k)) {
        col = c(col, tutticolors[k[i, ], ])
    }
    pairs = c()
    if (ncol(Score.x) >= 3) {
        pairs = c(3)
    } else {
        pairs = c(ncol(Score.x))
    }
    pairs(Score.x[, 1:pairs], col = col)
    dev.off()
}

#' @name shiftCor
#' @title shiftCor
#' @description  shiftCor provides the QC based signal correction for 
#' large scale metabolomics and targeted proteomics.
#' @param samPeno File path. The file with the meta information including the sample name,
#'  batches, class and order. 
#' @param samFile File path. The file with the expression information. 
#' @param Frule Modified n precent rule function. A variable will be kept if it has a non-zero value
#' for at least n precent of samples in any one group. 
#' (Default: 0.8)  
#' @param MLmethod The machine learning method for QC based signal correction, such as QC based random forest signal correction (QC-RFSC). QC-RLSC was deprecated .
#' @param ntree Number of trees to grow in random forest model.
#' @param imputeM The parameter for imputation method i.e., nearest neighbor 
#' averaging, 'KNN'; minimum values, 'min'; Half of minimum values, 'minHalf'; 
#' median values, 'median'.
#' @param coCV Define the cutoff value (0-100) of CV for controlling the number of features.
#' @param plot Defines if images of feature quality should be generated (TRUE) or not (FALSE). 
#' Defaults to FALSE.
#' @return the shiftCor files. See the details at https://stattarget.github.io
#' @examples 
#' datpath <- system.file('extdata',package = 'statTarget')
#' samPeno <- paste(datpath,'MTBLS79_sampleList.csv', sep='/')
#' samFile <- paste(datpath,'MTBLS79.csv', sep='/')
#' samPeno
#' samFile
#' shiftCor(samPeno,samFile, MLmethod = 'QCRFSC', imputeM = 'KNN',coCV = 30)
#' @keywords Quality Controls,Correction
#' @export 
shiftCor <- function(samPeno, samFile, Frule = 0.8, MLmethod = "QCRFSC", ntree = 500,imputeM = "KNN", coCV = 30, plot = FALSE) {
    cat("\n")
    
    
    
    cat("statTarget: Signal Correction Start... Time:", date(), "\n\n")
    cat("* Step 1: Data File Checking Start..., Time: ", date(), "\n")
    
    cat("\n", "Data Link", "\n")
    cat(" metaFile:", samPeno, "\n")
    cat(" profileFile:", samFile, "\n")
    
    # read the data
    samPeno <- read.csv(samPeno, header = TRUE, check.names = FALSE, stringsAsFactors = FALSE)
    samPeno <- as.data.frame(samPeno)
    # check the title
    checkPeno <- match(c("sample", "batch",  "class",  "order" ),colnames(samPeno))
    if(sum(is.na(checkPeno)) > 0) stop("The names in column of metaFile should be `sample`, `batch`,  `class`,  `order`")
    samPeno <- plyr::arrange(samPeno, order)
    samFile <- read.csv(samFile, header = FALSE, check.names = FALSE, stringsAsFactors = FALSE)
    samFile <- t(samFile)
    colnames(samFile) <- samFile[1, ]
    samFile <- as.data.frame(samFile[-1, ])
    rownames(samFile) <- samFile[,1]

    
    ############## Checking the input file#############
    
    
    ## .............................................
    
    ## Data Matching
    
    ## .............................................
    
    cat("\n", " ", dim(samPeno)[1], " Meta Samples vs ", dim(samFile)[1], " Profile samples", sep = "")
    cat("\n", "The Meta samples list (*NA, missing data from the Profile File)")
    cat("\n\n")
    mcdat <- samFile[, 1][match(samPeno[, 1], samFile[, 1])]
    print(as.vector(mcdat))
    
    ## .............................................
    if (any(is.na(mcdat))) {
        stop("\n", "Missing data from the Profile File! Check your data please!!")
    }
    
    if (dim(samFile)[1] - dim(samPeno)[1] > 0) {
        cat("\n", "Warning: The sample size in Profile File is larger than Meta File! ")
        cat("\n")
    } else if (dim(samFile)[1] - dim(samPeno)[1] < 0) {
        stop("\n", "The sample size in Profile File should be no less than Meta File!", " Check your data please!!")
    }
    
    samFP <- samFile[samPeno$sample, ]
    
    if (sum(is.na(samPeno$class)) <= 0) {
        stop("\n", "There were no QC samples in your data! NA should be defined for QC samples in `class` ")
    }
    
    samPeno_stat <- samPeno
    rownames(samPeno_stat) <- samPeno[, 1]
    qc_seq <- rownames(samPeno_stat)
    qc_seq_tmp <- grep("QC|qc|Qc", qc_seq)
    
    
    
    sam_seq_tmp <- samPeno_stat[-c(qc_seq_tmp), ]
    
    if (sum(is.na(samPeno_stat$batch)) > 0) {
        stop("\n", "There were missing values in batch! Check your data please!\n")
    }
    if (sum(is.na(sam_seq_tmp$class)) > 0) {
        stop("\n", "There were missing values (Unclassified data) in sample class! 
         Check your data please!\n")
    }
    
    
    samPeno_stat$class[is.na(samPeno_stat$class)] <- "QC"
    data_stat = aggregate(samPeno_stat$class, by = list(Category = samPeno_stat$class), FUN = length)
    batch_stat = aggregate(samPeno_stat$class, by = list(Category = samPeno_stat$batch), FUN = length)
    colnames(data_stat) <- c("Class", "No.")
    colnames(batch_stat) <- c("Batch", "No.")
    cat("\n", "Meta-information:", "\n\n")
    print(data_stat)
    print(batch_stat)
    num_sam <- dim(samFile[, 2:ncol(samFile)])
    num_sam <- data.frame(num_sam)
    colnames(num_sam) <- c("no.")
    rownames(num_sam) <- c("QC and samples", "Metabolites")
    cat("\n", "Metabolic profile information:", "\n\n")
    print(num_sam)
    
    
    ################# data NA########
    samFP_temp <- as.matrix(samFP[, 2:ncol(samFP)])
    
    if (length(samFP_temp[samFP_temp < 0L]) > 0) {
        samFP_temp[samFP_temp < 0L] <- 0L
    }
    samFP_temp[samFP_temp == 0L] <- NA
    
    cat("\n")
    cat("* Step 2: Evaluation of Missing Value...", "\n")
    cat("\n", "The number of missing value before QC based signal correction: ", sum(is.na(samFP_temp)))
    imsamFP <- samFP_temp
    ############# Filter miss value###################
    
    classF <- as.factor(samPeno$class)
    classF = addNA(classF)
    imsamFPF = FilterMV(imsamFP, classF, rule = Frule)
    Frule_warning = paste("The number of filtered variables using the modified ", Frule * 100, "% rule :", 
        sep = " ")
    cat("\n", Frule_warning, " ", dim(imsamFP)[2] - dim(imsamFPF)[2], "\n")
    imsamFP = as.matrix(imsamFPF)
    ############## impute missing value#################
    cat("\n")
    cat("* Step 3: Imputation start...", "\n")
    
    
    if (imputeM == "KNN") {
        # require(impute)
        cat("\n", "The imputation method was set at 'KNN'")
        mvd <- impute::impute.knn(imsamFP, rowmax = 0.99, colmax = 0.99, maxp = 15000)
        inputedData <- mvd$data
    } else if (imputeM == "min") {
        cat("\n", "The imputation method was set at 'min'")
        

        inputedData = minValue(imsamFP, classF)
        # inputedData = inputedData[,-1]
        
    } else if (imputeM == "minHalf") {
        cat("\n", "The imputation method was set at 'minHalf'")
        
        inputedData = minHalfValue(imsamFP, classF)
        # inputedData = inputedData[,-1]
        
    } else if (imputeM == "median") {
        

        
        cat("\n", "The imputation method was set at 'median'")
        
        inputedData = medianvalue(imsamFP, classF)
        # inputedData = inputedData[,-1]
    }
    cat("\n", "The number of missing value after imputation: ", sum(is.na(inputedData)))
    
    
    if (sum(is.na(inputedData)) > 0) {
        inputedData = minHalfValue(inputedData, classF)
        cat("\n", "The number of missing value after the second imputation: ", sum(is.na(inputedData)))
    }
    
    
    cat("\n", "Imputation Finished!", "\n", "\n")
    
    cat("* Step 4: QC-based Signal Correction Start... Time: ", date(), "\n")
    
    dat <- as.matrix(t(inputedData))
    numX <- 1:dim(dat)[2]
    
    
    if (MLmethod == "QCRFSC") {
        # require(randomForest)
        cat("\n", "The Signal Correction method was set at QC-RFSC", "\n")
        

        loessDat <- REGfit(x = dat, y = numX, ntree = ntree)
    }
    
    #################################################### 

    ############### dataCheck
    loessDatmp <- apply(loessDat, 2, function(x) as.numeric(as.character(x)))
    loessDatT <- loessDatmp * 1000
    rownames(loessDatT) <- rownames(dat)
    
    datmp <- apply(dat, 2, function(x) as.numeric(as.character(x)))
    rownames(datmp) <- rownames(loessDatT)
    
    
    if (length(loessDatT[loessDatT < 0L]) > 0) {
        loessDatT[loessDatT < 0L] <- 0
    }
    loessDatT[loessDatT == 0L] <- NA
    if (sum(is.na(loessDatT) > 0)) {

        loessDatT <- minHalfValue(t(loessDatT), classF)
        loessDatT <- t(loessDatT)
        cat("\n", "The number of missing value after QC-based signal correction: ", sum(is.na(loessDatT)), 
            "\n")
    }
    dirout.uni = paste(getwd(), "/statTarget/", sep = "")
    dirsc.ID = getwd()
    dir.create(dirout.uni)
    dirout.w = paste(getwd(), "/statTarget/shiftCor", sep = "")
    dir.create(dirout.w)
    dirout.Bs = paste(getwd(), "/statTarget/shiftCor/Before_shiftCor", sep = "")
    dir.create(dirout.Bs)
    dirout.As = paste(getwd(), "/statTarget/shiftCor/After_shiftCor", sep = "")
    dir.create(dirout.As)
    ########### Output images of each peak quality ############
    if(plot){
      
      cat("\n", "High-resolution images output...", "\n")
      pbloplot <- txtProgressBar(min = 1, max = dim(datmp)[1], style = 3)
      for (i in 1:dim(datmp)[1]) {
        loplot(datmp, loessDatT, i)
        setTxtProgressBar(pbloplot, i)
      }
      close(pbloplot)
    }
    
    ############### Raw output###########
    
    raw_temp <- cbind(samPeno, inputedData)
    nam_qc <- rownames(raw_temp)
    
    ##### QC Cal################
    QC_temp_raw <- grep("QC|qc|Qc", nam_qc)
    QC_temp_raw <- raw_temp[c(QC_temp_raw), ]
    raw_temp_qc <- QC_temp_raw[, -c(3, 4)]
    rownames(raw_temp_qc) <- NULL
    RSD30_CV = paste("shift_QC_raw", ".csv", sep = "")
    write.csv(raw_temp_qc, paste(dirout.Bs, RSD30_CV, sep = "/"))
    cat("\n", "Calculation of CV distribution of raw peaks (QC)...\n\n")
    # raw_temp_qc = cbind(seq(1,dim(raw_temp_qc)[1],1),raw_temp_qc)
    Rsdist_QC_raw = RsdCal(raw_temp_qc, batch = TRUE, DistPattern = TRUE, output = FALSE)
    
    ##### Sample Cal################
    sam_temp_raw <- grep("QC|qc|Qc", nam_qc)
    sam_temp_raw <- raw_temp[-c(sam_temp_raw), ]
    raw_temp_sam <- sam_temp_raw[, -c(3, 4)]
    rownames(raw_temp_sam) <- NULL
    RSD30_CV = paste("shift_sam_raw", ".csv", sep = "")
    write.csv(raw_temp_sam, paste(dirout.Bs, RSD30_CV, sep = "/"))
    # Rsdist_sam_raw = RsdCal(raw_temp_sam,DistPattern = FALSE) raw_temp_sam =
    # cbind(seq(1,dim(raw_temp_sam)[1],1),raw_temp_sam)
    Rsdist_sam_raw = RsdCal(raw_temp_sam, batch = FALSE, DistPattern = FALSE, output = FALSE)
    
    
    ############### Loess output###########
    
    lo_temp <- cbind(samPeno, t(loessDatT))
    nam_qc <- rownames(lo_temp)
      
    ############ loess QC Cal#############
    QC_temp <- grep("QC|qc|Qc", nam_qc)
    QC_temp <- lo_temp[c(QC_temp), ]
    lo_temp_qc <- QC_temp[, -c(3, 4)]
    rownames(lo_temp_qc) <- NULL
    cat("\n", "Calculation of CV distribution of corrected peaks (QC)...\n\n")
    # Rsdist_QC_cor=RsdCal(lo_temp_qc,DistPattern = TRUE) lo_temp_qc =
    # cbind(seq(1,dim(lo_temp_qc)[1],1),lo_temp_qc)
    Rsdist_QC_cor = RsdCal(lo_temp_qc, batch = TRUE, DistPattern = TRUE, output = TRUE)
    
    # cutoff CV
    
    cfid <- which(Rsdist_QC_cor > coCV/100) + 2
    
    # filter lo_temp_qc
    if(length(cfid) == 0) {
      lo_temp_qc_filter <- lo_temp_qc
    } else {
    lo_temp_qc_filter <- lo_temp_qc[,-cfid]}
    
    #output
    RSD30_CV = paste("shift_QC_cor", ".csv", sep = "")
    
    lo_temp_qc_filter <- t(lo_temp_qc_filter)
    write.table(lo_temp_qc_filter, paste(dirout.As, RSD30_CV, sep = "/"), sep = ",",col.names = FALSE)
    
    
    ############ loess sam Cal#############
    QC_temp <- grep("QC|qc|Qc", nam_qc)
    QC_temp <- lo_temp[-c(QC_temp), ]
    lo_temp_sam <- QC_temp[, -c(2, 4)]
    rownames(lo_temp_sam) <- NULL
    
    # filter 
    if(length(cfid) == 0) {
      lo_temp_sam_filter <- lo_temp_sam
    } else {
      lo_temp_sam_filter <- lo_temp_sam[,-cfid]}
    #lo_temp_sam_filter <- lo_temp_sam[,-cfid]
    
    RSD30_CV = paste("shift_sample_cor", ".csv", sep = "")
    
    lo_temp_sam_filter <- t(lo_temp_sam_filter)
    write.table(lo_temp_sam_filter, paste(dirout.As, RSD30_CV, sep = "/"), sep = ",",col.names = FALSE)
    
    ############ loess sample Cal############# Rsdist_sam_cor = RsdCal(lo_temp_sam,DistPattern = FALSE)
    ############ lo_temp_sam = cbind(seq(1,dim(lo_temp_sam)[1],1),lo_temp_sam)
    Rsdist_sam_cor = RsdCal(lo_temp_sam, batch = FALSE, DistPattern = FALSE, output = FALSE)
    
    
    
    ################# 
    RSDdist(Rsdist_sam_raw, Rsdist_sam_cor, Rsdist_QC_raw, Rsdist_QC_cor)
    lo_temp$class[is.na(lo_temp$class)] <- "QC"
    lo_temp_all <- lo_temp[, -c(2, 4)]
    rownames(lo_temp_all) <- NULL
    
    # fitler
    if(length(cfid) == 0) {
      lo_temp_all_filter <- lo_temp_all
    } else {
      lo_temp_all_filter <- lo_temp_all[,-cfid]}
    #lo_temp_all_filter <- lo_temp_all[,-cfid]
    
    RSD30_CV = paste("shift_all_cor", ".csv", sep = "")
    
    lo_temp_all_filter <- t(lo_temp_all_filter)
    write.table(lo_temp_all_filter, paste(dirout.As, RSD30_CV, sep = "/"), sep = ",",col.names = FALSE)
    
    cat("\n* Step 5: ", paste("Removal of the features (CV% > ", coCV,"%)",sep=""),date(), "\n")
    
    dn <- length(colnames(lo_temp_qc)[cfid])
    cat("\n", "No. of removed features:", dn)
    cat("\n","Feature name:", colnames(lo_temp_qc)[cfid], "\n")
    
    cat("\n", "Output Link:", getwd(), "\n")
    cat("\n", "Correction Finished! Time: ", date(), "\n")
    cat("\n", "####################################", "\n")
    cat(" # Software Version: statTarget 2.0 + #", "\n")
    cat(" # Email: hemi.luan@gmail.com #", "\n")
    cat(" ####################################", "\n")
    
    ################## Loess Plot########################
    setwd(dirsc.ID)
    
    # parameter output
    scPam1 <- c("Frule", "MLmethod", "ntree", "imputeM","coCV")
    scPam2 <- c(Frule, MLmethod, ntree, imputeM, coCV)
    scpam <- data.frame(scPam1, scPam2)
    colnames(scpam) <- c("parameter", "value")
    par_sh = paste("statTarget/ParameterShiftCor", ".log", sep = "")
    write.table(scpam, paste(getwd(), par_sh, sep = "/"), row.names = FALSE)
    
    tmpfilesc = paste(getwd(), "/tmp", sep = "")
    unlink(tmpfilesc, recursive = TRUE)
}

### file MASS/R/cov.trob.R
cov_trob_sT <- function(x, wt = rep(1, n), cor = FALSE, center = TRUE, nu = 5, maxit = 25, tol = 0.01) {
    test.values <- function(x) {
        if (any(is.na(x)) || any(is.infinite(x))) 
            stop("missing or infinite values in 'x'")
    }
    scale.simp <- function(x, center, n, p) x - rep(center, rep(n, p))
    
    x <- as.matrix(x)
    n <- nrow(x)
    p <- ncol(x)
    dn <- colnames(x)
    test.values(x)
    if (!(miss.wt <- missing(wt))) {
        wt0 <- wt
        test.values(wt)
        if (length(wt) != n) 
            stop("length of 'wt' must equal number of observations")
        if (any(wt < 0)) 
            stop("negative weights not allowed")
        if (!sum(wt)) 
            stop("no positive weights")
        x <- x[wt > 0, , drop = FALSE]
        wt <- wt[wt > 0]
        n <- nrow(x)
    }
    loc <- colSums(wt * x)/sum(wt)
    if (is.numeric(center)) {
        if (length(center) != p) 
            stop("'center' is not the right length")
        loc <- center
    } else if (is.logical(center) && !center) 
        loc <- rep(0, p)
    use.loc <- is.logical(center) && center
    w <- wt * (1 + p/nu)
    endit <- 0
    for (iter in 1L:maxit) {
        w0 <- w
        X <- scale.simp(x, loc, n, p)
        sX <- svd(sqrt(w/sum(w)) * X, nu = 0)
        wX <- X %*% sX$v %*% diag(1/sX$d, , p)
        Q <- drop(wX^2 %*% rep(1, p))
        w <- (wt * (nu + p))/(nu + Q)
        # print(summary(w))
        if (use.loc) 
            loc <- colSums(w * x)/sum(w)
        if (all(abs(w - w0) < tol)) 
            break
        endit <- iter
    }
    if (endit == maxit || abs(mean(w) - mean(wt)) > tol || abs(mean(w * Q)/p - 1) > tol) 
        warning("Probable convergence failure")
    covv <- crossprod(sqrt(w) * X)/sum(wt)
    if (length(dn)) {
        dimnames(covv) <- list(dn, dn)
        names(loc) <- dn
    }
    if (miss.wt) 
        ans <- list(covv = covv, center = loc, n.obs = n) else ans <- list(covv = covv, center = loc, wt = wt0, n.obs = n)
    if (cor) {
        sd <- sqrt(diag(covv))
        cor <- (covv/sd)/rep(sd, rep.int(p, p))
        if (length(dn)) 
            dimnames(cor) <- list(dn, dn)
        ans <- c(ans, list(cor = cor))
    }
    ans$call <- match.call()
    ans$iter <- endit
    ans
}

## FoldChange value is calculated from the raw data (dataSummary file).
volcano <- function(file, upper.lim, lower.lim, sig.lim) {
    pwdfile = paste(getwd(), "/Univariate/DataTable.csv", sep = "")
    file = pwdfile
    x <- read.csv(file, sep = ",", header = TRUE)
    x.x = x[, 3:ncol(x)]
    rownames(x.x) = x[, 2]
    k = matrix(x[, 1], ncol = 1)
    slink = paste(getwd(), "/DataPretreatment", "/slink.csv", sep = "")
    slink = read.csv(slink, header = TRUE)
    x.n = cbind(k, x.x)
    sorted = x.n[order(x.n[, 1]), ]
    sorted.x = as.matrix(sorted[, -1], ncol = ncol(sorted) - 1)
    g = c()
    for (i in 1:nrow(sorted)) {
        if (any(g == sorted[i, 1])) {
            g = g
        } else {
            g = matrix(c(g, sorted[i, 1]), ncol = 1)
        }
    }
    NoF = nrow(g)
    dirout.fc = paste(getwd(), "/Univariate/Fold_Changes/", sep = "")
    dir.create(dirout.fc)
    dirout.vol = paste(getwd(), "/Univariate/Volcano_Plots/", sep = "")
    dir.create(dirout.vol)
    for (i in 1:NoF) {
        for (j in 1:NoF) {
            if (i < j) {
                ni = paste("dataSumm_", ExcName(i, slink), ".csv", sep = "")
                nj = paste("dataSumm_", ExcName(j, slink), ".csv", sep = "")
                # pwdi = paste(getwd(), '/Univariate/Groups/', ni, sep='') pwdj = paste(getwd(),
                # '/Univariate/Groups/', nj, sep='')
                pwdi = paste(getwd(), "/dataSummary/", ni, sep = "")
                pwdj = paste(getwd(), "/dataSummary/", nj, sep = "")
                pv = paste(getwd(), "/Univariate/Pvalues/Pvalues_", ExcName(i, slink), "vs", ExcName(j, 
                  slink), ".csv", sep = "")
                
                
                I = read.csv(pwdi, header = TRUE)
                J = read.csv(pwdj, header = TRUE)
                mI = I[, "Mean"]
                mJ = J[, "Mean"]
                
                FC = matrix(abs(mI/mJ), ncol = 1)
                rownames(FC) = I[, 1]
                colnames(FC) <- "foldChanges"
                fc.csvfile = paste("Fold_Change_", ExcName(i, slink), "-to-", ExcName(j, slink), ".csv", 
                  sep = "")
                write.csv(FC, paste(dirout.fc, fc.csvfile, sep = ""))
                PV = read.csv(pv, header = TRUE)
                if("pvalue" %in% colnames(PV)) adjCheck <- "pvalue" else adjCheck <- "adjust.pvalue"
                PV = matrix(PV[, -1], ncol = 1)
                PV[is.na(PV)] <- 1
                logfc = log2(FC)
                # logfc = glog(FC,2)
                logpv = -log10(PV)
                # logfc = FC
                
                upper <- log2(upper.lim)
                lower <- log2(lower.lim)
                # upper <- upper.lim lower <- lower.lim
                
                
                sig <- -log10(sig.lim)
                colpv = matrix(rep(NA, nrow(PV)), ncol = 1)
                rownames(colpv) <- rownames(logfc)
                for (p in 1:nrow(PV)) {
                  if (logfc[p, ] < lower & logpv[p, ] > sig) {
                    colpv[p, ] = "navy"
                  } else if (logfc[p, ] > upper & logpv[p, ] > sig) {
                    colpv[p, ] = "#D55E00"
                  } else {
                    colpv[p, ] = "dark grey"
                  }
                }
                
                max.fc = 1.3 * max((abs(logfc)))
                V = paste(dirout.vol, "VolcanoPlot_", ExcName(i, slink), "-to-", ExcName(j, slink), 
                  ".pdf", sep = "")
                pospv = matrix(rep(NA, nrow(PV)), ncol = 1)
                for (p in 1:nrow(PV)) {
                  if (logfc[p, ] < lower) {
                    pospv[p, ] = 2L
                  } else {
                    pospv[p, ] = 4L
                  }
                }
                
                pdf(V)
                graphics::plot(logfc, logpv, col = colpv, cex = 1.2, pch = 19, cex.sub = 0.8, xlim = c(-max.fc, 
                  max.fc), xlab = "log2 (Fold Change)", ylab = paste("-log10(",adjCheck,")",sep = ""), main = paste("Volcano Plot ", 
                  ExcName(i, slink), "-to-", ExcName(j, slink), sep = ""), sub = paste("(Blue dot, down-regulated; red dot, up-regulated; ", 
                  "Pvalue < ", sig.lim, ", FC > ", upper.lim, " or < ", lower.lim, ")", sep = ""))
                
                
                voldata <- cbind(logfc, logpv)
                
                if (length(colnames(sorted.x)[grep("navy", colpv)]) > 0 & length(colnames(sorted.x)[grep("#D55E00", 
                  colpv)]) > 0) {
                  text(voldata[grep("navy", colpv), 1], voldata[grep("navy", colpv), 2], labels = rownames(voldata)[grep("navy", 
                    colpv)], cex = 0.6, pos = pospv[grep("navy", colpv)], col = colpv[grep("navy", 
                    colpv)])
                  text(voldata[grep("#D55E00", colpv), 1], voldata[grep("#D55E00", colpv), 2], labels = rownames(voldata)[grep("#D55E00", 
                    colpv)], cex = 0.6, pos = pospv[grep("#D55E00", colpv)], col = colpv[grep("#D55E00", 
                    colpv)])
                } else if (length(colnames(sorted.x)[grep("navy", colpv)]) > 0 & length(colnames(sorted.x)[grep("#D55E00", 
                  colpv)]) == 0) {
                  text(voldata[grep("navy", colpv), 1], voldata[grep("navy", colpv), 2], labels = rownames(voldata)[grep("navy", 
                    colpv)], cex = 0.6, pos = pospv[grep("navy", colpv)], col = colpv[grep("navy", 
                    colpv)])
                } else if (length(colnames(sorted.x)[grep("navy", colpv)]) == 0 & length(colnames(sorted.x)[grep("#D55E00", 
                  colpv)]) > 0) {
                  text(voldata[grep("#D55E00", colpv), 1], voldata[grep("#D55E00", colpv), 2], labels = rownames(voldata)[grep("#D55E00", 
                    colpv)], cex = 0.6, pos = pospv[grep("#D55E00", colpv)], col = colpv[grep("#D55E00", 
                    colpv)])
                }
                
                
                
                if (length(colnames(sorted.x)[grep("navy", colpv)]) > 0 | length(colnames(sorted.x)[grep("#D55E00", 
                  colpv)]) > 0) {
                  
                  Vol_sig = paste(dirout.vol, "VolcanoMarker_", ExcName(i, slink), "-to-", ExcName(j, 
                    slink), ".csv", sep = "")
                  dirout.name = c(rownames(voldata)[grep("navy", colpv)], rownames(voldata)[grep("#D55E00", 
                    colpv)])
                  
                  dirout.name <- as.data.frame(dirout.name)
                  colnames(dirout.name) <- c("Diff_VolcanoMarker")
                  write.csv(dirout.name, Vol_sig, row.names = FALSE)
                } else {
                  NULL
                }
                axis(2, at = c(-1, 150), pos = c(lower, 0), col = "blue", lwd = 1, lty = 2)
                axis(2, at = c(-1, 150), pos = c(upper, 0), col = "blue", lwd = 1, lty = 2)
                axis(1, at = c(-150, 150), pos = c(sig, 0), col = "blue", lwd = 1, lty = 2)
                dev.off()
                
            }
        }
    }
}

#' @importFrom utils read.csv
#' @importFrom utils read.delim
#' @importFrom utils write.csv
#' @importFrom utils write.table
#' @importFrom stats binomial
#' @importFrom utils capture.output
#' @importFrom stats cor
#' @importFrom stats var
## @importFrom stats cov
#' @importFrom stats density
#' @importFrom stats ecdf
#' @importFrom stats glm
#' @importFrom stats median
#' @importFrom stats p.adjust
#' @importFrom stats pf
#' @importFrom stats prcomp
#' @importFrom stats qt
#' @importFrom stats quantile
#' @importFrom stats runif
#' @importFrom stats sd
#' @importFrom stats shapiro.test
#' @importFrom stats t.test
## @importFrom stats var
#' @importFrom stats wilcox.test
#' @importFrom stats loess
#' @importFrom stats predict
#' @importFrom stats update
#' @importFrom stats resid
#' @importFrom graphics axis
#' @importFrom graphics barplot
#' @importFrom graphics boxplot
#' @importFrom graphics grid
#' @importFrom graphics hist
#' @importFrom graphics legend
#' @importFrom graphics par
#' @importFrom graphics plot
#' @importFrom graphics points
#' @importFrom graphics polygon
#' @importFrom graphics segments
#' @importFrom graphics text
#' @importFrom graphics strwidth
#' @importFrom graphics strheight
#' @importFrom graphics box
#' @importFrom graphics identify
#' @importFrom graphics lines
#' @importFrom graphics title
#' @importFrom graphics mtext
#' @importFrom utils install.packages
#' @importFrom randomForest randomForest
#' @importFrom randomForest MDSplot
#' @importFrom randomForest importance
#' @importFrom grDevices rainbow
#' @importFrom grDevices colorRampPalette
#' @importFrom grDevices rgb
#' @importFrom grDevices pdf
#' @importFrom grDevices dev.copy2pdf
#' @importFrom grDevices dev.off
#' @importFrom grDevices dev.new
#' @importFrom grDevices colors
#' @importFrom grDevices palette
#' @importFrom grDevices col2rgb
# @importFrom gWidgets2 svalue
#' @importFrom plyr rlply
#' @importFrom plyr daply
#' @importFrom plyr .
# @importFrom gWidgets2 gwindow @importFrom gWidgets2 ggroup @importFrom gWidgets2 gimage
# @importFrom gWidgets2 gframe @importFrom gWidgets2 glayout @importFrom gWidgets2 gbutton
# @importFrom gWidgets2 svalue @importFrom gWidgets2 gfile @importFrom gWidgets2 gedit @importFrom
# gWidgets2 font @importFrom gWidgets2 gstatusbar
#' @import pls
#' @import pdist 
#' @importFrom ROC rocdemo.sca
#' @importFrom ROC AUC
#' @importFrom rrcov plot
#' @import impute
## @import stats @import gWidgets2RGtk2
#' @importFrom graphics abline
#' @importFrom stats lm
#' @importFrom stats complete.cases
#' @importFrom stats cov.wt
#' @importFrom stats qf
#' @importFrom graphics layout
#' @importFrom stats aggregate
#' @importFrom utils setTxtProgressBar 
#' @importFrom utils txtProgressBar


work_dir <- function(dir.name) {
    WorkinDir = paste(getwd(), "/", dir.name, "/", sep = "")
    dir.create(WorkinDir)
    file = list.files()
    file.copy(file, WorkinDir)
    setwd(WorkinDir)
    unlink("statTarget.R")
}

col_pvalues <- function(file) {
    pwdfile = paste(getwd(), "/Univariate/DataTable.csv", sep = "")
    file = pwdfile
    x <- read.csv(file, sep = ",", header = TRUE)
    x.x = x[, 3:ncol(x)]
    rownames(x.x) = x[, 2]
    k = matrix(x[, 1], ncol = 1)
    slink = paste(getwd(), "/DataPretreatment", "/slink.csv", sep = "")
    slink = read.csv(slink, header = TRUE)
    x.n = cbind(k, x.x)
    sorted = x.n[order(x.n[, 1]), ]
    g = c()
    for (i in 1:nrow(sorted)) {
        if (any(g == sorted[i, 1])) {
            g = g
        } else {
            g = matrix(c(g, sorted[i, 1]), ncol = 1)
        }
    }
    NoF = nrow(g)
    all.pvalues = matrix(rep(1, ncol(sorted) - 1), ncol = 1)
    dirout.col = paste(getwd(), "/Univariate/Pvalues/", sep = "")
    fin = ncol(sorted) - 1
    for (i in 1:NoF) {
        for (j in 1:NoF) {
            if (i < j) {
                ni = paste("Pvalues_", ExcName(i, slink), "vs", ExcName(j, slink), ".csv", sep = "")
                pwdi = paste(getwd(), "/Univariate/Pvalues/", ni, sep = "")
                I = read.csv(pwdi, header = TRUE)
                I = matrix(I[, -1])
                for (q in 1:fin) {
                  if (I[q, ] < 0.05 & all.pvalues[q, ] == 1) {
                    all.pvalues[q, ] = I[q, ]
                  } else {
                    all.pvalues[q, ] = all.pvalues[q, ]
                  }
                }
            }
        }
    }
    colp = matrix(rep(NA, ncol(sorted) - 1), ncol = 1)
    for (i in 1:fin) {
        if (all.pvalues[i, ] < 1) {
            colp[i, ] = "red"
        } else {
            colp[i, ] = "black"
        }
        colnam = "Colors_Pvalues"
        assign(colnam, colp)
        write.csv(colp, paste(dirout.col, colnam, sep = ""))
    }
}

### Missing value imputation Miss value will be replaced with median value.  degree is the group
### factor mv is th data file A imputed matrix missvalue(mv, degree)
missvalue <- function(mv, degree) {
    for (i in 1:dim(mv)[1]) {
        for (j in 2:dim(mv)[2]) {
            if (mv[i, j] == "0") {
                
                mv[i, j] <- tapply(mv[, j], degree, median, na = TRUE)[degree[i]]
                
            }
        }
    }
    return(mv)
}

FilterMV = function(m, degree,rule) {
  dx <- c()
  for (i in 1:ncol(m)) {
    freq <- as.vector(tapply(m[, i], degree, function(x) {
      sum(is.na(x))/length(x)
    }))
    if (sum(freq > 1 - rule) > 0) 
      dx <- c(dx, i)
  }
  if (length(dx) > 0) 
    m <- m[, -dx]
  return(m)
}
minValue <- function(x, group) {
  group = as.factor(as.numeric(group))
  for (i in 1:dim(x)[1]) {
    for (j in 1:dim(x)[2]) {
      if (is.na(x[i, j]) == TRUE) {
        x[i, j] <- tapply(as.numeric(x[, j]), group, min, na.rm = TRUE)[group[i]]
      }
    }
  }
  return(x)
}
minHalfValue <- function(x, group) {
  group = as.factor(as.numeric(group))
  for (i in 1:dim(x)[1]) {
    for (j in 1:dim(x)[2]) {
      if (is.na(x[i, j]) == TRUE) {
        x[i, j] <- tapply(as.numeric(x[, j]), group, min, na.rm = TRUE)[group[i]]/2
      }
    }
  }
  return(x)
}
medianvalue <- function(x, group) {
  group = as.factor(as.numeric(group))
  for (i in 1:dim(x)[1]) {
    for (j in 1:dim(x)[2]) {
      if (is.na(x[i, j]) == TRUE) {
        x[i, j] <- tapply(as.numeric(x[, j]), group, median, na.rm = TRUE)[group[i]]
      }
    }
  }
  return(x)
}


REGfit = function(x, y, ntree = ntree) {
  cn <- colnames(x)
  x <- as.matrix(x)
  #### Check########
  st_QC <- grep("QC|qc|Qc", cn[1])
  ed_QC <- grep("QC|qc|Qc", cn[length(cn)])
  if (length(st_QC) == 0) {
    stop("\nWrong: the first sample must be QC sample; please check ......")
  }
  if (length(ed_QC) == 0) {
    stop("\nWrong: the sample at the end of sequence must be QC sample; 
          please check ......")
  }
  qcid <- grep("QC|qc|Qc", cn)
  pb <- txtProgressBar(min = 1, max = dim(x)[1], style = 3)
  for (i in 1:dim(x)[1]) {
    temp <- randomForest(data.frame(qcid), as.numeric(x[i, qcid]), ntree = ntree)
    y <- data.frame(y)
    colnames(y) <- "qcid"
    rfP <- predict(temp, y)
    x[i, ] <- as.numeric(x[i, ])/rfP
    setTxtProgressBar(pb, i)
  }
  close(pb)
  loessDat = x
  return(loessDat)
}



minHalfValue <- function(x, group) {
  group = as.factor(as.numeric(group))
  for (i in 1:dim(x)[1]) {
    for (j in 1:dim(x)[2]) {
      if (is.na(x[i, j]) == TRUE) {
        x[i, j] <- tapply(as.numeric(x[, j]), group, min, na.rm = TRUE)[group[i]]/2
      }
    }
  }
  return(x)
}

dataEllipse_sT <- function(x, y, groups, group.labels = group.levels, ellipse.label, weights, log = "", 
    levels = c(0.5, 0.95), center.pch = 19, center.cex = 1.5, draw = TRUE, plot.points = draw, add = !plot.points, 
    segments = 51, robust = FALSE, xlab = deparse(substitute(x)), ylab = deparse(substitute(y)), col = if (missing(groups)) palette()[1:2] else palette()[1:length(group.levels)], 
    pch = if (missing(groups)) 1 else seq(group.levels), lwd = 2, fill = FALSE, fill.alpha = 0.3, 
    grid = TRUE, labels, id.method = "mahal", id.n = if (id.method[1] == "identify") Inf else 0, id.cex = 1, 
    id.col = if (missing(groups)) palette()[1] else palette()(1:length(groups)), ...) {
    label.ellipse <- function(ellipse, label, col, ...) {
        # Ellipses (orignally by J. Fox and G. Monette) This sub-function from Michael Friendly position
        # label above top right
        if (cor(ellipse)[1, 2] >= 0) {
            index <- which.max(ellipse[, 2])
            x <- ellipse[index, 1] + 0.5 * strwidth(label)
            y <- ellipse[index, 2] + 0.5 * strheight("A")
            adj <- c(1, 0)
        } else {
            # position label below bot left
            index <- which.min(ellipse[, 2])
            x <- ellipse[index, 1] - 0.5 * strwidth(label)
            y <- ellipse[index, 2] - 0.5 * strheight("A")
            adj <- c(0, 1)
        }
        text(x, y, label, adj = adj, col = col, ...)
    }
    if (missing(y)) {
        if (is.matrix(x) && ncol(x) == 2) {
            if (missing(xlab)) 
                xlab <- colnames(x)[1]
            if (missing(ylab)) 
                ylab <- colnames(x)[2]
            y <- x[, 2]
            x <- x[, 1]
        } else stop("x and y must be vectors, or x must be a 2 column matrix")
    } else if (!(is.vector(x) && is.vector(y) && length(x) == length(y))) 
        stop("x and y must be vectors of the same length")
    if (missing(weights)) 
        weights <- rep(1, length(x))
    if (length(weights) != length(x)) 
        stop("weights must be of the same length as x and y")
    if (!missing(groups)) {
        xlab
        ylab
        if (!is.factor(groups)) 
            stop("groups must be a factor")
        if (!(length(groups) == length(x))) 
            stop("groups, x, and y must all be of the same length")
        if (missing(labels)) 
            labels <- seq(length(x))
        valid <- complete.cases(x, y, groups)
        x <- x[valid]
        y <- y[valid]
        weights <- weights[valid]
        groups <- groups[valid]
        labels <- labels[valid]
        group.levels <- levels(groups)
        col <- col[!is.na(col)]
        if (length(col) < length(group.levels)) 
            stop("too few colors for number of groups")
        result <- vector(length(group.levels), mode = "list")
        names(result) <- group.levels
        if (draw) {
            if (!add) {
                plot(x, y, type = "n", xlab = xlab, ylab = ylab, ...)
                if (grid) {
                  grid(lty = 1, equilogs = FALSE)
                  box()
                }
            }
        }
        for (lev in 1:length(group.levels)) {
            level <- group.levels[lev]
            sel <- groups == level
            result[[lev]] <- dataEllipse_sT(x[sel], y[sel], weights = weights[sel], log = log, levels = levels, 
                center.pch = center.pch, center.cex = center.cex, draw = draw, plot.points = plot.points, 
                add = TRUE, segments = segments, robust = robust, col = rep(col[lev], 2), pch = pch[lev], 
                lwd = lwd, fill = fill, fill.alpha = fill.alpha, labels = labels[sel], id.method = id.method, 
                id.n = id.n, id.cex = id.cex, id.col = col[lev], ellipse.label = group.labels[lev], 
                ...)
        }
        return(invisible(result))
    }
    if (length(col) == 1) 
        col <- rep(col, 2)
    if (draw) {
        if (!add) {
            plot(x, y, type = "n", xlab = xlab, ylab = ylab, ...)
            if (grid) {
                grid(lty = 1, equilogs = FALSE)
                box()
            }
        }
        if (plot.points) 
            points(x, y, col = col[1], pch = pch[1], ...)
    }
    dfn <- 2
    dfd <- length(x) - 1
    if (robust) {
        use <- weights > 0
        v <- cov_trob_sT(cbind(x[use], y[use]), wt = weights[use])
        shape <- v$cov
        center <- v$center
    } else {
        v <- cov.wt(cbind(x, y), wt = weights)
        shape <- v$cov
        center <- v$center
    }
    result <- vector("list", length = length(levels))
    names(result) <- levels
    for (i in seq(along = levels)) {
        level <- levels[i]
        radius <- sqrt(dfn * qf(level, dfn, dfd))
        result[[i]] <- ellipse_sT(center, shape, radius, log = log, center.pch = center.pch, center.cex = center.cex, 
            segments = segments, col = col[2], lwd = lwd, fill = fill, fill.alpha = fill.alpha, draw = draw, 
            ...)
        if (!missing(ellipse.label)) {
            lab <- if (length(ellipse.label) < i) 
                ellipse.label[1] else ellipse.label[i]
            label.ellipse(result[[i]], lab, col[2], ...)
        }
    }
    if (missing(labels)) 
        labels <- seq(length(x))
    if (draw) 
        showLabels_sT(x, y, labels = labels, id.method = id.method, id.n = id.n, id.cex = id.cex, 
            id.col = id.col)
    invisible(if (length(levels) == 1) result[[1]] else result)
}

library(testthat)
library(statTarget)
#library(statTarget)
datpath <- system.file("extdata",package = "statTarget")
samPeno <- paste(datpath,"MTBLS79_sampleList.csv", sep="/")
samFile <- paste(datpath,"MTBLS79.csv", sep="/")
file <- paste(datpath,"data_example_two_groups.csv", sep="/")
shiftCor(samPeno,samFile,Frule = 0.8,imputeM = "KNN")

#test_check("statTarget")

#' @name pvimPlot
#' @title Gini importance and permutation-based variable 
#' importance measures plots
#' @description Create plots for Gini importance and permutation-based variable 
#' Gini importance measures.
#' @param rForest an object of class randomForest that contains the proximity 
#' component from statTarget rForest function.
#' @param pimpModel an object of permutation-based variable Gini importance 
#' measures (PIMP-algorithm) from statTarget rForest function.
#' @param nvarRF The number of variables in importance plot of randomForest.
#' @param border The color to be used for the border of the bars. 
#' Use border = NA to omit borders. see also barplot.
#' @param space The amount of space (as a fraction of the average bar width) 
#' left before each bar. May be given as a single number or one number per bar.
#' see also barplot
#' @param ...  A generic barplot function from graphics package.
#' @usage pvimPlot(rForest,pimpModel,nvarRF = 6,border= NA,
#' space = 0.3,...)
#' @return The output of the name of selected variable importance.
#' @examples 
#' datpath <- system.file('extdata',package = 'statTarget')
#' statFile <- paste(datpath,'data_example_long.csv', sep='/')
#' getFile <- read.csv(statFile,header=TRUE)
#' rFtest <- rForest(getFile,ntree = 10,times = 5)
#' pvimPlot(rFtest$randomForest,rFtest$pimpTest)
#' @author Hemi Luan, hemi.luan@gmail.com
#' @export
pvimPlot <- function(rForest, pimpModel, nvarRF = 6, border = NA, space = 0.3, ...) {
    
    gini <- data.frame(rForest$importance[, "MeanDecreaseGini"])
    colnames(gini) <- "GiniImportance"
    gini = cbind(vehicle = row.names(gini), gini)
    if (nvarRF > dim(gini)[1]) {
        stop(paste("The number of variables should not be less than", " nvarRF value (", nvarRF, ")", 
            sep = ""))
    }
    
    sort.gini <- plyr::arrange(gini, plyr::desc(GiniImportance))
    # permutation
    giniPer <- data.frame(pimpModel$pvalue)
    giniPer = cbind(vehicle = row.names(giniPer), giniPer)
    sort.giniP <- data.frame(plyr::arrange(giniPer, p.value))
    sort.giniP[sort.giniP == 0] <- 0.5 * min(sort.giniP[, 2][sort.giniP[, 2] > 0], na.rm = FALSE)
    logtansform <- data.frame(-log10(sort.giniP$p.value))
    sort.giniP.log <- data.frame(cbind(sort.giniP$vehicle, logtansform))
    colnames(sort.giniP.log) <- c("vehicle", "-log10(p.value)")
    # plot
    
    layout(matrix(c(1, 2, 3), 1), c(3, 3, 1))
    # layout(matrix(c(1,2),ncol=2))
    colfunc <- colorRampPalette(c("#009E73", "#D55E00"))(nvarRF)
    
    # barDat_gini <- plyr::arrange(sort.gini[1:nvarRF,2])
    op <- par(mar = c(4.5, 4, 3, 0.5))
    barGini <- graphics::barplot(rev(sort.gini[1:nvarRF, 2]), horiz = TRUE, beside = TRUE, names.arg = rev(sort.gini[1:nvarRF, 
        1]), col = colfunc, space = space, border = border, ylab = "Variables", las = 2, cex.lab = 0.6, 
        cex.main = 0.6, cex.axis = 0.6, cex.names = 0.5, xlab = "Gini Importance", ...)
    text(as.matrix(rev(sort.gini[1:nvarRF, 2])), barGini, labels = round(rev(sort.gini[1:nvarRF, 2]), 
        2), xpd = TRUE, cex = 0.3, col = "grey48", pos = 4)
    par()
    dat <- as.matrix(rev(sort.giniP.log[1:nvarRF, 2]))
    par(op)
    barPvalue <- graphics::barplot(dat, horiz = TRUE, beside = TRUE, space = space, border = border, 
        names.arg = rev(sort.giniP.log[1:nvarRF, 1]), col = colfunc, ylab = "Variables (-log10(p.value))", 
        las = 2, cex.lab = 0.6, cex.main = 0.6, cex.axis = 0.6, cex.names = 0.5, xlab = "Pvalue-based Importance", 
        ...)
    text(dat, barPvalue, labels = round(rev(sort.giniP.log[1:nvarRF, 2]), 2), xpd = TRUE, cex = 0.3, 
        col = "grey48", pos = 4)
    par()
    
    par(mar = c(25, 2, 60, 12), cex = 0.3)
    # par(op1)
    color.bar(colorRampPalette(c("#009E73", "#D55E00"))(6), min = min(sort.giniP.log[1:nvarRF, 2]), 
        max = max(sort.giniP.log[1:nvarRF, 2]))
    return(list(giniImp = sort.gini[1:nvarRF, 1], pvalueImp = sort.giniP.log[1:nvarRF, 1]))
}

pimpRF <- function(X, y, rForest, times = 100, seed = 123, gDist = FALSE) {
    
    ################################################################## randomForest?
    if (!inherits(rForest, "randomForest")) 
        stop("rForest is not of class randomForest")
    mtry = rForest$mtry
    ntree = rForest$ntree
    n = nrow(X)
    p = ncol(X)
    
    set.seed(seed)
    classRF = is.factor(y)
    if (classRF) {
        if (rForest$type == "regression") 
            stop("rForest$type = regression !! y a factor ")
        apply_pb <- function(X, MARGIN, FUN, ...) {
            env <- environment()
            pb_Total <- sum(dim(X)[MARGIN])
            counter <- 0
            pb <- txtProgressBar(min = 0, max = pb_Total, style = 3)
            
            wrapper <- function(...) {
                curVal <- get("counter", envir = env)
                assign("counter", curVal + 1, envir = env)
                setTxtProgressBar(get("pb", envir = env), curVal + 1)
                FUN(...)
            }
            res <- apply(X, MARGIN, wrapper, ...)
            close(pb)
            res
        }
        
        y.s = replicate(times, sample(as.integer(y)))
        # Number of permutation
        varip <- apply_pb(y.s, 2, function(y) {
            randomForest::randomForest(X, as.factor(y), mtry = mtry, importance = TRUE, ntree = ntree)[[9]][, 
                "MeanDecreaseGini"]
        })
    } else {
        if (rForest$type == "classification") 
            stop("rForest$type = classification !! y not a factor ")
        y.s = replicate(times, sample(y))
        varip <- apply_pb(y.s, 2, function(y) {
            randomForest::randomForest(X, as.factor(y), mtry = mtry, importance = TRUE, ntree = ntree)[[7]][, 
                "IncNodePurity"]
        })
    }
    # 
    dimNames = dimnames(rForest$importance)[[1]]
    Pimp = list(VarImp = matrix(rForest$importance[, "MeanDecreaseGini"], ncol = 1, dimnames = list(dimNames, 
        "VarImp")), PerVarImp = varip, type = if (classRF) {
        "classification"
    } else {
        "regression"
    }, call = match.call())
    p = nrow(Pimp$PerVarImp)
    if (gDist) {
        mean.PerVarImp = apply(Pimp$PerVarImp, 1, mean)
        sd.PerVarImp = apply(Pimp$PerVarImp, 1, sd)
        j = 1:p
        test.norm = sapply(j, function(j) {
            stats::ks.test(unique(Pimp$PerVarImp[j, ]), "pnorm", mean = mean.PerVarImp[j], sd = sd.PerVarImp[j])$p.value
        })
        
        # the p-value is the probability of observing the original VarImp or one more extreme,
        p.val = sapply(j, function(j) {
            stats::pnorm(Pimp$VarImp[j], mean.PerVarImp[j], sd.PerVarImp[j], lower.tail = FALSE)
        })
        
    } else {
        j = 1:p
        Fn0 = sapply(j, function(j) {
            stats::ecdf(Pimp$PerVarImp[j, ])
        })
        p.val = sapply(j, function(j) {
            1 - Fn0[[j]](Pimp$VarImp[j])
        })
    }
    dimNames = dimnames(Pimp$VarImp)[[1]]
    outTest = list(VarImp = Pimp$VarImp, PerVarImp = Pimp$PerVarImp, gDist = gDist, meanPerVarImp = if (gDist) {
        matrix(mean.PerVarImp, ncol = 1, dimnames = list(dimNames, "mean(PerVarImp)"))
    } else {
        NULL
    }, sdPerVarImp = if (gDist) {
        matrix(sd.PerVarImp, ncol = 1, dimnames = list(dimNames, "sd(PerVarImp)"))
    } else {
        NULL
    }, p.ks.test = if (gDist) {
        matrix(test.norm, ncol = 1, dimnames = list(dimNames, "ks.test"))
    } else {
        NULL
    }, pvalue = matrix(p.val, ncol = 1, dimnames = list(dimNames, "p-value")), type = Pimp$type, call.PIMP = Pimp$call, 
        call = match.call())
    return(outTest)
}

# Function to plot color bar
color.bar <- function(lut, min, max = -min, nticks = 10, ticks = seq(min, max, len = nticks), title = "") {
    scale = (length(lut) - 1)/(max - min)
    plot(c(0, 10), c(min, max), type = "n", bty = "n", xaxt = "n", xlab = "", yaxt = "n", ylab = "", 
        main = title)
    axis(4, round(ticks, 1), las = 1)
    for (i in 1:(length(lut) - 1)) {
        y = (i - 1)/scale + min
        graphics::rect(0, y, 10, y + 1/scale, col = lut[i], border = NA, angle = 0)
    }
}

showLabels_sT <- function(x, y, labels = NULL, id.method = "identify", id.n = length(x), id.cex = 1, 
    id.col = palette()[1], ...) {
    res <- NULL
    id.method <- if (is.list(id.method)) 
        id.method else list(id.method)
    for (meth in id.method) res <- c(res, showLabels1(x, y, labels, meth, id.n, id.cex, id.col, ...))
    return(if (is.null(res)) invisible(res) else res)
}

showLabels1 <- function(x, y, labels = NULL, id.method = "identify", id.n = length(x), id.cex = 1, 
    id.col = palette()[1], all = NULL, ...) {
    # If labels are NULL, try to get the labels from x:
    if (is.null(labels)) 
        labels <- names(x)
    if (is.null(labels)) 
        labels <- paste(seq_along(x))
    if (is.null(id.col)) 
        id.col <- palette()[1]
    # logged-axes?
    log.x <- par("xlog")
    log.y <- par("ylog")
    # id.method can be any of the following: --- a list of row numbers --- a list of labels --- a
    # vector of n numbers --- a text string: 'identify', 'x', 'y', 'mahal', 'r'
    idmeth <- pmatch(id.method[1], c("x", "y", "mahal", "identify", "r"))
    if (!is.na(idmeth)) 
        idmeth <- c("x", "y", "mahal", "identify", "r")[idmeth]
    # if idmeth is NA, then it must be <= n numbers or row names
    id.var <- NULL
    if (is.na(idmeth)) {
        if (is.null(all)) 
            all <- list(labels = labels, subs = rep(TRUE, length(labels)))
        names(all$labels) <- all$labels
        if (length(id.method) >= length(x)) {
            id.var <- id.method[which(all$subs)]
            id.n <- min(id.n, length(id.var))
        } else {
            id.var <- rep(0, length(x))
            names(id.var) <- labels
            inSubset <- all$labels[all$subs] %in% all$labels[id.method]
            id.var[inSubset] <- 1
            id.n <- sum(inSubset)
        }
    } else {
        # use identify?
        if (idmeth == "identify") {
            result <- labels[identify(x, y, labels, n = length(x), cex = id.cex, col = id.col, ...)]
            if (length(result) > 0) 
                return(unique(result)) else return(NULL)
        }
        # missing values need to be removed
        ismissing <- is.na(x) | is.na(y) | is.na(labels)
        if (any(ismissing)) {
            x <- x[!ismissing]
            y <- y[!ismissing]
            labels <- labels[!ismissing]
        }
        # other methods:
        id.var <- switch(id.method, x = if (log.x == TRUE) suppressWarnings(if (all(x) > 0) abs(log(x) - 
            mean(log(x))) else return(invisible(NULL))) else abs(x - mean(x)), y = if (log.y == TRUE) suppressWarnings(if (all(y) > 
            0) abs(log(y) - mean(log(y))) else return(invisible(NULL))) else abs(y - mean(y)), r = if (log.y == 
            TRUE) suppressWarnings(if (all(y) > 0) abs(log(y)) else return(invisible(NULL))) else abs(y), 
            mahal = if (log.x == TRUE & log.y == TRUE) {
                suppressWarnings(if (all(x) > 0 & all(y) > 0) rowSums(qr.Q(qr(cbind(1, log(x), log(y))))^2) else return(invisible(NULL)))
            } else {
                if (log.x == TRUE) {
                  suppressWarnings(if (all(x) > 0) rowSums(qr.Q(qr(cbind(1, log(x), y)))^2) else return(invisible(NULL)))
                } else {
                  if (log.y == TRUE) {
                    suppressWarnings(if (all(y) > 0) rowSums(qr.Q(qr(cbind(1, x, log(y))))^2) else return(invisible(NULL)))
                  } else {
                    rowSums(qr.Q(qr(cbind(1, x, y)))^2)
                  }
                }
            })
    }
    # require id.n positive
    if (id.n <= 0L) 
        return(invisible(NULL))
    # criterion
    ind <- order(id.var, decreasing = TRUE)[1L:min(length(id.var), id.n)]
    # position
    mid <- mean(if (par("xlog") == TRUE) 
        10^(par("usr")[1:2]) else par("usr")[1:2])
    labpos <- c(4, 2)[1 + as.numeric(x > mid)]
    # print
    for (i in ind) {
        text(x[i], y[i], labels[i], cex = id.cex, xpd = TRUE, col = id.col, pos = labpos[i], offset = 0.25, 
            ...)
    }
    names(ind) <- labels[ind]
    result <- ind
    if (length(result) == 0) 
        return(NULL) else return(result)
}

Plot_pca_score_stat <- function(pcx, pcy, scaling, Labels) {
    score = paste(getwd(), "/PCA_Data_", scaling, "/PCA_ScoreMatrix.csv", sep = "")
    ppppp = paste(getwd(), "/PCA_Data_", scaling, "/PCA_P", sep = "")
    Score <- read.csv(score, sep = ",", header = TRUE)
    Score.x <- Score[, 2:ncol(Score)]
    rownames(Score.x) <- Score[, 1]
    pwdK = paste(getwd(), "/scaleData_", scaling, "/class.csv", sep = "")
    k = read.csv(pwdK)
    slink = paste(getwd(), "/DataPretreatment", "/slink.csv", sep = "")
    slink = read.csv(slink, header = TRUE)
    Pvar <- read.csv(ppppp, sep = ",", header = TRUE)
    Pvar.x <- Pvar[, 2:ncol(Pvar)]
    rownames(Pvar.x) <- Pvar[, 1]
    pca <- paste("PC", pcx, " (", Pvar[pcx, 2], ") %")
    pcb <- paste("PC", pcy, " (", Pvar[pcy, 2], ")%")
    cum = Pvar[pcx, 2] + Pvar[pcy, 2]
    xlab = c(pca)
    ylab = c(pcb)
    lim = c()
    max.pc1 = 1.3 * (max(abs(Score.x[, pcx])))
    max.pc2 = 1.3 * (max(abs(Score.x[, pcy])))
    if (max.pc1 > max.pc2) {
        lim = c(-max.pc1, max.pc1)
    } else {
        lim = c(-max.pc2, max.pc2)
    }
    
    
    Epaxis <- dataEllipse_sT(Score.x[, pcx], Score.x[, pcy], levels = c(0.95), add = FALSE, draw = FALSE, 
        col = "black", lwd = 0.4, plot.points = FALSE, center.cex = 0.2)
    
    if (1.1 * max(Epaxis[, "x"]) < max(lim) & 1.1 * max(Epaxis[, "y"]) < max(lim)) {
        lim = lim
    } else {
        lim = c(1.2 * min(as.numeric(Epaxis)), 1.2 * max(as.numeric(Epaxis)))
    }
    
    
    
    tutticolors = matrix(c(1, 2, 3, 4, 5, 6, 7, 8, "rosybrown4", "green4", "navy", "purple2", "orange", 
        "pink", "chocolate2", "coral3", "khaki3", "thistle", "turquoise3", "palegreen1", "moccasin", 
        "olivedrab3", "azure4", "gold3", "deeppink"), ncol = 1)
    col = c()
    for (i in 1:nrow(k)) {
        col = c(col, tutticolors[k[i, 2], ])
    }
    D <- paste(getwd(), "/PCA_Data_", scaling, "/ScorePlot_PC", pcx, "vsPC", pcy, ".pdf", sep = "")
    pdf(D)
    # dev.new()
    graphics::plot(Score.x[, pcx], Score.x[, pcy], col = col, xlab = xlab, ylab = ylab, xlim = lim, 
        ylim = lim, pch = 19, sub = paste("Cumulative Proportion of Variance Explained = ", cum, "%", 
            sep = ""), main = paste("PCA Score Plot (", scaling, ")", sep = ""))
    axis(1, at = lim * 2, pos = c(0, 0), labels = FALSE, col = "grey", lwd = 0.7)
    axis(2, at = lim * 2, pos = c(0, 0), labels = FALSE, col = "grey", lwd = 0.7)
    # requireNamespace(car) library(car)
    dataEllipse_sT(Score.x[, pcx], Score.x[, pcy], levels = c(0.95), add = TRUE, col = "grey48", lwd = 0.4, 
        plot.points = FALSE, center.cex = 0.2)
    
    if (Labels) {
        text(Score.x[, pcx], Score.x[, pcy], col = col, cex = 0.5, labels = rownames(Score.x), pos = 1)
    } else {
        legend("topright", legend = levels(factor(slink[, 1])), bty = "n", pch = 19, col = seq_along(levels(factor(slink[, 
            1]))), text.col = seq_along(levels(factor(slink[, 1]))))
    }
    # dev.copy2pdf(file=D)
    dev.off()
}

#' @name predict_RF
#' @title preidict function for random forest objects in statTarget
#' @description  Prediction of test data using random forest in statTarget.
#' @param object An object created by the function statTarget_rForest.
#' @param newdata A data frame or matrix containing new data. (Note: If not 
#' given, the out-of-bag prediction in object is returned. see randomForest 
#' package.
#' @param type One of response, prob. or votes, indicating the type of output: 
#' predicted values, matrix of class probabilities, or matrix of vote counts. 
#' class is allowed, but automatically converted to 'response', for backward 
#' compatibility.
#' @param ... A generic predict function from randomForest package.
#' @return A class of predicted values is returned. 
#' Object type is classification, for detail see randomForest package.
#' @usage predict_RF(object, newdata, type='response',...)
#' @author Hemi Luan, hemi.luan@gmail.com
#' @seealso randomForest
#' @export 
predict_RF <- function(object, newdata, type = "response", ...) {
    # require(randomForest) need object from statTarget_rForest
    if (!inherits(object$randomForest, "randomForest")) 
        stop("Object should be from statTarget_rForest")
    newModel <- object$randomForest
    output <- predict(newModel, newdata, type = "response", ...)
    return(output)
}

### Shapiro test This function provide the Shapiro Wilk test is a test of normality of each pairwise
### comparison of sample groups.  file The connection to the data file.  A result with Shapiro Wilk
### test
shapiro <- function(file) {
    pwdfile = paste(getwd(), "/Univariate/DataTable.csv", sep = "")
    file = pwdfile
    x <- read.csv(file, sep = ",", header = TRUE)
    x.x = x[, 3:ncol(x)]
    rownames(x.x) = x[, 2]
    k = matrix(x[, 1], ncol = 1)
    slink = paste(getwd(), "/DataPretreatment", "/slink.csv", sep = "")
    slink = read.csv(slink, header = TRUE)
    x.n = cbind(k, x.x)
    sorted = x.n[order(x.n[, 1]), ]
    g = c()
    for (i in 1:nrow(sorted)) {
        if (any(g == sorted[i, 1])) {
            g = g
        } else {
            g = matrix(c(g, sorted[i, 1]), ncol = 1)
        }
    }
    dirout.r = paste(getwd(), "/Univariate/Groups", sep = "")
    dir.create(dirout.r)
    dirout.s = paste(getwd(), "/Univariate/Shapiro_Tests", sep = "")
    dir.create(dirout.s)
    for (i in 1:nrow(g)) {
        vuota <- c()
        fin = matrix(rep(NA, ncol(sorted)), nrow = 1)
        for (j in 1:nrow(sorted)) {
            if (sorted[j, 1] == i) {
                vuota <- matrix(sorted[j, ], nrow = 1)
                rownames(vuota) = rownames(sorted)[j]
                fin = rbind(fin, vuota)
            }
        }
        nam = paste("r", ExcName(i, slink), sep = ".")
        n = matrix(fin[-1, ], ncol = ncol(sorted))
        n.x = matrix(n[, -1], ncol = ncol(sorted) - 1)
        lastcol = ncol(sorted)
        n.x = n.x[, -lastcol]
        colnames(n.x) = colnames(x.x)
        name = as.matrix(assign(nam, n.x))
        shapname = paste("shapiro", ExcName(i, slink), sep = ".")
        shapiro = matrix(rep(NA, ncol(n.x)))
        for (q in 1:ncol(name)) {
            notAlist = c()
            notAlist = as.numeric(matrix(unlist(name[, q])))
            if (diff(range(notAlist)) == 0) {
                shapiro[q, ] = NA
            } else {
                shapiro[q, ] = shapiro.test(as.numeric(notAlist))$p.value
            }
            assign(shapname, shapiro)
        }
        outputfile = paste("r.", ExcName(i, slink), ".csv", sep = "")
        write.csv(name, paste(dirout.r, outputfile, sep = "/"))
        outshapiro = paste("ShapiroTest.", ExcName(i, slink), ".csv", sep = "")
        shapiro[is.na(shapiro)] = 1
        write.csv(shapiro, paste(dirout.s, outshapiro, sep = "/"))
    }
}

ComBat_qcFree <- function (dat, batch, mod = NULL, par.prior = TRUE, prior.plots = FALSE,
                    mean.only = FALSE, ref.batch = NULL) {
  ## make batch a factor and make a set of indicators for batch
  if(mean.only==TRUE){
    message("Using the 'mean only' version of ComBat")
  }
  if(length(dim(batch))>1){
    stop("This version of ComBat only allows one batch variable")
  }  ## to be updated soon!
  batch <- as.factor(batch)
  batchmod <- model.matrix(~-1+batch)  
  if (!is.null(ref.batch)){
    ## check for reference batch, check value, and make appropriate changes
    if (!(ref.batch%in%levels(batch))) {
      stop("reference level ref.batch is not one of the levels of the batch variable")
    }
    cat("Using batch =",ref.batch, "as a reference batch (this batch won't change)\n")
    ref <- which(levels(as.factor(batch))==ref.batch) # find the reference
    batchmod[,ref] <- 1
  } else {
    ref <- NULL
  }
  message("Found", nlevels(batch), "batches")
  
  ## A few other characteristics on the batches
  n.batch <- nlevels(batch)
  batches <- list()
  for (i in 1:n.batch) {
    batches[[i]] <- which(batch == levels(batch)[i])
  } # list of samples in each batch  
  n.batches <- sapply(batches, length)
  if(any(n.batches==1)){
    mean.only=TRUE
    message("Note: one batch has only one sample, setting mean.only=TRUE")
  }
  n.array <- sum(n.batches)
  ## combine batch variable and covariates
  design <- cbind(batchmod,mod)
  
  ## check for intercept in covariates, and drop if present
  check <- apply(design, 2, function(x) all(x == 1))
  if(!is.null(ref)){
    check[ref] <- FALSE
  } ## except don't throw away the reference batch indicator
  design <- as.matrix(design[,!check])
  
  ## Number of covariates or covariate levels
  message("Adjusting for ", ncol(design)-ncol(batchmod), ' covariate(s) or covariate level(s)')
  
  ## Check if the design is confounded
  if(qr(design)$rank < ncol(design)) {
    ## if(ncol(design)<=(n.batch)){stop("Batch variables are redundant! Remove one or more of the batch variables so they are no longer confounded")}
    if(ncol(design)==(n.batch+1)) {
      stop("The covariate is confounded with batch! Remove the covariate and rerun ComBat")
    }
    if(ncol(design)>(n.batch+1)) {
      if((qr(design[,-c(1:n.batch)])$rank<ncol(design[,-c(1:n.batch)]))){
        stop('The covariates are confounded! Please remove one or more of the covariates so the design is not confounded')
      } else {
        stop("At least one covariate is confounded with batch! Please remove confounded covariates and rerun ComBat")
      }
    }
  }
  
  ## Check for missing values
  NAs <- any(is.na(dat))
  if(NAs){
    message(c('Found',sum(is.na(dat)),'Missing Data Values'), sep=' ')}
  ## print(dat[1:2,])
  
  ##Standardize Data across genes
  cat('Standardizing Data across features\n')
  if (!NAs){
    B.hat <- solve(crossprod(design), tcrossprod(t(design), as.matrix(dat)))
  } else { 
    B.hat <- apply(dat, 1, Beta.NA, design) # FIXME
  }
  
  ## change grand.mean for ref batch
  if(!is.null(ref.batch)){
    grand.mean <- t(B.hat[ref, ])
  } else {
    grand.mean <- crossprod(n.batches/n.array, B.hat[1:n.batch,])
  }
  
  ## change var.pooled for ref batch
  if (!NAs){
    if(!is.null(ref.batch)) {
      ref.dat <- dat[, batches[[ref]]]
      var.pooled <- ((ref.dat-t(design[batches[[ref]], ] %*% B.hat))^2) %*% rep(1/n.batches[ref],n.batches[ref]) # FIXME
    } else {
      var.pooled <- ((dat-t(design %*% B.hat))^2) %*% rep(1/n.array,n.array) # FIXME
    }
  } else {
    if(!is.null(ref.batch)) {
      ref.dat <- dat[, batches[[ref]]]
      var.pooled <- rowVars(ref.dat-t(design[batches[[ref]], ]%*%B.hat), na.rm=TRUE)
    } else {
      var.pooled <- rowVars(dat-t(design %*% B.hat), na.rm=TRUE)
    }
  }
  
  stand.mean <- t(grand.mean) %*% t(rep(1,n.array)) # FIXME
  if(!is.null(design)){
    tmp <- design
    tmp[,c(1:n.batch)] <- 0
    stand.mean <- stand.mean+t(tmp %*% B.hat) #FIXME
  }  
  s.data <- (dat-stand.mean)/(sqrt(var.pooled) %*% t(rep(1,n.array))) # FIXME
  
  ##Get regression batch effect parameters
  message("Fitting L/S model and finding priors")
  batch.design <- design[, 1:n.batch]
  if (!NAs){
    gamma.hat <- solve(crossprod(batch.design), tcrossprod(t(batch.design),
                                                           as.matrix(s.data)))
  } else{
    gamma.hat <- apply(s.data, 1, Beta.NA, batch.design) # FIXME
  }
  delta.hat <- NULL
  for (i in batches){
    if(mean.only==TRUE) {
      delta.hat <- rbind(delta.hat,rep(1,nrow(s.data))) 
    } else {
      delta.hat <- rbind(delta.hat, rowVars(s.data[,i], na.rm=TRUE))
    }
  }
  
  ##Find Priors
  gamma.bar <- rowMeans(gamma.hat)
  t2 <- rowVars(gamma.hat)
  a.prior <- apply(delta.hat, 1, aprior) # FIXME 
  b.prior <- apply(delta.hat, 1, bprior) # FIXME
  
  ## Plot empirical and parametric priors
  
  if (prior.plots && par.prior) {
    par(mfrow=c(1,2))
    
    ## Top left
    tmp <- density(gamma.hat[1,])
    plot(tmp,  type='l', main=expression(paste("Density Plot of First Batch ",  hat(gamma))))
    xx <- seq(min(tmp$x), max(tmp$x), length=100)
    lines(xx,dnorm(xx,gamma.bar[1],sqrt(t2[1])), col=2)
    
    ## Top Right
    qqnorm(gamma.hat[1,], main=expression(paste("Normal Q-Q Plot of First Batch ", hat(gamma))))
    qqline(gamma.hat[1,], col=2)
    
    ## Bottom Left
    #tmp <- density(delta.hat[1,])
    #xx <- seq(min(tmp$x), max(tmp$x), length=100)
    #tmp1 <- list(x=xx, y=dinvgamma(xx, a.prior[1], b.prior[1]))
    #plot(tmp, typ="l", ylim=c(0, max(tmp$y, tmp1$y)),
    #     main=expression(paste("Density Plot of First Batch ", hat(delta))))
    #lines(tmp1, col=2)
    
    ## Bottom Right
    #invgam <- 1/qgamma(1-ppoints(ncol(delta.hat)), a.prior[1], b.prior[1])
    #qqplot(invgam, delta.hat[1,],
    #       main=expression(paste("Inverse Gamma Q-Q Plot of First Batch ", hat(delta))),
    #       ylab="Sample Quantiles", xlab="Theoretical Quantiles")
    #lines(c(0, max(invgam)), c(0, max(invgam)), col=2)
  }
  
  ## Find EB batch adjustments
  
  gamma.star <- delta.star <- matrix(NA, nrow=n.batch, ncol=nrow(s.data))
  if (par.prior) {
    message("Finding parametric adjustments")
    results <- lapply(1:n.batch, function(i) {
      if (mean.only) {
        gamma.star <- postmean(gamma.hat[i,], gamma.bar[i], 1, 1, t2[i])
        delta.star <- rep(1, nrow(s.data))
      }
      else {
        temp <- it.sol(s.data[, batches[[i]]], gamma.hat[i, ],
                       delta.hat[i, ], gamma.bar[i], t2[i], a.prior[i],
                       b.prior[i])
        gamma.star <- temp[1, ]
        delta.star <- temp[2, ]
      }
      list(gamma.star=gamma.star, delta.star=delta.star)
    })
    for (i in 1:n.batch) {
      gamma.star[i,] <- results[[i]]$gamma.star
      delta.star[i,] <- results[[i]]$delta.star
    }
  }
  else {
    message("Finding nonparametric adjustments")
    results <- lapply(1:n.batch, function(i) {
      if (mean.only) {
        delta.hat[i, ] = 1
      }
      temp <- int.eprior(as.matrix(s.data[, batches[[i]]]),
                         gamma.hat[i, ], delta.hat[i, ])
      list(gamma.star=temp[1,], delta.star=temp[2,])
    })
    for (i in 1:n.batch) {
      gamma.star[i,] <- results[[i]]$gamma.star
      delta.star[i,] <- results[[i]]$delta.star
    }
  }
  
  if(!is.null(ref.batch)){
    gamma.star[ref,] <- 0  ## set reference batch mean equal to 0
    delta.star[ref,] <- 1  ## set reference batch variance equal to 1
  }
  
  ## Normalize the Data ###
  message("Adjusting the Data\n")
  
  bayesdata <- s.data
  j <- 1
  for (i in batches){
    bayesdata[,i] <- (bayesdata[,i]-t(batch.design[i,]%*%gamma.star))/(sqrt(delta.star[j,])%*%t(rep(1,n.batches[j]))) # FIXME
    j <- j+1
  }
  
  bayesdata <- (bayesdata*(sqrt(var.pooled)%*%t(rep(1,n.array))))+stand.mean # FIXME
  
  ## tiny change still exist when tested on bladder data
  ## total sum of change within each batch around 1e-15 
  ## (could be computational system error).  
  ## Do not change ref batch at all in reference version
  if(!is.null(ref.batch)){
    bayesdata[, batches[[ref]]] <- dat[, batches[[ref]]]
  }
  
  return(bayesdata)
}

### Wilcoxon-Mann Whitney tests This function provide two.sided Wilcoxon Mann Whitney tests for each
### pairwise comparison of sample groups.  file The connection to the data file.  wmw(file) A matrix
### with p values
wmw <- function(file) {
    pwdfile = paste(getwd(), "/Univariate/DataTable.csv", sep = "")
    file = pwdfile
    x <- read.csv(file, sep = ",", header = TRUE)
    x.x = x[, 3:ncol(x)]
    rownames(x.x) = x[, 2]
    k = matrix(x[, 1], ncol = 1)
    slink = paste(getwd(), "/DataPretreatment", "/slink.csv", sep = "")
    slink = read.csv(slink, header = TRUE)
    x.n = cbind(k, x.x)
    sorted = x.n[order(x.n[, 1]), ]
    g = c()
    for (i in 1:nrow(sorted)) {
        if (any(g == sorted[i, 1])) {
            g = g
        } else {
            g = matrix(c(g, sorted[i, 1]), ncol = 1)
        }
    }
    NoF = nrow(g)
    dirout.wm = paste(getwd(), "/Univariate/MannWhitneyTests/", sep = "")
    dir.create(dirout.wm)
    for (i in 1:NoF) {
        for (j in 1:NoF) {
            if (i < j) {
                ni = paste("r.", ExcName(i, slink), ".csv", sep = "")
                nj = paste("r.", ExcName(j, slink), ".csv", sep = "")
                pwdi = paste(getwd(), "/Univariate/Groups/", ni, sep = "")
                pwdj = paste(getwd(), "/Univariate/Groups/", nj, sep = "")
                I = read.csv(pwdi, header = TRUE)
                J = read.csv(pwdj, header = TRUE)
                I = I[, -1]
                J = J[, -1]
                fin = ncol(sorted) - 1
                # perform WMW test for the effective combinations and extract p-values
                wilx.pv <- matrix(rep(NA, fin))
                for (q in 1:fin) {
                  wilx.pv[q, ] <- wilcox.test(I[, q], J[, q], paired = FALSE, exact = NULL, correct = FALSE, 
                    conf.level = 0.95, alternative = "two.sided")$p.value
                }
                rownames(wilx.pv) <- colnames(x.x)
                colnames(wilx.pv) <- c("pvalue")
                
                wmw.ij.pv = paste("WMWTest_pvalues_", ExcName(i, slink), "vs", ExcName(j, slink), 
                  ".csv", sep = "")
                assign(wmw.ij.pv, wilx.pv)
                write.csv(wilx.pv, paste(dirout.wm, wmw.ij.pv, sep = ""))
            }
        }
    }
}

ST_rForest <- function(file, ntree = 500, times = 100, gDist = TRUE, seed = 123, Labels = TRUE, nvarRF = 6) {
    
    dirout.w = paste(getwd(), "/randomForest", sep = "")
    # dirout.RF = paste(getwd(), '/RF_Plots/', sep='')
    dir.create(dirout.w)
    # dir.create(dirout.RF)
    cat("\n")
    statRF <- rForest(file, ntree = ntree, times = times, gDist = gDist, seed = seed)
    
    slinkPath = paste(getwd(), "/DataPretreatment", "/slink.csv", sep = "")
    getSlink = read.csv(slinkPath, header = TRUE)
    
    cat("\n", "Random Forest Model")
    cat("\n", "Type: ", statRF$randomForest$type)
    
    # cat('\n','Response Vector:','\n') pinfo <- unique(getSlink) rownames(pinfo) <- pinfo[,2];
    # pinfo[,2] <- NULL print(t(pinfo))
    cat("\n", "ntree:", statRF$randomForest$ntree)
    cat("\n", "mtry: ", statRF$randomForest$mtry)
    OOB <- round(statRF$randomForest$err.rate[statRF$randomForest$ntree, "OOB"] * 100, digits = 2)
    cat("\n", "OOB estimate of error rate:", OOB, "%", "\n")
    # print(statRF$randomForest$confusion)
    machGini <- data.frame(statRF$randomForest$importance[, "MeanDecreaseGini"])
    colnames(machGini) <- "giniImportance"
    machfile = paste(getwd(), "/randomForest/RF_giniImportance.csv", sep = "")
    write.csv(machGini, machfile, row.names = TRUE)
    
    machpvalue <- statRF$pimpTest$pvalue
    machfilep = paste(getwd(), "/randomForest/RF_pvalueImportance.csv", sep = "")
    write.csv(machpvalue, machfilep, row.names = TRUE)
    
    machfileMDS = paste(getwd(), "/randomForest/MultiDimensionalScalingPlot.pdf", sep = "")
    pdf(machfileMDS)
    mdsPlot <- mdsPlot(statRF$randomForest, statRF$pimpTest, Labels = Labels, slink = TRUE, slinkDat = getSlink)
    dev.off()
    machfileDim = paste(getwd(), "/randomForest/RF_mdsplotData.csv", sep = "")
    write.csv(mdsPlot, machfileDim, row.names = TRUE)
    
    cat("\n", "Permutation-based Gini importance measures")
    cat("\n", "The number of permutations: ", times)
    cat("\n", "Selected", nvarRF, "variables with top importance", "\n\n")
    
    machfileGini = paste(getwd(), "/randomForest/VarableImportancePlot.pdf", sep = "")
    pdf(machfileGini)
    giniPlot <- pvimPlot(statRF$randomForest, statRF$pimpTest, nvarRF = nvarRF)
    dev.off()
    selectedVar <- data.frame(giniImp = giniPlot$giniImp, pvalueImp = giniPlot$pvalueImp)
    selectedVar <- t(selectedVar)
    colnames(selectedVar) <- seq(1, nvarRF, 1)
    selectedVarPath = paste(getwd(), "/randomForest/RF_selectedVariablesData.csv", sep = "")
    write.csv(t(selectedVar), selectedVarPath, row.names = TRUE)
    print(selectedVar)
    
}

Plot_plsda_stat <- function(pcx, pcy, scaling, Labels) {
    pwd.score = paste(getwd(), "/PLS_DA_", scaling, "/PLSDA_Scores_", scaling, ".csv", sep = "")
    score = read.csv(pwd.score, header = TRUE)
    score.x = score[, -1]
    rownames(score.x) = score[, 1]
    pwd.load = paste(getwd(), "/PLS_DA_", scaling, "/PLSDA_Loadings_", scaling, ".csv", sep = "")
    loading = read.csv(pwd.load, header = TRUE)
    loading.x = loading[, -1]
    rownames(loading.x) = loading[, 1]
    pwd.p = paste(getwd(), "/PLS_DA_", scaling, "/PLSDA_P_", scaling, ".csv", sep = "")
    p = read.csv(pwd.p, header = TRUE)
    p.x = matrix(p[, -1], ncol = 1)
    pwd.ptot = paste(getwd(), "/PLS_DA_", scaling, "/PLSDA_Ptot_", scaling, ".csv", sep = "")
    p = read.csv(pwd.ptot, header = TRUE)
    slink = paste(getwd(), "/DataPretreatment", "/slink.csv", sep = "")
    slink = read.csv(slink, header = TRUE)
    pvar.a = p.x[pcx, ]/p
    pvar.b = p.x[pcy, ]/p
    pvar.ai = round(pvar.a * 100, 1)
    pvar.bi = round(pvar.b * 100, 1)
    cum = pvar.ai + pvar.bi
    xlab = paste("Component", pcx, "(", pvar.ai, "%)", sep = "")
    ylab = paste("Component", pcy, "(", pvar.bi, "%)", sep = "")
    
    max.pc1 = 1.3 * (max(abs(score.x[, pcx])))
    max.pc2 = 1.3 * (max(abs(score.x[, pcy])))
    lim = c()
    if (max.pc1 > max.pc2) {
        lim = c(-max.pc1, max.pc1)
    } else {
        lim = c(-max.pc2, max.pc2)
    }
    
    
    Epaxis <- dataEllipse_sT(score.x[, pcx], score.x[, pcy], levels = c(0.95), add = FALSE, draw = FALSE, 
        col = "black", lwd = 0.4, plot.points = FALSE, center.cex = 0.2)
    
    if (1.1 * max(Epaxis[, "x"]) < max(lim) & 1.1 * max(Epaxis[, "y"]) < max(lim)) {
        lim = lim
    } else {
        lim = c(1.2 * min(as.numeric(Epaxis)), 1.2 * max(as.numeric(Epaxis)))
    }
    
    
    pwdK = paste(getwd(), "/scaleData_", scaling, "/class.csv", sep = "")
    k = read.csv(pwdK, header = TRUE)
    k.s = k[order(k[, 2]), ]
    tutticolors = matrix(c(1, 2, 3, 4, 5, 6, 7, 8, "rosybrown4", "green4", "navy", "purple2", "orange", 
        "pink", "chocolate2", "coral3", "khaki3", "thistle", "turquoise3", "palegreen1", "moccasin", 
        "olivedrab3", "azure4", "gold3", "deeppink"), ncol = 1)
    col = c()
    for (i in 1:nrow(k.s)) {
        col = c(col, tutticolors[k.s[i, 2], ])
    }
    dirout = paste(getwd(), "/PLS_DA_", scaling, "/", sep = "")
    scor = paste(dirout, "ScorePlot_PLS_DA_", scaling, ".pdf", sep = "")
    pdf(scor)
    graphics::plot(score.x[, pcx], score.x[, pcy], col = col, pch = 19, xlab = c(xlab), ylab = c(ylab), 
        xlim = lim, ylim = lim, sub = paste("Cumulative Proportion of Variance Explained = ", cum, 
            "%", sep = ""), main = paste("PLS-DA Score Plot (", scaling, ")", sep = ""))
    axis(1, at = lim * 2, pos = c(0, 0), labels = FALSE, col = "grey", lwd = 0.7)
    axis(2, at = lim * 2, pos = c(0, 0), labels = FALSE, col = "grey", lwd = 0.7)
    
    
    if (Labels) {
        text(score.x[, pcx], score.x[, pcy], col = col, cex = 0.5, labels = rownames(score.x), pos = 1)
    } else {
        legend("topright", legend = levels(factor(slink[, 1])), bty = "n", pch = 19, col = seq_along(levels(factor(slink[, 
            1]))), text.col = seq_along(levels(factor(slink[, 1]))))
    }
    
    # requireNamespace(car) library(car)
    dataEllipse_sT(score.x[, pcx], score.x[, pcy], levels = c(0.95), add = TRUE, col = "black", lwd = 0.4, 
        plot.points = FALSE, center.cex = 0.2)
    dirout = paste(getwd(), "/PLS_DA_", scaling, "/", sep = "")
    # scor = paste(dirout, 'ScorePlot_PLS_DA_', scaling, '.pdf', sep='') dev.copy2pdf(file=scor)
    dev.off()
    Max.pc1 = 1.1 * (max(loading.x[, pcx]))
    Min.pc1 = 1.1 * (min(loading.x[, pcx]))
    Mpc1 = c(Min.pc1, Max.pc1)
    Max.pc2 = 1.1 * (max(loading.x[, pcy]))
    Min.pc2 = 1.1 * (min(loading.x[, pcy]))
    Mpc2 = c(Min.pc2, Max.pc2)
    load = paste(dirout, "W.cPlot_PLS_DA_", scaling, ".pdf", sep = "")
    # par(new = T)
    pdf(load)
    # dev.new()
    graphics::plot(loading.x[, pcx], loading.x[, pcy], xlim = Mpc1, ylim = Mpc2, xlab = paste("w*c values ", 
        pcx, sep = ""), ylab = paste("w*c values ", pcy, sep = ""), main = paste("PLS-DA Loading Plot (", 
        scaling, ")", sep = ""))
    text(loading.x[, pcx], loading.x[, pcy], labels = rownames(loading.x), cex = 0.7, pos = 1)
    axis(1, at = Mpc1 * 2, pos = c(0, 0), labels = FALSE, col = "grey", lwd = 0.7)
    axis(2, at = Mpc2 * 2, pos = c(0, 0), labels = FALSE, col = "grey", lwd = 0.7)
    # load = paste(dirout, 'W.cPlot_PLS_DA_', scaling, '.pdf', sep='') dev.copy2pdf(file=load)
    dev.off()
}

#' @name statTarget-package
#' @aliases statTarget-package
#' @docType package
#' @title Statistical Analysis of Molecular Profiles
#' @description An streamlined tool provides graphical user interface 
#' for quality control based signal correction, integration of 
#' MS-based data from multiple batches, and the 
#' comprehensive statistical analysis for omics studies.
#' @details Package: statTarget
#' @details Type: package
#' @details License: LGPL (>= 3)
#' @author Hemi Luan
#' @author Maintainer: Hemi Luan hemi.luan@gmail.com
#' @return A description of statTarget. See the details at https://stattarget.github.io
statTarget <- function() {
    message(date(), "\nStart the statTarget GUI session by using 'statTargetGUI()' ")
}

outlier_stat <- function(pcx, pcy, scaling) {
    pwd.score = paste(getwd(), "/PCA_Data_", scaling, "/PCA_ScoreMatrix.csv", sep = "")
    Score <- read.csv(pwd.score, sep = ",", header = TRUE)
    Score.x <- Score[, 2:ncol(Score)]
    rownames(Score.x) <- Score[, 1]
    dx = scale(Score.x[, pcx], scale = FALSE)
    dy = scale(Score.x[, pcy], scale = FALSE)
    sumdxdx = sum(dx * dx)
    sumdydy = sum(dy * dy)
    sumdxdy = sum(dx * dy)
    theta = 0.5 * atan((2 * sumdxdy)/(sumdydy - sumdxdx))
    c = cos(theta)
    s = sin(theta)
    X = (c * dx) - (s * dy)
    Y = (s * dx) + (c * dy)
    varX = stats::var(X)
    varY = stats::var(Y)
    M = sqrt(varX)
    m = sqrt(varY)
    M95 = M * 3.03315
    m95 = m * 3.03315
    Fx = sqrt(abs((M95^2) - (m95^2)))
    Fy = Fx * tan(theta)
    F1 = c(-Fx, -Fy)
    F2 = c(Fx, Fy)
    one = matrix(rep(1, nrow(Score.x)), ncol = 1)
    F1.m = one %*% F1
    F2.m = one %*% F2
    # requireNamespace(pdist) library(pdist)
    Punti = cbind(dx, dy)
    dist1 = pdist::pdist(Punti[, 1:2], F1.m)
    dist2 = pdist::pdist(Punti[, 1:2], F2.m)
    dist1 = as.matrix(dist1)
    dist2 = as.matrix(dist2)
    D = matrix(dist1[, 1] + dist2[, 1], ncol = 1)
    v = M95 * 2
    outliers <- rownames(Score.x)[which(D > as.numeric(v))]
    n = length(outliers)
    cat("\n", "The following observations are calculated as outliers: ")
    if (n == 0) {
        print("No outliers were detected")
    } else {
        print(outliers)
    }
}

### acu.ROC This function provide area under the Curve of ROC.  file The connection to the data in
### the Univariate file.  A matrix to outline the AUC.
aucROC <- function(file) {
    pwdfile = paste(getwd(), "/Univariate/DataTable.csv", sep = "")
    file = pwdfile
    x <- read.csv(file, sep = ",", header = TRUE)
    x.x = x[, 3:ncol(x)]
    rownames(x.x) = x[, 2]
    k = matrix(x[, 1], ncol = 1)
    slink = paste(getwd(), "/DataPretreatment", "/slink.csv", sep = "")
    slink = read.csv(slink, header = TRUE)
    x.n = cbind(k, x.x)
    sorted = x.n[order(x.n[, 1]), ]
    g = c()
    for (i in 1:nrow(sorted)) {
        if (any(g == sorted[i, 1])) {
            g = g
        } else {
            g = matrix(c(g, sorted[i, 1]), ncol = 1)
        }
    }
    NoF = nrow(g)
    dirout.w = paste(getwd(), "/Univariate/ROC", sep = "")
    dir.create(dirout.w)
    
    for (i in 1:NoF) {
        for (j in 1:NoF) {
            
            if (i < j) {
                # pb <- txtProgressBar(min = 1, max = NoF, style = 3)
                
                # Sys.sleep(0.000001)
                
                ni = paste("r.", ExcName(i, slink), ".csv", sep = "")
                nj = paste("r.", ExcName(j, slink), ".csv", sep = "")
                pwdi = paste(getwd(), "/Univariate/Groups/", ni, sep = "")
                pwdj = paste(getwd(), "/Univariate/Groups/", nj, sep = "")
                I = read.csv(pwdi, header = TRUE)
                J = read.csv(pwdj, header = TRUE)
                I = I[, -1]
                J = J[, -1]
                Ilf = matrix(rep(1, i), nrow(I))
                Jlf = matrix(rep(0, j), nrow(J))
                colnames(Ilf) = c("lf")
                colnames(Jlf) = c("lf")
                I = cbind(Ilf, I)
                J = cbind(Jlf, J)
                IJ = rbind(I, J)
                IJM <- as.matrix(IJ[, 2:ncol(IJ)])
                outf <- as.factor(IJ[, 1])
                # message(paste('*Group.', ExcName(i,slink), sep = ''), ' Vs.', paste(' Group.', ExcName(j,slink),
                # sep = ''))
                myROC = function(x, y) {
                  roc.obj <- suppressMessages(ROC::rocdemo.sca(y, x, function(x, thresh) ROC::dxrule.sca(x, 
                    thresh)))
                  auc <- ROC::AUC(roc.obj)
                  # rocdata <- c(roc.obj$auc,auc.ci[1],auc.ci[3])
                  if (auc >= 0.5) {
                    return(auc)
                  } else {
                    return(1 - auc)
                  }
                }
                
                myroc <- apply(IJM, 2, function(x) {
                  myROC(x, outf)
                })
                
                # myroc <- apply(IJM,2,function(x){myROC(x,outf)})
                myroc <- as.data.frame(myroc)
                colnames(myroc) = c("AUC")
                
                myroc.ij = paste("auc_roc_", ExcName(i, slink), "vs", ExcName(j, slink), ".csv", sep = "")
                assign(myroc.ij, myroc)
                write.csv(myroc, paste(dirout.w, myroc.ij, sep = "/"))
                # setTxtProgressBar(pb, i)
                
                
            }
        }
    }
}

chose_driver <- function(scaling) {
    pwd.n = paste(getwd(), "/Preprocessing_Data_", scaling, "/ProcessedTable.csv", sep = "")
    x <- read.csv(pwd.n, sep = ",", header = TRUE)
    x.x <- x[, 2:ncol(x)]
    rownames(x.x) <- x[, 1]
    print(colnames(x.x[1:10]))
}

flipdim_sT <- function(a, dim = 1) {
    if (!is.matrix(a)) 
        stop("Argument 'a' must ba a matrix.")
    if (!(dim %in% c(1, 2))) 
        stop("Argument 'dim' must be 1 or 2 (for rows or columns).")
    
    if (dim == 1) {
        a <- a[nrow(a):1, ]
    } else {
        a <- a[, ncol(a):1]
    }
    return(a)
}

RsdCal = function(file, batch = TRUE, DistPattern = TRUE, output = TRUE) {
    # xr <- read.csv(file, sep=',', header=TRUE)
    xr = file
    # xr = xr[,-1]
    xs = xr[, 3:ncol(xr)]
    if (batch) {
        x = cbind(xr[, 2], xr[, 1], xs)
        x.nn = x
        sorted = x.nn[order(x.nn[, 1]), ]
        g = c()
        for (i in 1:nrow(sorted)) {
            if (any(g == sorted[i, 1])) {
                g = g
            } else {
                g = matrix(c(g, sorted[i, 1]), ncol = 1)
            }
        }
        dirout.g = paste(getwd(), "/tmp", sep = "")
        dir.create(dirout.g)
        for (i in 1:nrow(g)) {
            vuota <- c()
            fin = matrix(rep(NA, ncol(sorted)), nrow = 1)
            for (j in 1:nrow(sorted)) {
                if (sorted[j, 1] == i) {
                  vuota <- as.matrix(sorted[j, ], nrow = 1)
                  rownames(vuota) = rownames(sorted)[j]
                  fin = rbind(fin, vuota)
                }
            }
            nam = paste("r", i, sep = ".")
            n = matrix(fin[-1, ], ncol = ncol(sorted))
            n.x = matrix(n[, -1], ncol = ncol(sorted) - 1)
            colnames(n.x) = colnames(x.nn[, 2:ncol(x.nn)])
            name = as.matrix(assign(nam, n.x))
            outputfileg = paste("r.", i, ".csv", sep = "")
            write.csv(name, paste(dirout.g, outputfileg, sep = "/"), row.names = FALSE)
        }
        dirout.w = paste(getwd(), "/statTarget/shiftCor/RSDresult", sep = "")
        dir.create(dirout.w)
        NoF = nrow(g)
        nrow = dim(xr)[2] - 2
        batchQC <- matrix(rep(NA), ncol = NoF, nrow = nrow)
        for (i in 1:NoF) {
            ni = paste("r.", i, ".csv", sep = "")
            pwdi = paste(getwd(), "/tmp/", ni, sep = "")
            I = read.csv(pwdi, header = TRUE)
            I = I[, -1]
            bS = t(bStatCor(I)[nrow(bStatCor(I)), ])
            bS = as.matrix(bS)
            batchQC[, i] = bS
        }
        bSall = t(bStatCor(xs)[nrow(bStatCor(xs)), ])
        batchQC <- cbind(batchQC, bSall)
        rownames(batchQC) <- NULL
        colnames(batchQC) <- NULL
        ########### RSDdist########################## message('Summary QC distribution of the CV in each batch and
        ########### all batch (last one):')
        distout <- matrix(rep(NA), nrow = NoF + 1, ncol = 20)
        for (i in 1:(NoF + 1)) {
            valueB <- batchQC[, i]
            Batchdist <- sapply(seq(0, 95, 5), function(x) {
                sum(valueB * 100 <= x + 5, na.rm = TRUE) * 100/length(valueB)
            })
            distout[i, ] <- matrix(Batchdist, 1)
        }
        colnames(distout) <- paste("CV", "<", seq(5, 100, 5), "%", sep = "")
        rownames(distout) <- paste("Batch", "_", seq(1, nrow(distout), 1), sep = "")
        rownames(distout)[nrow(distout)] <- "Total"
        
        if (DistPattern) {
            print(distout)
        }
        
        
        # return(distout)
        
        if (output) {
            RSDdist_CV = paste("RSDdist_QC_stat", ".csv", sep = "")
            write.csv(distout, paste(dirout.w, RSDdist_CV, sep = "/"))
            RSD_all = paste("RSD_all", ".csv", sep = "")
            write.csv(bSall, paste(dirout.w, RSD_all, sep = "/"))
        }
        tmpfile = paste(getwd(), "/tmp/", sep = "")
        unlink(tmpfile, recursive = TRUE)
    } else {
        bSall = t(bStatCor(xs)[nrow(bStatCor(xs)), ])
    }
    
    return(bSall)
    
}

plsda_stat <- function(scaling, silt) {
    pwd.x = paste(getwd(), "/scaleData_", scaling, "/ProcessedTable.csv", sep = "")
    x = read.csv(pwd.x, header = TRUE)
    x.x = x[, 2:ncol(x)]
    rownames(x.x) = x[, 1]
    pwdK = paste(getwd(), "/scaleData_", scaling, "/class.csv", sep = "")
    k = read.csv(pwdK, header = TRUE)
    k.x = matrix(k[, -1], ncol = 1)
    x.n = cbind(k.x, x.x)
    sorted = x.n[order(x.n[, 1]), ]
    g = c()
    for (i in 1:nrow(sorted)) {
        if (any(g == sorted[i, 1])) {
            g = g
        } else {
            g = matrix(c(g, sorted[i, 1]), ncol = 1)
        }
    }
    dimB = nrow(g) * nrow(sorted)
    B = matrix(rep(NA, dimB), ncol = nrow(g))
    for (i in 1:nrow(sorted)) {
        for (j in 1:nrow(g)) {
            jn <- g[j, 1]
            if (sorted[i, 1] == jn) {
                B[i, j] = 1
            } else {
                B[i, j] = 0
            }
        }
    }
    
    sorted.x = sorted[, -1]
    sorted.un = matrix(unlist(sorted.x), ncol = ncol(sorted.x))
    
    P = pls::plsr(B ~ sorted.un, method = c("oscorespls"), ncomp = 2, validation = "LOO")
    
    cat("\n", "PLS(-DA) Two Component Model Summary")
    
    summPls <- utils::capture.output(summary(P))
    # summPls[c(3,4,7)] print(summary(P))
    rownames(P$scores) = rownames(sorted.x)
    rownames(P$loadings) = colnames(sorted.x)
    dirout = paste(getwd(), "/PLS_DA_", scaling, "/", sep = "")
    dir.create(dirout)
    out.score = paste(dirout, "PLSDA_Scores_", scaling, ".csv", sep = "")
    write.csv(P$scores, out.score)
    out.load = paste(dirout, "PLSDA_Loadings_", scaling, ".csv", sep = "")
    write.csv(P$loadings, out.load)
    k = matrix(sorted[, 1], ncol = 1)
    tutticolors = matrix(c(1, 2, 3, 4, 5, 6, 7, 8, "rosybrown4", "green4", "navy", "purple2", "orange", 
        "pink", "chocolate2", "coral3", "khaki3", "thistle", "turquoise3", "palegreen1", "moccasin", 
        "olivedrab3", "azure4", "gold3", "deeppink"), ncol = 1)
    col = c()
    for (i in 1:nrow(k)) {
        col = c(col, tutticolors[k[i, ], ])
    }
    if (ncol(P$scores) == 1) {
        xlab = "Samples"
        ylab = "Score values Component 1"
        graphics::plot(P$scores[, 1], col = col, pch = 19, xlab = c(xlab), ylab = c(ylab), main = paste("PLS-DA Score Plot (", 
            scaling, ")", sep = ""))
        lim1 = nrow(P$scores) * 2
        axis(1, at = c(-lim1, lim1), col = "grey", pos = c(0, 0), labels = FALSE, lwd = 1)
        text(P$scores[, 1], col = col, cex = 0.5, pos = 1, labels = rownames(P$scores))
        pwdout = paste(dirout, "ScorePlot_PLSDA_1Component_", scaling, ".pdf", sep = "")
        dev.copy2pdf(file = pwdout)
        Max.pc2 = 1.1 * (max(P$loadings[, 1]))
        Min.pc2 = 1.1 * (min(P$loadings[, 1]))
        Mpc2 = c(Min.pc2, Max.pc2)
        pwdout1 = paste(dirout, "W.cPlot_PLSDA_1Component_", scaling, ".pdf", sep = "")
        pdf(pwdout1)
        # dev.new()
        graphics::plot(P$loadings[, 1], ylim = Mpc2, main = paste("PLS-DA Loading Plot (", scaling, 
            ")", sep = ""), xlab = "Variables", ylab = "W*c values (Component1)")
        text(P$loadings[, 1], cex = 0.7, pos = 1, labels = rownames(P$loadings))
        pwdout1 = paste(dirout, "W.cPlot_PLSDA_1Component_", scaling, ".pdf", sep = "")
        dev.off()
    } else {
        pairs = paste(dirout, "Pairs_PLSDA_", scaling, ".pdf", sep = "")
        pdf(pairs)
        pairs = c()
        if (ncol(P$scores) >= 3) {
            pairs = c(3)
        } else {
            pairs = c(ncol(P$scores))
        }
        
        pairs(P$scores[, 1:pairs], col = col)
        # pairs = paste(dirout, 'Pairs_PLSDA_', scaling, '.pdf', sep='')
        dev.off()
    }
    p.v1 = matrix(P$Xvar, ncol = 1)
    p.v.csv1 = paste(dirout, "PLSDA_P_", scaling, ".csv", sep = "")
    write.csv(p.v1, file = p.v.csv1)
    p.vtot = matrix(P$Xtotvar, ncol = 1)
    p.vtot.csv = paste(dirout, "PLSDA_Ptot_", scaling, ".csv", sep = "")
    write.csv(p.vtot, file = p.vtot.csv, row.names = FALSE)
    dp.vtot = as.numeric(p.vtot)
    p.v = p.v1/dp.vtot
    p.v = matrix(100 * p.v1/dp.vtot, ncol = 1)
    colnames(p.v) = c("R2X")
    p.v.csv = paste(dirout, "PLSDA_R2X_", scaling, ".csv", sep = "")
    write.csv(p.v, file = p.v.csv)
    # message(date(),'\\PLS-DA Finished!') VIP#########
    corr = cor(sorted.un, P$scores[, 1])
    splot = cbind(P$loadings[, 1], corr)
    splotstat = paste(dirout, "PLSDA_SPlot_", scaling, ".csv", sep = "")
    write.table(splot, splotstat)
    pwdsplot = paste(dirout, "PLS_DA_SPlot_", scaling, ".pdf", sep = "")
    pdf(pwdsplot)
    graphics::plot(splot[, 1], splot[, 2], main = paste("PLS-DA SPlot (", scaling, ")", sep = ""), 
        xlab = "W*c values (Component1)", ylab = "W*c values (Component1)")
    text(splot, cex = 0.7, pos = 1, labels = colnames(sorted.x))
    dev.off()
    # significant comp, when Q2S>0 R2_Q2........................................  R2X_pls=explvar(P)
    # R2X_pls = data.frame(P$'Xvar')
    Q2_all = as.data.frame(pls::R2(P, "all", intercept = 0)$val)
    # R2[which estimate,which response,which comp]
    unk <- t(Q2_all)
    Q2_all <- unk[apply(unk, 1, function(x) !all(is.na(x))), ]
    # rownames(Q2_all) = rownames(R2X_pls) stat=cbind(R2X_pls*0.01,Q2_all)
    colnames(Q2_all) <- c("R2Y(cum)", "Q2(cum)")
    R2T <- pls::R2(P, "all", intercept = 0)$val
    R2_Q2 <- data.frame(R2T[, , 2])  # two components
    rownames(R2_Q2) <- c("R2Y(cum)", "Q2Y(cum)")
    # cat('PLS(-DA)')
    cat("\n", nrow(x), "samples x", ncol(x) - 1, "variables")
    cat("\n", summPls[3])
    cat("\n", summPls[4])
    cat("\n", summPls[7], "\n")
    cat("\n", " Cumulative Proportion of Variance Explained: R2X(cum) = ", p.v[1] + p.v[2], "%", sep = "")
    cat("\n", "Cumulative Proportion of Response(s):", "\n\n")
    print(R2_Q2)
    plsdastat = paste(dirout, "PLSDA_R2Q2_", scaling, ".csv", sep = "")
    write.table(Q2_all, plsdastat)
    
    # PERMUTATION ........................................
    
    leng = length(g)
    if (leng == 2) {
        cat("\n", "Permutation of PLSDA Model START...!", "\n")
        # message('\\...........Tea Time! Take A Rest!...........') Y=c()
        Dx = list(X = sorted.un, Y = B[, 1])
        r2_sim_CV <- with(Dx, permut(Dx, silt = silt))
        
        colnames(r2_sim_CV) <- c("R2", "Q2", "correlation.coefficient")
        RQ_line = c(Q2_all[3, ], 1)
        r2_sim_P <- rbind(r2_sim_CV, RQ_line)
        permutstat = paste(dirout, "PLSDA_permut_", scaling, ".csv", sep = "")
        write.table(r2_sim_P, permutstat)
        permuplot = paste(dirout, "Permutation_", scaling, ".pdf", sep = "")
        par(cex.axis = 1, cex.lab = 1)
        pdf(permuplot, width = 6, height = 6)
        lim_R = 1.3 * (max(abs(r2_sim_P[, 1])))
        lim_Q = 1.3 * (max(abs(r2_sim_P[, 2])))
        if (lim_R > lim_Q) {
            lim = c(-lim_R)
        } else {
            lim = c(-lim_Q)
        }
        if (Q2_all[3, 1] > Q2_all[3, 2]) {
            lim_mY = c(Q2_all[3, 1])
        } else {
            lim_mY = c(Q2_all[3, 2])
        }
        graphics::plot(abs(r2_sim_P[, 3]), r2_sim_P[, 1], col = "grey", xlab = "Correlation", ylab = "", 
            xlim = c(-0.05, 1.05), ylim = c(lim, 1.3 * lim_mY), pch = 19, main = paste("Permutation Plot (", 
                scaling, ")", sep = ""))
        points(abs(r2_sim_P[, 3]), r2_sim_P[, 2], col = "black", xlim = c(-0.05, 1.05), ylim = lim, 
            pch = 19)
        graphics::abline(h = Q2_all[3, 1], col = "grey", lwd = 1.2)
        graphics::abline(h = Q2_all[3, 2], col = "black", lwd = 1.2)
        points(1, Q2_all[3, 1], col = "grey", lwd = 1, pch = 19)
        points(1, Q2_all[3, 2], col = "black", lwd = 1, pch = 19)
        # interceptR <- as.numeric(interR) interQ <-
        # lm(median(r2_sim_CV[,2])~median(r2_sim_CV[,3]))$fitted.values interceptQ <- as.numeric(interQ)
        # segments(0,interceptR,1,Q2_all[3,1],col='#D55E00')
        # segments(0,interceptQ,1,Q2_all[3,2],col='#009E73') abline(h=Q2_all[3,1],col='grey')
        # abline(h=Q2_all[3,1],col='gery')
        legend("bottomright", legend = c("R2Y", "Q2Y"), bty = "n", cex = 1.2, pch = 19, col = c("grey", 
            "black"))
        # legend('bottomright',1.0, legend = c('Q2'),bty='n', cex=1.5, pch = 19, col = '#009E73')
        grid(lwd = 0.8)
        dev.off()
        # message( '\nR2 and Q2!') output <- Q2_all[1:6,] colnames(output) <- c('R2 comp1', 'R2 comp2')
        # print(output) message(date(),'\\permutation time Finished!') permutstat = paste(dirout,
        # 'PLSDA_permut_', scaling, '.csv', sep='') write.table(r2_sim_CV,permutstat)
        # permuplot=paste(dirout, 'Permutation_', scaling, '.pdf', sep='') pdf(permuplot)
        # x_lim_CV=min(r2_sim[,2]) - 1.2*sd(r2_sim[,2]) minV = min(r2_sim_CV[,2]) - 1.2*sd(r2_sim_CV[,2])
        # maxV = max(r2_sim_CV[,2]) + 5*sd(r2_sim_CV[,2]) maxV = max(r2_sim_CV[,2]) +
        # abs(min(r2_sim_CV[,2]))
        
        # if (max(r2_sim_CV[,2]) > 1.2*Q2_all[3,2]) { x_lim_CV = c(minV, maxV) } else { x_lim_CV = c(minV,
        # 1.2*Q2_all[3,2]) }
        
        # hist(r2_sim_CV[,2],breaks=40,col='darkblue',border='white',
        # xlim=x_lim_CV,ylim=c(0,85),main='Permutation',axes = TRUE, xlab= 'Q2')
        # segments(Q2_all[3,2],38,Q2_all[3,2],0,col='#FF4F00',lwd=4)
        # points(Q2_all[3,2],42,col='red',lwd=1,bg='#FF4F00',pch=8) d = density(r2_sim_CV[,2])
        # polygon(d,col = 'lightgreen',border = NA) dev.off() message(date(),'\\Q2!') output <-
        # Q2_all[1:6,] colnames(output) <- c('R2 comp1', 'R2 comp2') print(output)
        # message(date(),'\\permutation time Finished!')
    }
    if (leng > 2) {
        cat("\n", "Warning: More than two groups, permutation test skipped!", "\n")
    }
    # detach() PERMUTATION .................................................. VIP
    if (nrow(P$Yloadings) > 2) {
        cat("\n", "Warning: VIP was only implemented for the binary response model!", "\n")
    } else {
        if (P$method != "oscorespls") 
            stop("\n", "Only implemented for orthogonal scores algorithm.  
         Refit with 'method = \"oscorespls\"'")
        
        SS <- c(P$Yloadings[1, ])^2 * colSums(P$scores^2)
        Wnorm2 <- colSums(P$loading.weights^2)
        SSW <- sweep(P$loading.weights^2, 2, SS/Wnorm2, "*")
        vip <- sqrt(nrow(SSW) * apply(SSW, 1, cumsum)/cumsum(SS))
        colnames(vip) <- colnames(x.x)
        vipstat = paste(dirout, "PLSDA_VIP_", scaling, ".csv", sep = "")
        write.table(t(vip), vipstat)
    }
}


permut <- function(file, silt) {
    Y = file$Y
    file = file
    env <- environment()
    r2_sim_CV_2 <- matrix(0, nrow = as.numeric(silt), ncol = 2)
    corrY <- c()
    # pb <- txtProgressBar(min = 1, max = silt, style = 3)
    pCV <- function(x) {
        Y <- get("Y", envir = env)
        file <- get("file", envir = env)
        sid <- sample(length(Y), length(Y))
        p1 = pls::plsr(Y[sid] ~ X, data = file, ncomp = 2, validation = "LOO", method = "oscorespls")
        q2r2 = pls::R2(p1, estimate = "all")
        corrY <- cor(Y[sid], Y)
        # r2_cum <- cum(r2) r2_cum <- r2_cum[1:slt]
        r2_sim_CV_2 <- cbind(t(as.data.frame(q2r2$val[5:6])), corrY)
    }
    output <- plyr::llply(1:silt, .fun = pCV, .progress = "text")
    output = do.call(rbind, output)
}

RSDdist <- function(sample.rsd, sample.nor.rsd, QC.rsd, QC.nor.rsd) {
    # browser()
    colour1 <- NULL
    colour2 <- NULL
    colour1[(sample.nor.rsd/sample.rsd) > 1] <- "#D55E00"
    colour1[(sample.nor.rsd/sample.rsd) == 1] <- "grey48"
    colour1[(sample.nor.rsd/sample.rsd) < 1] <- "#009E73"
    
    colour2[(QC.nor.rsd/QC.rsd) > 1] <- "#D55E00"
    colour2[(QC.nor.rsd/QC.rsd) == 1] <- "grey48"
    colour2[(QC.nor.rsd/QC.rsd) < 1] <- "#009E73"
    
    s.rsd.up <- sum(colour1 == "#D55E00") * 100/length(colour1)
    s.rsd.no <- sum(colour1 == "grey48") * 100/length(colour1)
    s.rsd.down <- sum(colour1 == "#009E73") * 100/length(colour1)
    
    q.rsd.up <- sum(colour2 == "#D55E00") * 100/length(colour2)
    q.rsd.no <- sum(colour2 == "grey48") * 100/length(colour2)
    q.rsd.down <- sum(colour2 == "#009E73") * 100/length(colour2)
    
    dirout.w = paste(getwd(), "/statTarget/shiftCor/RSDresult", sep = "")
    
    pdf(file.path(dirout.w, "RSD variation.pdf"), width = 10, height = 7)
    layout(matrix(c(1, 2), ncol = 2))
    # par(mar=c(5,5,4,2))
    plot(sample.rsd, sample.nor.rsd, xlab = "RSD (Before correction)", ylab = "RSD (After correcion)", 
        col = colour1, cex.lab = 0.8, cex.axis = 0.8, main = "RSD variation of samples", cex.main = 1, 
        pch = 19)
    abline(0, 1, lwd = 1, lty = 2)
    abline(h = 30, lwd = 1, lty = 2)
    abline(v = 30, lwd = 1, lty = 2)
    
    legend("topleft", c(paste("Increased RSD after correction:", round(s.rsd.up, 2), "%"), paste("No changed RSD after correction:", 
        round(s.rsd.no, 2), "%"), paste("Decreased RSD after correction:", round(s.rsd.down, 2), "%")), 
        col = c("#D55E00", "grey48", "#009E73"), pch = 19, cex = 0.6)
    
    
    plot(QC.rsd, QC.nor.rsd, xlab = "RSD (Before correction)", ylab = "RSD (After correction)", col = colour2, 
        cex.lab = 0.8, cex.axis = 0.8, main = "RSD variation QC", cex.main = 1, pch = 19)
    
    abline(0, 1, lwd = 1, lty = 2)
    abline(h = 30, lwd = 1, lty = 2)
    abline(v = 30, lwd = 1, lty = 2)
    
    
    legend("topleft", c(paste("Increased RSDafter correction:", round(q.rsd.up, 2), "%"), paste("Not changed RSD after correction:", 
        round(q.rsd.no, 2), "%"), paste("Decreased RSD after correction:", round(q.rsd.down, 2), "%")), 
        col = c("#D55E00", "grey48", "#009E73"), pch = 19, cex = 0.6)
    dev.off()
    ## 
    s.rsd.dis <- sapply(seq(0, 1.9, 0.1), function(x) {
        sum(sample.rsd > x & sample.rsd <= x + 0.1)
    }) * 100/length(sample.rsd)
    s.nor.rsd.dis <- sapply(seq(0, 1.9, 0.1), function(x) {
        sum(sample.nor.rsd > x & sample.nor.rsd <= x + 0.1)
    }) * 100/length(sample.nor.rsd)
    q.rsd.dis <- sapply(seq(0, 1.9, 0.1), function(x) {
        sum(QC.rsd > x & QC.rsd <= x + 0.1)
    }) * 100/length(QC.rsd)
    q.nor.rsd.dis <- sapply(seq(0, 1.9, 0.1), function(x) {
        sum(QC.nor.rsd > x & QC.nor.rsd <= x + 0.1)
    }) * 100/length(QC.nor.rsd)
    
    rsd.dis <- rbind(s.rsd.dis, s.nor.rsd.dis, q.rsd.dis, q.nor.rsd.dis)
    colnames(rsd.dis) <- paste(paste(seq(0, 190, 10), seq(10, 200, 10), sep = "-"), "%", sep = "")
    rownames(rsd.dis) <- c("sample", "sample.nor", "QC", "QC.nor")
    # write.csv(rsd.dis,file.path(path,'RSD distribution.csv'))
    
    # par(new=TRUE)
    pdf(file.path(dirout.w, "RSD distribution.pdf"), width = 8, height = 6)
    layout(matrix(c(1, 2), ncol = 2))
    # par(mar=c(3,6,4,5))
    barp <- barplot(rsd.dis[1:2, ], horiz = TRUE, beside = TRUE, col = c("#D55E00", "#009E73"), border = c("#D55E00", 
        "#009E73"), names.arg = paste(seq(0, 190, 10), seq(10, 200, 10), sep = "-"), ylab = "RSD (%)", 
        las = 2, cex.lab = 0.6, cex.main = 0.8, cex.axis = 0.6, cex.names = 0.5, main = "Samples")
    text(rsd.dis[1:2, ] + 1, barp, labels = round(rsd.dis[1:2, ], 0), xpd = TRUE, cex = 0.3, col = "grey48")
    legend("topright", c("Before Correction", "After Correction"), col = c("#D55E00", "#009E73"), 
        lty = 0.5, pch = 15, cex = 0.75, horiz = FALSE, bty = "n")
    # par(mar=c(5,6,4,5))
    barp <- barplot(rsd.dis[3:4, ], horiz = TRUE, beside = TRUE, col = c("#D55E00", "#009E73"), border = c("#D55E00", 
        "#009E73"), names.arg = paste(seq(0, 190, 10), seq(10, 200, 10), sep = "-"), ylab = "RSD (%)", 
        las = 2, cex.lab = 0.6, cex.main = 0.8, cex.axis = 0.6, cex.names = 0.5, main = "Quality Controls (QC)")
    text(rsd.dis[3:4, ] + 1, barp, labels = round(rsd.dis[3:4, ], 0), xpd = TRUE, cex = 0.3, col = "grey48")
    legend("topright", c("Before Correction", "After Correction"), col = c("#D55E00", "#009E73"), 
        lty = 0.5, pch = 15, cex = 0.75, horiz = FALSE, bty = "n")
    # dev.copy2pdf(file.path(dirout.w,'RSD distribution.pdf'),width=8,height=6)
    dev.off()
}

ellipse_sT <- function(center, shape, radius, log = "", center.pch = 19, center.cex = 1.5, segments = 51, 
    draw = TRUE, add = draw, xlab = "", ylab = "", col = palette()[2], lwd = 2, fill = FALSE, fill.alpha = 0.3, 
    grid = TRUE, ...) {
    trans.colors <- function(col, alpha = 0.5, names = NULL) {
        # this function by Michael Friendly
        nc <- length(col)
        na <- length(alpha)
        # make lengths conform, filling out to the longest
        if (nc != na) {
            col <- rep(col, length.out = max(nc, na))
            alpha <- rep(alpha, length.out = max(nc, na))
        }
        clr <- rbind(col2rgb(col)/255, alpha = alpha)
        col <- rgb(clr[1, ], clr[2, ], clr[3, ], clr[4, ], names = names)
        col
    }
    logged <- function(axis = c("x", "y")) {
        axis <- match.arg(axis)
        0 != length(grep(axis, log))
    }
    if (!(is.vector(center) && 2 == length(center))) 
        stop("center must be a vector of length 2")
    if (!(is.matrix(shape) && all(2 == dim(shape)))) 
        stop("shape must be a 2 by 2 matrix")
    if (max(abs(shape - t(shape)))/max(abs(shape)) > 1e-10) 
        stop("shape must be a symmetric matrix")
    angles <- (0:segments) * 2 * pi/segments
    unit.circle <- cbind(cos(angles), sin(angles))
    # ellipse <- t(center + radius*t(unit.circle %*% chol(shape,pivot=TRUE)))
    Q <- chol(shape, pivot = TRUE)
    order <- order(attr(Q, "pivot"))
    ellipse <- t(center + radius * t(unit.circle %*% Q[, order]))
    colnames(ellipse) <- c("x", "y")
    if (logged("x")) 
        ellipse[, "x"] <- exp(ellipse[, "x"])
    if (logged("y")) 
        ellipse[, "y"] <- exp(ellipse[, "y"])
    fill.col <- trans.colors(col, fill.alpha)
    if (draw) {
        if (add) {
            lines(ellipse, col = col, lwd = lwd, ...)
            if (fill) 
                polygon(ellipse, col = fill.col, border = NA)
        } else {
            plot(ellipse, type = "n", xlab = xlab, ylab = ylab, ...)
            if (grid) {
                grid(lty = 1, equilogs = FALSE)
                box()
            }
            lines(ellipse, col = col, lwd = lwd, ...)
            if (fill) 
                polygon(ellipse, col = fill.col, border = NA)
        }
        if ((center.pch != FALSE) && (!is.null(center.pch))) 
            points(center[1], center[2], pch = center.pch, cex = center.cex, col = col)
    }
    invisible(ellipse)
}

### Description statistic bStatX provide the basic data description for each features.  Data
### descriptions includes mean value, median value, sum, quartile, standard derivatives, etc.  file
### The file with the expression information.  A matrix for data description bStatX(file)
bStatX <- function(file) {
    # xr <- read.csv(file, sep=',', header=TRUE)
    xr = file
    xs = xr[, 3:ncol(xr)]
    x = cbind(xr[, 2], xr[, 1], xs)
    x.nn = x
    sorted = x.nn[order(x.nn[, 1]), ]
    g = c()
    for (i in 1:nrow(sorted)) {
        if (any(g == as.numeric(sorted[i, 1]))) {
            g = g
        } else {
            g = matrix(c(g, as.numeric(sorted[i, 1])), ncol = 1)
        }
    }
    dirout.g = paste(getwd(), "/statTarget/statAnalysis/tmp", sep = "")
    dir.create(dirout.g)
    slink = paste(getwd(), "/statTarget/statAnalysis/DataPretreatment", "/slink.csv", sep = "")
    slink = read.csv(slink, header = TRUE)
    for (i in 1:nrow(g)) {
        vuota <- c()
        fin = matrix(rep(NA, ncol(sorted)), nrow = 1)
        for (j in 1:nrow(sorted)) {
            if (sorted[j, 1] == i) {
                vuota <- as.matrix(sorted[j, ], nrow = 1)
                rownames(vuota) = rownames(sorted)[j]
                fin = rbind(fin, vuota)
            }
        }
        
        nam = paste("r", ExcName(i, slink), sep = ".")
        n = matrix(fin[-1, ], ncol = ncol(sorted))
        n.x = matrix(n[, -1], ncol = ncol(sorted) - 1)
        colnames(n.x) = colnames(x.nn[, 2:ncol(x.nn)])
        name = as.matrix(assign(nam, n.x))
        outputfileg = paste("r.", ExcName(i, slink), ".csv", sep = "")
        write.csv(name, paste(dirout.g, outputfileg, sep = "/"), row.names = FALSE)
    }
    dirout.w = paste(getwd(), "/statTarget/statAnalysis/dataSummary", sep = "")
    dir.create(dirout.w)
    NoF = nrow(g)
    for (i in 1:NoF) {
        ni = paste("r.", ExcName(i, slink), ".csv", sep = "")
        pwdi = paste(getwd(), "/statTarget/statAnalysis/tmp/", ni, sep = "")
        I = read.csv(pwdi, header = TRUE)
        I = I[, -1]
        bS = bStat(I)
        bStat.i = paste("dataSumm_", ExcName(i, slink), ".csv", sep = "")
        assign(bStat.i, bS)
        write.csv(t(bS), paste(dirout.w, bStat.i, sep = "/"))
    }
    tmpfile = paste(getwd(), "/tmp/", sep = "")
    unlink(tmpfile, recursive = TRUE)
}


# This library is free software; you can redistribute it and/or modify it under the terms of the
# GNU Library General Public License as published by the Free Software Foundation; either version
# 2 of the License, or (at your option) any later version.  This library is distributed in the
# hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public License
# for more details.  You should have received A copy of the GNU Library General Public License
# along with this library; if not, write to the Free Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA


################################################################################ FUNCTION: BASIC STATISTICS: basicStats Returns a basic statistics summary


bStatCor = function(x, ci = 0.95) {
    # A function implemented by Diethelm Wuertz
    
    # Description: Calculates Basic Statistics
    
    # Arguments: x - an object which can be transformed by the function as.matrix() into an object of
    # class matrix.  ci - a numeric value setting the confidence interval.
    
    # Value: a two-column data frame, where the first column takes the value of the statistics, and
    # the second its name, e.g.  'nobs', 'NAs', 'Minimum', 'Maximum', '1. Quartile', '3.  Quartile',
    # 'Mean', 'Median', 'Sum', 'SE Mean', 'LCL Mean', 'UCL Mean', 'Variance', 'Stdev', 'Skewness',
    # 'Kurtosis')
    
    # FUNCTION:
    
    # Univariate/Multivariate:
    y = as.matrix(x)
    
    
    # Handle Column Names:
    if (is.null(colnames(y))) {
        Dim = dim(y)[2]
        if (Dim == 1) {
            colnames(y) = paste(substitute(x), collapse = ".")
        } else if (Dim > 1) {
            colnames(y) = paste(paste(substitute(x), collapse = ""), 1:Dim, sep = "")
        }
    }
    
    # Internal Function - CL Levels:
    cl.vals = function(x, ci) {
        x = x[!is.na(x)]
        n = length(x)
        if (n <= 1) 
            return(c(NA, NA))
        se.mean = sqrt(stats::var(x)/n)
        t.val = qt((1 - ci)/2, n - 1)
        mn = mean(x)
        lcl = mn + se.mean * t.val
        ucl = mn - se.mean * t.val
        c(lcl, ucl)
    }
    
    # Basic Statistics:
    nColumns = dim(y)[2]
    ans = NULL
    for (i in 1:nColumns) {
        X = as.numeric(y[, i])
        # Observations:
        X.length = length(X)
        X = X[!is.na(X)]
        X.na = X.length - length(X)
        # Basic Statistics:
        z = c(X.length, X.na, min(X), max(X), as.numeric(quantile(X, prob = 0.25, na.rm = TRUE)), 
            as.numeric(quantile(X, prob = 0.75, na.rm = TRUE)), mean(X), median(X), sum(X), sqrt(stats::var(X)/length(X)), 
            cl.vals(X, ci)[1], cl.vals(X, ci)[2], stats::var(X), sqrt(stats::var(X)), sqrt(stats::var(X))/mean(X))
        # Row Names:
        znames = c("nobs", "NAs", "Minimum", "Maximum", "1. Quartile", "3. Quartile", "Mean", "Median", 
            "Sum", "SE Mean", "LCL Mean", "UCL Mean", "Variance", "Stdev", "RSD")
        # Output as data.frame
        result = matrix(z, ncol = 1)
        row.names(result) = znames
        ans = cbind(ans, result)
    }
    
    # Column Names:
    colnames(ans) = colnames(y)
    
    # Return Value:
    data.frame(round(ans, digits = 6))
}


################################################################################ 


bStat = function(x, ci = 0.95) {
    # A function implemented by Diethelm Wuertz
    
    # Description: Calculates Basic Statistics
    
    # Arguments: x - an object which can be transformed by the function as.matrix() into an object of
    # class matrix.  ci - a numeric value setting the confidence interval.
    
    # Value: a two-column data frame, where the first column takes the value of the statistics, and
    # the second its name, e.g.  'nobs', 'NAs', 'Minimum', 'Maximum', '1. Quartile', '3.  Quartile',
    # 'Mean', 'Median', 'Sum', 'SE Mean', 'LCL Mean', 'UCL Mean', 'Variance', 'Stdev', 'Skewness',
    # 'Kurtosis')
    
    # FUNCTION:
    
    # Univariate/Multivariate:
    y = as.matrix(x)
    
    
    # Handle Column Names:
    if (is.null(colnames(y))) {
        Dim = dim(y)[2]
        if (Dim == 1) {
            colnames(y) = paste(substitute(x), collapse = ".")
        } else if (Dim > 1) {
            colnames(y) = paste(paste(substitute(x), collapse = ""), 1:Dim, sep = "")
        }
    }
    
    # Internal Function - CL Levels:
    cl.vals = function(x, ci) {
        x = x[!is.na(x)]
        n = length(x)
        if (n <= 1) 
            return(c(NA, NA))
        se.mean = sqrt(stats::var(x)/n)
        t.val = qt((1 - ci)/2, n - 1)
        mn = mean(x)
        lcl = mn + se.mean * t.val
        ucl = mn - se.mean * t.val
        c(lcl, ucl)
    }
    
    # Basic Statistics:
    nColumns = dim(y)[2]
    ans = NULL
    for (i in 1:nColumns) {
        X = y[, i]
        # Observations:
        X.length = length(X)
        X = X[!is.na(X)]
        X.na = X.length - length(X)
        # Basic Statistics:
        z = c(X.length, X.na, min(X), max(X), as.numeric(quantile(X, prob = 0.25, na.rm = TRUE)), 
            as.numeric(quantile(X, prob = 0.75, na.rm = TRUE)), mean(X), median(X), sum(X), sqrt(stats::var(X)/length(X)), 
            cl.vals(X, ci)[1], cl.vals(X, ci)[2], stats::var(X), sqrt(stats::var(X)))
        # Row Names:
        znames = c("nobs", "NAs", "Minimum", "Maximum", "1. Quartile", "3. Quartile", "Mean", "Median", 
            "Sum", "SE Mean", "LCL Mean", "UCL Mean", "Variance", "Stdev")
        # Output as data.frame
        result = matrix(z, ncol = 1)
        row.names(result) = znames
        ans = cbind(ans, result)
    }
    
    # Column Names:
    colnames(ans) = colnames(y)
    
    # Return Value:
    data.frame(round(ans, digits = 6))
}


################################################################################ 

permut <-
  function(file, silt){
    Y = file$Y
    file = file
    env <- environment()
    r2_sim_CV_2 <- matrix(0,nrow = as.numeric(silt), ncol = 2)
    corrY <- c()
    #pb <- txtProgressBar(min = 1, max = silt, style = 3)
    pCV <- function(x){
      Y <-  get("Y", envir=env) 
      file <- get("file", envir=env) 
      sid <- sample(length(Y),length(Y))
      p1=pls::plsr(Y[sid]~X,data = file, ncomp = 2, validation = "CV",
                   method ="oscorespls")
      q2r2=pls::R2(p1,estimate = "all")
      corrY <- cor(Y[sid],Y)
      #r2_cum <- cum(r2)
      #r2_cum <- r2_cum[1:slt]
      r2_sim_CV_2 <- cbind(t(as.data.frame(q2r2$val[5:6])),corrY)
    }
    output <- plyr::llply(1:silt,.fun= pCV,.progress = "text")
    output = do.call(rbind,output)
  }    
  

### ...........................................................................  This function
### provide the odd.ratio value for metabolomics data.  file The file with the expression
### information.  An odds ratio is a measure of association between an exposure and an outcome.  OR
### is equal to 1, which means exposure does not affect odds of outcome. OR is high than 1, which
### means exposure associated with higher odds of outcome.  OR is less than 1, which means exposure
### associated with lower odds of outcome.The 95 percent of confidence interval (CI) is used to
### estimate the precision of the OR. A large CI indicates a low level of precision of the OR,
### whereas a small CI indicates a higher precision of the OR. It is important to note however, that
### unlike the p value, the 95 percent of CI does not report a statistical significance.
### ...........................................................................
oddRatio <- function(file) {
    pwdfile = paste(getwd(), "/Univariate/DataTable.csv", sep = "")
    file = pwdfile
    x <- read.csv(file, sep = ",", header = TRUE)
    x.x = x[, 3:ncol(x)]
    rownames(x.x) = x[, 2]
    k = matrix(x[, 1], ncol = 1)
    slink = paste(getwd(), "/DataPretreatment", "/slink.csv", sep = "")
    slink = read.csv(slink, header = TRUE)
    x.n = cbind(k, x.x)
    sorted = x.n[order(x.n[, 1]), ]
    g = c()
    for (i in 1:nrow(sorted)) {
        if (any(g == sorted[i, 1])) {
            g = g
        } else {
            g = matrix(c(g, sorted[i, 1]), ncol = 1)
        }
    }
    NoF = nrow(g)
    dirout.w = paste(getwd(), "/Univariate/oddRatio", sep = "")
    dir.create(dirout.w)
    for (i in 1:NoF) {
        for (j in 1:NoF) {
            if (i < j) {
                ni = paste("r.", ExcName(i, slink), ".csv", sep = "")
                nj = paste("r.", ExcName(j, slink), ".csv", sep = "")
                pwdi = paste(getwd(), "/Univariate/Groups/", ni, sep = "")
                pwdj = paste(getwd(), "/Univariate/Groups/", nj, sep = "")
                I = read.csv(pwdi, header = TRUE)
                J = read.csv(pwdj, header = TRUE)
                I = I[, -1]
                J = J[, -1]
                Ilf = matrix(rep(i), nrow(I))
                Jlf = matrix(rep(j), nrow(J))
                colnames(Ilf) = c("lf")
                colnames(Jlf) = c("lf")
                I = cbind(Ilf, I)
                J = cbind(Jlf, J)
                IJ = rbind(I, J)
                # fin = ncol(sorted) - 1 we <- matrix(rep(NA, fin))
                # odds.radio..........................................................
                IJM <- as.matrix(IJ[, 2:ncol(IJ)])
                outFactor <- as.factor(IJ[, 1])
                logit_or <- function(outcome, a) {
                  
                  if (sd(a) == 0) {
                    Out <- c(0, 0, 0, 1)
                    names(Out) = c("Odd.radio", "2.5%CI", "97.5%CI", "p.value")
                  } else {
                    # check the odds of A to B
                    #outcome <- as.factor(c(rep("A",30),rep("B",28)))
                    
                    levels(outcome)[1] <- "pos" # pos
                    levels(outcome)[2] <- "neg" # neg
                    outcome <-relevel(outcome, ref= "neg")
                    glm.log <- glm(outcome ~ scale(a), control = list(maxit = 50), family = binomial(link = "logit"))
                    res <- summary(glm.log)$coefficients[2, 1:2]
                    p <- summary(glm.log)$coefficients[2, 4]
                    out <- c(res, p)
                    Odd.radio <- exp(out[1])
                    #Odd.radio <- exp(glm.log$coefficients["scale(a)"])
                    Std.error1 <- exp(out[1] - 1.96 * out[2])
                    Std.error2 <- exp(out[1] + 1.96 * out[2])
                    Out <- c(Odd.radio, Std.error1, Std.error2, p)
                    names(Out) = c("Odd.radio", "2.5%CI", "97.5%CI", "p.value")
                  }
                  return(Out)
                  
                }
                or <- apply(t(IJM), 1, function(x) {
                  logit_or(outFactor, x)
                })
                or <- as.data.frame(t(or))
                # write.table(or,'odds_radio.txt',sep='\t',quote=F)
                or.ij = paste("odds_of_", ExcName(i, slink), "to", ExcName(j, slink), ".csv", sep = "")
                assign(or.ij, or)
                write.csv(or, paste(dirout.w, or.ij, sep = "/"))
            }
        }
    }
}

#' @name rForest
#' @title Random Forest classfication in statTarget
#' @description  rForest provides the Breiman's random 
#' forest algorithm for classification and permutation-based variable 
#'importance measures (PIMP-algorithm).
#' @param file An data frame or 'Stat File' from statTarget software.
#' @param ntree Number of trees to grow. This should not be set to too small a 
#' number, to ensure that every input row gets predicted at least a few times.
#' @param times The number of permutations for 
#' permutation-based variable importance measures. 
#' @param gDist If gDist is TRUE the null importance distributions are 
#' approximated with Gaussian distributions else with empirical cumulative 
#' distributions.
#' @param seed For the same set of random variables and reproducible results.
#' @param ...  A generic function in randomForest package
#' @return Objects  Two objects from statTarget_rForest (1. randomForest,rfModel; 
#' 2. PIMPresult, pimpModel)
#' @return VarImp  The original Gini importance
#' @return PerVarImp  A matrix, where the permuted VarImp measures for the
#' predictor variable.
#' @return p-value  The probability of observing the original VarImp or a larger
#' value, given the fitted null importance distribution.
#' @return 
#' p.ks.test The p-values of the Kolmogorov-Smirnov Tests for 
#' each row PerVarImp. 
#' @usage rForest(file,ntree = 100,times = 100, gDist = TRUE,
#' seed = 123,...)
#' @examples 
#' datpath <- system.file('extdata',package = 'statTarget')
#' statFile <- paste(datpath,'data_example_long.csv', sep='/')
#' getFile <- read.csv(statFile,header=TRUE)
#' rFtest <- rForest(getFile,ntree = 10,times = 5)
#' @references 
#' Altmann A.,Tolosi L.,Sander O. and Lengauer T. (2010) Permutation importance: 
#' a corrected feature importance measure, Bioinformatics 26 (10), 1340-1347.
#' @references 
#' Ender Celik. (2015) vita: Variable Importance Testing Approaches. R package 
#' version 1.0.0 https://CRAN.R-project.org/package=vita
#' @author Hemi Luan, hemi.luan@gmail.com
#' @export 
rForest <- function(file, ntree = 100, times = 100, gDist = TRUE, seed = 123, ...) {
    x = file
    x.x = x[, 3:ncol(x)]
    rownames(x.x) = x[, 1]
    k = matrix(x[, 2], ncol = 1)
    x.n = cbind(k, x.x)
    sorted = x.n[order(x.n[, 1]), ]
    g = c()
    for (i in 1:nrow(sorted)) {
        if (any(g == sorted[i, 1])) {
            g = g
        } else {
            g = matrix(c(g, sorted[i, 1]), ncol = 1)
        }
    }
    NoF = nrow(g)
    if (NoF >= 2) {
        yF <- as.factor(k[, 1])
        rfModel <- randomForest::randomForest(x.x, yF, ntree = ntree, importance = TRUE, proximity = TRUE, 
            ...)
        pimpModel <- pimpRF(x.x, yF, rfModel, times = times, seed = seed, gDist = gDist)
    } else {
        stop("Single group is not allowed !")
    }
    return(list(randomForest = rfModel, pimpTest = pimpModel))
}

#' @name mdsPlot
#' @title MDSplot in statTarget
#' @description  Multi-dimensional scaling plot of proximity matrix from 
#' randomForest.
#' @param rForest An object of class randomForest that contains the proximity 
#' component from statTarget_rForest function.
#' @param pimpModel An object of permutation-based variable Gini importance 
#' measures (PIMP-algorithm) from statTarget_rForest function.
#' @param Labels Labels is TRUE for visible the sample name in the figure else 
#' with the index for class.
#' @param slink Logical indicating if slinkDat is active for extenal classID.
#' @param slinkDat A data frame for the extenal classID.
#' @param ...  A generic MDSplot function in randomForest package
#' @usage mdsPlot(rForest,pimpModel,Labels = TRUE,slink = FALSE, 
#' slinkDat, ...)
#' @return The output of cmdscale on 1 - rf$proximity is returned invisibly.
#' @seealso MDSplot
#' @author Hemi Luan, hemi.luan@gmail.com
#' @export
mdsPlot <- function(rForest, pimpModel, Labels = TRUE, slink = FALSE, slinkDat, ...) {
    
    mdsData <- stats::cmdscale(1 - rForest$proximity, eig = TRUE, k = 2)
    colnames(mdsData$points) <- paste("Dim", 1:2)
    OOBer <- round(rForest$err.rate[rForest$ntree, "OOB"] * 100, digits = 2)
    if (slink) {
        slink <- slinkDat
    } else {
        xF <- rForest$y
        txF <- factor(xF)
        for (f in 1:length(levels(txF))) {
            levels(txF)[f] <- f
        }
        slink <- data.frame(cbind(xF, as.numeric(txF)))
    }
    if (is.data.frame(slink)) {
        NULL
    } else {
        stop("The slink Data should be set to data frame")
    }
    
    k <- slink[, 2]
    tutticolors = matrix(c(1, 2, 3, 4, 5, 6, 7, 8, "rosybrown4", "green4", "navy", "purple2", "orange", 
        "pink", "chocolate2", "coral3", "khaki3", "thistle", "turquoise3", "palegreen1", "moccasin", 
        "olivedrab3", "azure4", "gold3", "deeppink"), ncol = 1)
    col = c()
    for (s in 1:length(k)) {
        col = c(col, tutticolors[k[s], ])
    }
    mds <- mdsData$points
    lim = c()
    max.pc1 = 1.3 * (max(abs(mds[, 1])))
    max.pc2 = 1.3 * (max(abs(mds[, 2])))
    if (max.pc1 > max.pc2) {
        lim = c(-max.pc1, max.pc1)
    } else {
        lim = c(-max.pc2, max.pc2)
    }
    
    
    Epaxis <- dataEllipse_sT(mds[, 1], mds[, 2], levels = c(0.95), add = FALSE, draw = FALSE, col = "black", 
        lwd = 0.4, plot.points = FALSE, center.cex = 0.2)
    
    if (1.1 * max(Epaxis[, "x"]) < max(lim) & 1.1 * max(Epaxis[, "y"]) < max(lim)) {
        lim = lim
    } else {
        lim = c(1.2 * min(as.numeric(Epaxis)), 1.2 * max(as.numeric(Epaxis)))
    }
    
    # MDS
    mdsPDF <- graphics::plot(mds[, 1], mds[, 2], col = col, pch = 19, xlim = lim, ylim = lim, xlab = "Dim 1", 
        ylab = "Dim 2", sub = paste("OOB estimate of error rate = ", round(OOBer, 2), " %", sep = ""), 
        main = paste("RandomForest MDSPlot", sep = ""), ...)
    axis(1, at = lim * 2, pos = c(0, 0), labels = FALSE, col = "grey", lwd = 0.7)
    axis(2, at = lim * 2, pos = c(0, 0), labels = FALSE, col = "grey", lwd = 0.7)
    dataEllipse_sT(mds[, 1], mds[, 2], levels = c(0.95), add = TRUE, col = "grey48", lwd = 0.4, plot.points = FALSE, 
        center.cex = 0.2)
    
    if (Labels) {
        text(mds[, 1], mds[, 2], mdsPDF, col = col, cex = 0.5, labels = rownames(mds), pos = 1)
    } else {
        legend("topright", legend = levels(factor(slink[, 1])), bty = "n", pch = 19, col = seq_along(levels(factor(slink[, 
            1]))), text.col = seq_along(levels(factor(slink[, 1]))), cex = 0.8)
    }
    return(mds)
}

### Box Plot Box Plot of each metabolite between groups.  file The connection for intensity of
### metabolites in each group.  The box plot
boxPlot <- function(file) {
    pwdfile = paste(getwd(), "/Univariate/DataTable.csv", sep = "")
    file = pwdfile
    x <- read.csv(file, sep = ",", header = TRUE)
    x.x = x[, 3:ncol(x)]
    rownames(x.x) = x[, 2]
    k = matrix(x[, 1], ncol = 1)
    slink = paste(getwd(), "/DataPretreatment", "/slink.csv", sep = "")
    slink = read.csv(slink, header = TRUE)
    colnames(k)[1] = "Class"
    x.x = cbind(k, x.x)
    sorted = x.x[order(x.x[, 1]), ]
    sorted.x = as.matrix(sorted[, -1], ncol = ncol(sorted) - 1)
    g = c()
    for (i in 1:nrow(sorted)) {
        if (any(g == sorted[i, 1])) {
            g = g
        } else {
            g = matrix(c(g, sorted[i, 1]), ncol = 1)
        }
    }
    NoF = nrow(g)
    dirbox = paste(getwd(), "/Univariate/BoxPlot/", sep = "")
    dir.create(dirbox)
    for (i in 2:ncol(x.x)) {
        name = paste(getwd(), "/Univariate/BoxPlot/", colnames(x.x)[i], ".pdf", sep = "")
        pdf(name)
        boxplot(x.x[, i] ~ slink[, 1], boxfill = c(seq(1, NoF, 1)), ylab = colnames(x.x)[i], xlab = "Groups", 
            border = "grey30", main = paste("Boxplot ", colnames(x.x)[i], sep = ""))
        dev.off()
    }
    
}

#' @name statAnalysis
#' @title statAnalysis for statistical analysis for omics data or others.
#' @description statAnalysis provides the statistical analysis for metabolomics
#'  data or others.
#' @param file The file with the expression information with long format 
#' @param Frule Modified n precent rule function. A variable will be kept if it has a non-zero value
#' for at least n precent of samples in any one group. (Default: 0.8)  
#' @param imputeM The parameter for imputation method i.e., nearest neighbor 
#' averaging, 'KNN'; minimum values, 'min'; Half of minimum values, 'minHalf'; 
#' median values, 'median'. 
#' @param normM The parameter for normalization method (i.e median quotient 
#'  normalization, 'PQN'; integral normalization , 'SUM', and 'NONE').
#' @param glog  Generalised logarithm (glog) transformation, with the default 
#' value TRUE. The glog is a better behaved log transformation when some data 
#' values are zero or just near zero.
#' @param FDR The false discovery rate for conceptualizing the rate of type I 
#' errors in null hypothesis testing when conducting multiple comparisons.
#' @param scaling Scaling method before statistic analysis (PCA or PLS-DA). 
#' 'pareto', 'Pareto', 'p' or 'P' can be used for specifying the Pareto scaling.
#'  'auto', 'Auto', 'auto', 'a' or 'A' can be used for specifying the Auto 
#'  scaling (or unit variance scaling). 'vast', 'Vast', 'v' or 'V' can be used
#'   for specifying the vast scaling. 'range', 'Range', 'r' or 'R' can be used 
#'   for specifying the Range scaling.
#' @param ntree Number of trees to grow for randomForest model. This should not 
#' be set to too small a number, to ensure that every input row gets predicted 
#' at least a few times.
#' @param nvarRF The number of the variables with top importance in randomforest
#'  model
#' @param silt The number of permutation for PLS-DA model and variable importance of randomForest.
#' @param pcax Principal components in PCA model for the x-axis.
#' @param pcay Principal components in PCA model for the y-axis.
#' @param Labels Name labels for score plot of multiple statistical analysis
#' @param save.boxplot if TRUE, the box plot is performed
#' @param plot.volcano if TRUE, the volcano plot is performed
#' @param upper.lim The up-regulated metabolites using Fold Changes cut off 
#' values in the Volcano plot. Fold change values will be calculated 
#' before normalization step. 
#' @param lower.lim The down-regulated metabolites using Fold Changes cut off
#' values in the Volcano plot. Fold change values will be calculated before 
#' normalization step.
#' @param sig.lim The significance level for metabolites in the Pvalues file in 
#' the Volcano plot.
#' @return The statAnalsis output files. See the details at https://stattarget.github.io
#' @examples 
#' datpath <- system.file('extdata',package = 'statTarget')
#' file <- paste(datpath,'data_example_long.csv', sep='/')
#' statAnalysis(file,Frule = 0.8, normM = 'NONE', imputeM = 'KNN', glog = TRUE,scaling = 'Pareto')
#' @author Hemi Luan, hemi.luan@gmail.com
#' @export
#' @keywords PCA PLSDA P-value 
statAnalysis <- function(file, Frule = 0.8, normM = "NONE", imputeM = "KNN", glog = TRUE, FDR = TRUE, 
    ntree = 500, nvarRF = 5, scaling = "Pareto", plot.volcano = TRUE, save.boxplot =FALSE,silt = 20, pcax = 1, pcay = 2, Labels = TRUE, upper.lim = 2, 
    lower.lim = 0.5, sig.lim = 0.05) {
    dirout.uni = paste(getwd(), "/statTarget/", sep = "")
    dirsc.IDA = getwd()
    
    dir.create(dirout.uni)
    dirout.uni = paste(getwd(), "/statTarget/statAnalysis/", sep = "")
    dir.create(dirout.uni)
    
    dat <- read.delim2(file, header = FALSE, sep = ",")
    dat2 <- t(dat)
    colnames(dat2) <- dat2[1,]
    dat <- as.data.frame(dat2[-1,])

    cat("\n\nstatTarget: statistical analysis start... Time: ", date(), "\n\n")
    
    cat("* Step 1: Evaluation of missing value...", "\n")
    
    
    cat("\n", "Data Link", "\n")
    cat(" statFile:", file, "\n")
    
    imdat <- dat[, 3:ncol(dat)]
    imdat <- apply(imdat,2, as.numeric)
    
    if (length(imdat[imdat < 0L]) > 0) {
        imdat[imdat < 0L] <- 0L
    }
    imdat[imdat == 0L] <- NA
    
    imdat <- cbind(dat[, 1:2], imdat)
    
    cat("\n", "The number of missing value in Data Profile: ", sum(is.na(imdat)))
    
    ############# Filter miss value###################
    
    FilterMV = function(m, degree) {
        dx <- c()
        for (i in 1:ncol(m)) {
            freq <- as.vector(tapply(m[, i], degree, function(x) {
                sum(is.na(x) | as.matrix(x) == 0)/length(x)
            }
            ))
            if (sum(freq > 1 - Frule) > 0) 
                dx <- c(dx, i)
        }
        if (length(dx) > 0) 
            m <- m[, -dx]
        return(m)
    }
    classF <- as.factor(imdat[, 2])
    imdatF = FilterMV(imdat, classF)
    Frule_warning = paste("The number of filtered variables using the modified ", Frule * 100, "% rule :", 
        sep = " ")
    cat("\n", Frule_warning, " ", dim(imdat)[2] - dim(imdatF)[2], "\n\n")
    imdatF <- data.frame(imdatF)
    
    dirout.uni = paste(getwd(), "/statTarget/statAnalysis/DataPretreatment/", sep = "")
    dir.create(dirout.uni)
    
    cat("* Step 2: Summary statistics start... Time: ", date(), "\n\n")
    
    
    cat("* Step 3: Missing value imputation start... Time: ", date(), "\n")
    
    ############## impute missing value#################
    cat("\n", "Imputation method was set at", imputeM)
    
    if (imputeM == "KNN") {
        # require(impute)
        mvd <- impute::impute.knn(as.matrix(imdatF[, 3:ncol(imdatF)]), rowmax = 0.99, colmax = 0.99, 
            maxp = 15000)
        inputedData <- mvd$data
    } else if (imputeM == "min") {
        
        minValue <- function(x, group) {
            group = as.factor(as.numeric(group))
            for (i in 1:dim(x)[1]) {
                for (j in 3:dim(x)[2]) {
                  if (is.na(x[i, j]) == TRUE) {
                    x[i, j] <- tapply(as.numeric(x[, j]), group, min, na.rm = TRUE)[group[i]]
                  }
                }
            }
            return(x)
        }
        inputedData = minValue(imdatF, classF)
        inputedData = inputedData[, -c(1, 2)]
        
    } else if (imputeM == "minHalf") {
        
        minHalfValue <- function(x, group) {
            group = as.factor(as.numeric(group))
            for (i in 1:dim(x)[1]) {
                for (j in 3:dim(x)[2]) {
                  if (is.na(x[i, j]) == TRUE) {
                    x[i, j] <- tapply(as.numeric(x[, j]), group, min, na.rm = TRUE)[group[i]]/2
                  }
                }
            }
            return(x)
        }
        inputedData = minHalfValue(imdatF, classF)
        inputedData = inputedData[, -c(1, 2)]
        
    } else if (imputeM == "median") {
        medianvalue <- function(x, group) {
            group = as.factor(as.numeric(group))
            for (i in 1:dim(x)[1]) {
                for (j in 3:dim(x)[2]) {
                  if (is.na(x[i, j]) == TRUE) {
                    x[i, j] <- tapply(as.numeric(x[, j]), group, median, na.rm = TRUE)[group[i]]
                  }
                }
            }
            return(x)
        }
        inputedData = medianvalue(imdatF, classF)
        inputedData = inputedData[, -c(1, 2)]
    }
    cat("\n", "The number of NA value after imputation:", sum(is.na(inputedData)))
    
    if (sum(is.na(inputedData)) > 0) {
        minHalfValue2 <- function(x, group) {
            group = as.factor(as.numeric(group))
            for (i in 1:dim(x)[1]) {
                for (j in 1:dim(x)[2]) {
                  if (is.na(x[i, j]) == TRUE) {
                    x[i, j] <- tapply(as.numeric(x[, j]), group, min, na.rm = TRUE)[group[i]]/2
                  }
                }
            }
            return(x)
        }
        inputedData = minHalfValue2(inputedData, classF)
        
        # mvd2 <- impute::impute.knn(as.matrix(inputedData[,1:ncol(inputedData)]), rowmax = 0.99, colmax =
        # 0.99, maxp = 15000) inputedData <- mvd2$data
        cat("\n", "The number of missing value after", "the second imputation: ", sum(is.na(inputedData)))
    }
    
    cat("\n", "Imputation Finished!", "\n")
    
    ## .....................................
    
    ## Transform the Factor
    
    ## .....................................
    
    TraceFc <- function(x) {
        xF <- factor(x[, 2])
        for (i in 1:length(levels(xF))) {
            levels(xF)[i] <- i
        }
        x[, 2] <- xF
        return(x)
    }
    
    imdatF2 <- TraceFc(imdatF)
    
    
    mach <- cbind(data.frame(dat[, 2]), data.frame(imdatF2[, 2]))
    colnames(mach) <- c("Class", "Number")
    machfile = paste(getwd(), "/statTarget/statAnalysis/DataPretreatment/slink.csv", sep = "")
    write.csv(mach, machfile, row.names = FALSE)
    
    prefile = paste(getwd(), "/statTarget/statAnalysis/DataPretreatment/data_imputation_", imputeM, 
        ".csv", sep = "")
    write.csv(cbind(imdatF[, 1:2], inputedData), prefile, row.names = FALSE)
    
    # Summary statistics
    imdatStat <- TraceFc(imdatF)
    bStatX(imdatStat)
    
    ############## Normalization #################
    cat("\n", "* Step 4: Normalization start... Time: ", date(), "\n")
    normTemp <- t(inputedData)
    normimdatF <- normTarget(normTemp, method = normM)
    
    cat("\n", "Normalization method was set at", normM, "\n")
    
    normF <- cbind(imdatF2[, 1:2], t(normimdatF))
    normF <- as.matrix(normF)
    # dataNorm output
    normfile = paste(getwd(), "/statTarget/statAnalysis/DataPretreatment/data_normalization_", normM, 
        ".csv", sep = "")
    write.csv(normF, normfile, row.names = FALSE)
    # dataPreprocess inputedData <- imdatF
    
    if (glog) {
        # glog trans
        x <- read.csv(normfile, header = TRUE)
        GloggedSmpd <- glog(x[, 3:ncol(x)], 2)
        sdv <- apply(GloggedSmpd, 2, sd)
        meanI <- apply(GloggedSmpd, 2, mean)
        logvarI <- data.frame(meanI, sdv)
        log_rankI <- logvarI[order(logvarI[, 1], decreasing = FALSE), ]
        
        sdvf <- apply(x[, 3:ncol(x)], 2, sd)
        meanII <- apply(x[, 3:ncol(x)], 2, mean)
        logvarII <- data.frame(meanII, sdvf)
        log_rankII <- logvarII[order(logvarII[, 1], decreasing = FALSE), ]
        
        # dataLog output
        logfile = paste(getwd(), "/statTarget/statAnalysis/DataPretreatment/data_glog.csv", sep = "")
        write.csv(cbind(x[, 1:2], GloggedSmpd), logfile, row.names = FALSE)
        pdf("./statTarget/statAnalysis/DataPretreatment/glogPlot.pdf")
        par(mfrow = c(1, 2))
        plot(1:dim(log_rankII)[1], log_rankII[, 2], pch = 21, bg = "green", col = rgb(0, 0, 0, 100, 
            maxColorValue = 255), xlab = "rank of mean intensity", ylab = "Standard deviation")
        plot(1:dim(log_rankI)[1], log_rankI[, 2], pch = 21, bg = "red", col = rgb(0, 0, 0, 100, maxColorValue = 255), 
            xlab = "rank of mean intensity", ylab = "Standard deviation (after glog-transformation)")
        graphics::title("Variance stabilization with glog-transformation", line = -3, outer = TRUE)
        dev.off()
    } else {
        cat("\n", "Warning: glog-transformation skipped!", "\n")
    }
    
    ## .....................................
    
    ## Multiple statistical analysis
    
    ## .....................................
    
    
    if (glog) {
        setwd("./statTarget/statAnalysis/")
        cat("\n")
        cat("* Step 5: Glog PCA-PLSDA start... Time: ", date(), "\n")
        
        cat("\n", "Scaling method was set at", scaling, "\n")
        explore_data_stat(logfile, scaling, normalize = FALSE)
        Plot_pca_score_stat(pcax, pcay, scaling, Labels)
        Plot_pca_loading(pcax, pcay, scaling)
        outlier_stat(pcax, pcay, scaling)
        plsda_stat(scaling, silt)
        Plot_plsda_stat(1, 2, scaling, Labels)
        cat("\n")
        
        cat("* Step 6: Glog Random Forest start... Time: ", date(), "\n")
        logf <- read.csv(logfile, header = TRUE)
        ST_rForest(logf, ntree = ntree, times = silt, nvarRF = nvarRF, Labels = Labels)
        
        cat("\n")
        cat("* Step 7: Univariate Test Start...! Time: ", date(), "\n")
        log = sT_univariate(logfile, FDR = FDR, upper.lim = upper.lim, lower.lim = lower.lim, sig.lim = sig.lim,plot.volcano = plot.volcano, save.boxplot = save.boxplot)
    } else {
        setwd("./statTarget/statAnalysis/")
        cat("\n", "Step 5: PCA-PLSDA start... Time: ", date(), "\n")
        cat("\n", "Scaling method was set at", scaling, "\n")
        explore_data_stat(normfile, scaling, normalize = FALSE)
        Plot_pca_score_stat(pcax, pcay, scaling, Labels)
        Plot_pca_loading(pcax, pcay, scaling)
        outlier_stat(pcax, pcay, scaling)
        plsda_stat(scaling, silt)
        Plot_plsda_stat(1, 2, scaling, Labels)
        cat("\n")
        cat("* Step 6: Random Forest start... Time: ", date(), "\n")
        logFF <- read.csv(normfile, header = TRUE)
        ST_rForest(logFF, ntree = ntree, times = silt, nvarRF = nvarRF, Labels = Labels)
        
        cat("\n")
        cat("* Step 7: Univariate Test Start...! Time: ", date(), "\n")
        log = sT_univariate(normfile, FDR = FDR, upper.lim = upper.lim, lower.lim = lower.lim, sig.lim = sig.lim, plot.volcano = plot.volcano, save.boxplot = save.boxplot)
    }
    cat("\n", "Output Link:", getwd(), "\n")
    cat("\n", "Statistical Analysis Finished! Time: ", date(), "\n")
    #cat("\n", "Correction Finished! Time: ", date(), "\n")
    cat("\n", "####################################", "\n")
    cat(" # Software Version: statTarget 2.0 #", "\n")
    cat(" ####################################", "\n")
    
    # Parameter set
    
    stPam1 <- c("Frule", "normM", "imputeM", "glog", "FDR", "ntree", "nvarRF", "scaling", "silt", 
        "pcax", "pcay", "Labels", "upper.lim", "lower.lim", "sig.lim")
    stPam2 <- c(Frule, normM, imputeM, glog, FDR, ntree, nvarRF, scaling, silt, pcax, pcay, Labels, 
        upper.lim, lower.lim, sig.lim)
    stpam <- data.frame(stPam1, stPam2)
    colnames(stpam) <- c("parameter", "value")
    par_st = paste("statTarget/ParameterStatAnalysis", ".log", sep = "")
    write.table(stpam, paste(dirsc.IDA, par_st, sep = "/"), row.names = FALSE)
    
    tmpfile = paste(getwd(), "/tmp/", sep = "")
    unlink(tmpfile, recursive = TRUE)
    tmpfiles = paste(getwd(), "/Groups/", sep = "")
    unlink(tmpfiles, recursive = TRUE)
    setwd(dirsc.IDA)
    
}
normTarget <- function(object, method = "PQN", use_percent = 100) {
    # Note features should be in rows, samples in columns Dieterle F,Ross A, Schlotterbeck G, Senn H.:
    # Probabilistic Quotient Normalization as Robust Method to Account for Diluition of Complex
    # Biological Mixtures. Application in 1H NMR Metabolomics.  Anal Chem 2006;78:4281-90.
    
    if (method == "PQN") {
        # SUM Normalization
        x.s <- matrix(colSums(object, na.rm = TRUE), nrow = 1)
        uni = matrix(rep(1, nrow(object)), ncol = 1)
        area.uni <- uni %*% x.s
        normSUM <- object/area.uni
        # PQN
        fectures <- abs(normSUM * 100)
        rem <- floor(nrow(fectures) - (nrow(fectures) * use_percent/100))
        # calculate variance
        varian <- apply(fectures, 1, stats::var)  #calculate variance for each feature across samples
        # remove features with highest variance
        sel <- order(varian)[1:(nrow(fectures) - rem)]
        fectures2 <- fectures[sel, ]  #data matrix that contains only features with low variance
        refer <- apply(fectures2, 1, median, na.rm = TRUE)
        quotient <- fectures2/refer
        quotient.median <- apply(quotient, 2, median, na.rm = TRUE)
        norm.fectures <- t(t(fectures2)/quotient.median)
        rownames(norm.fectures) <- rownames(fectures2)
        colnames(norm.fectures) <- colnames(fectures2)
    }
    
    if (method == "SUM") {
        x.s <- matrix(colSums(object, na.rm = TRUE), nrow = 1)
        uni = matrix(rep(1, nrow(object)), ncol = 1)
        area.uni <- uni %*% x.s
        norm.fectures <- (object/area.uni) * 100
    }
    if (method == "NONE") {
        norm.fectures <- object
    }
    
    normtarget <- norm.fectures
    return(normtarget)
}

### 80 percent rule for metabolomics data
### This function provide the 80 percent rule 
### for the missing value correction
### degree The group factor in the data
### m The data input
### A matrix with adjusted data
### A variable will be kept if it has a non-zero value 
### for at least 80 percent of each group.
### Smilde A. Fusion of mass spectrometry-based 
### metabolomics data. 2005, Anal. Chem. 77, 6729-36.
rule80 <- function(m,degree) {
  dx <- c() 
  for(i in 1:ncol(m)){
    freq <- as.vector(tapply(m[,i] , degree, function(x){
      sum(x == "0")/length(x)}))
    if(sum(freq > 0.8) > 0) dx <- c(dx , i)
  } 
  if(length(dx) >0) m <- m[,-dx]
  return(m)
}

Plot_pca_loading <- function(pcx, pcy, scaling) {
    loadi = paste(getwd(), "/PCA_Data_", scaling, "/PCA_LoadingsMatrix.csv", sep = "")
    Loading <- read.csv(loadi, sep = ",", header = TRUE)
    Loading.x <- Loading[, 2:ncol(Loading)]
    rownames(Loading.x) <- Loading[, 1]
    ppppp = paste(getwd(), "/PCA_Data_", scaling, "/PCA_P", sep = "")
    Pvar <- read.csv(ppppp, sep = ",", header = TRUE)
    Pvar.x <- Pvar[, 2:ncol(Pvar)]
    rownames(Pvar.x) <- Pvar[, 1]
    cum = Pvar[pcx, 2] + Pvar[pcy, 2]
    pca <- paste("Loadings PC", pcx, " (", Pvar[pcx, 2], ") %")
    pcb <- paste("Loadings PC", pcy, " (", Pvar[pcy, 2], ")%")
    lim.load = c()
    Max.pc1 = 1.1 * (max(Loading.x[, pcx]))
    Min.pc1 = 1.1 * (min(Loading.x[, pcx]))
    Mpc1 = c(Min.pc1 * 2, Max.pc1 * 2)
    Max.pc2 = 1.1 * (max(Loading.x[, pcy]))
    Min.pc2 = 1.1 * (min(Loading.x[, pcy]))
    Mpc2 = c(Min.pc2 * 2, Max.pc2 * 2)
    # dev.new()
    E = paste(getwd(), "/PCA_Data_", scaling, "/LoadingPlot_PC", pcx, "vsPC", pcy, ".pdf", sep = "")
    pdf(file = E)
    graphics::plot(Loading.x[, pcx], Loading.x[, pcy], xlab = pca, ylab = pcb, xlim = c(Min.pc1, Max.pc1), 
        ylim = c(Min.pc2, Max.pc2), main = paste("PCA Loading Plot (", scaling, ")", sep = ""), sub = paste("Cumulative Proportion of Variance Explained = ", 
            cum, "%", sep = ""))
    axis(1, at = Mpc1, pos = c(0, 0), labels = FALSE, col = "grey", lwd = 0.7)
    axis(2, at = Mpc2, pos = c(0, 0), labels = FALSE, col = "grey", lwd = 0.7)
    text(Loading.x[, pcx], Loading.x[, pcy], labels = rownames(Loading.x), cex = 0.6, pos = 1)
    dev.off()
}

### Random forest function.
### nvarRF shows the number of variables in 
### Gini plot of Randomforest model (=< 100). 
### file The data of metabolites expression
### Multi result was outputed, such as MDS plot, 
### variable Gini plot and list results
### RandomF(file, nvarRF)
### The MDSplot and data matrix
RandomF <- function(file,nvarRF) {
  pwdfile = paste(getwd(), "/Univariate/DataTable.csv", sep = "")
  file = pwdfile
  x <- read.csv(file, sep = ",", header = TRUE)
  x.x = x[, 3:ncol(x)]
  rownames(x.x) = x[, 2]
  k = matrix(x[, 1], ncol = 1)
  slink = paste(getwd(), "/PreTable","/slink.csv", sep="")
  slink = read.csv(slink, header=TRUE)
  x.n = cbind(k, x.x)
  sorted = x.n[order(x.n[, 1]), ]
  g = c()
  for (i in 1:nrow(sorted)) {
    if (any(g == sorted[i, 1])) {
      g = g
    }
    else {
      g = matrix(c(g, sorted[i, 1]), ncol = 1)
    }
  }
  NoF = nrow(g)
  dirout.w = paste(getwd(), "/Univariate/RForest", sep = "")
  dirout.RF = paste(getwd(), "/Univariate/RForest/RF_Plots/", sep="")
  dir.create(dirout.w)
  dir.create(dirout.RF)
  
  for (i in 1:NoF) {
    for (j in 1:NoF) {
      if (i < j) {
        ni = paste("r.", ExcName(i,slink), ".csv", sep = "")
        nj = paste("r.", ExcName(j,slink), ".csv", sep = "")
        pwdi = paste(getwd(), "/Univariate/Groups/", ni, sep = "")
        pwdj = paste(getwd(), "/Univariate/Groups/", nj, sep = "")
        I = read.csv(pwdi, header = TRUE)
        J = read.csv(pwdj, header = TRUE)
        I = I[, -1]
        J = J[, -1]
        Ilf = matrix(rep(i),nrow(I))
        Jlf = matrix(rep(j),nrow(J))    
        colnames(Ilf) = c("lf")
        colnames(Jlf) = c("lf")
        I = cbind(Ilf,I)
        J = cbind(Jlf,J)
        IJ = rbind(I,J)
        #fin = ncol(sorted) - 1
        #we <- matrix(rep(NA, fin))
        #odds.radio...........................................................
        IJM <- as.matrix(IJ[,2:ncol(IJ)])
        outf <-as.factor(IJ[,1])
         
        #data(iris)
        #y <- as.factor(ifelse(iris$Species == "setosa" |
        #                        iris$Species == "virginica", 1, 0) )
        #xdata <- iris[,1:4]
        #yf <- c(1,1,2,2)
        #y <- as.factor(yf)
        message(paste("\n*Group.", ExcName(i,slink), sep = "")," Vs.", 
                paste(" Group.", ExcName(j,slink), sep = ""))
        nrep = 20
        mods <- rlply(nrep, randomForest(IJM, outf, ntree=500),
                      .progress = "text")
        dat <- matrix(rep(NA),nrep,length(IJM[1,]))
        #dat1 <- matrix(rep(NA),1,)
        dat1 <-c()
        for (k in 1:nrep) {
          for (h in 1:length(IJM[1,])){
            p <- function(x,y) { dat = as.list(importance(mods[[x]]))[[y]]}
            dat[k,h] <- p(k,h)
            dat1[k] <- median(mods[[k]][[4]][,1])
          }
          colnames(dat) = colnames(IJM)
          dat1 = matrix(dat1)
          colnames(dat1) = c("error.rate")
        }
        #write.csv(dat,"Gini_RF.csv")
        #write.csv(dat1,"error.rate.csv")
        Gini.ij = paste("Gini_", ExcName(i,slink), "vs", 
                        ExcName(j,slink), ".csv", sep = "")
        assign(Gini.ij, dat)
        write.csv(dat, paste(dirout.w, Gini.ij, sep = "/"))
        error.rate.ij = paste("error.rate_", ExcName(i,slink), "vs", 
                              ExcName(j,slink), ".csv", sep = "")
        assign(error.rate.ij, dat1)
        write.csv(dat1, paste(dirout.w, error.rate.ij, sep = "/"))
        
        ########### Randomforest MDS,boxplot ######
        
        rfmods <- randomForest(IJM, outf, ntree=200,proximity=TRUE)
        MDS = paste(dirout.RF, "MDSPlot_", ExcName(i,slink), "vs", 
                    ExcName(j,slink), ".pdf", sep="")
        pdf(MDS)
        rfplot <- MDSplot(rfmods, outf,k=2,
        #bg=rainbow(length(levels(as.factor(outf))))[unclass=outf], 
        #palette=c(1,2),
        pch=19, 
        #col=c("#D55E00","#009E73"), 
        palette=c("#D55E00","#009E73"), 
        xlab="Dimension 1", 
        ylab="Dimension 2", 
        cex=1.2, main=paste("MDS Plot ", ExcName(i,slink), " vs ", 
                            ExcName(j,slink), sep=""))
        legend("bottomright",
          legend=levels(outf),
          pch=19, 
          col=c("#D55E00","#009E73"), 
          cex=0.6)
        #text(Dim1, Dim2, rownames(euro.mds), cex=0.8, col="red")
        dev.off()
        RFscore.ij = paste("RFscore_", ExcName(i,slink), "vs", 
                           ExcName(j,slink), ".csv", sep = "")
        assign(RFscore.ij, rfplot$points)
        write.csv(rfplot$points, paste(dirout.w, RFscore.ij, sep = "/"))
        Giniplot = paste(dirout.RF, "Giniplot_", ExcName(i,slink), "vs", 
                         ExcName(j,slink), ".pdf", sep="")
        pdf(Giniplot)
        orderindex <- apply(dat,2,median)
        datindex <- cbind(orderindex,t(dat))
        sort.dat <- datindex[order(datindex[,1], 
                                   na.last=NA, decreasing = TRUE),]
        sort.dat <- t(sort.dat[,-1]) 
        sort.range <- sort.dat[,1:nvarRF]
        sort.datFilter <- flipdim_sT(sort.range, 2)
        par(mar=c(3, 10, 3, 10) + 0.1,mgp=c(1.5,0.5,0))
        colfunc <- colorRampPalette(c("grey48", "#D55E00"))
        #colfunc <- colorRampPalette(colors = brewer.pal(9,"reds"))
        boxplot(sort.datFilter, 
                main = paste("Gini Plot ", 
                ExcName(i,slink), " vs ", ExcName(j,slink), sep=""),
                notch = FALSE, col =colfunc(nvarRF), 
                cex =0.1, las = 1, xlab= "Mean Decrease Gini",
                ylab="Variables", horizontal = TRUE,
                pars = list(boxwex = 0.6, staplewex = 0.3,
                outwex = 0.3),lwd=0.3,
                cex.lab=0.8, cex.axis =0.4,medlwd=0.3, xaxt='n',yaxt="n")
        axis(2,labels= c(colnames(sort.datFilter)), at = 1:nvarRF, 
        cex = 0.3, tck= -0.015,cex.axis=0.3,las=1)
        axis(1,cex = 0.5, font= 0.05, tck= -0.015,cex.axis=0.3)
        grid(NULL,NA, lwd = 1,col = "gray")
        #axis(side=1, tcl=-0.1, labels=FALSE)
        dev.off()
        #library(plyr)
        #library(randomforest)
        #library(pracma)
      }
    }
  }
}

#' @name shiftCor_dQC (deprecated)
#' @title QC-free based signal correction
#' @description  shiftCor_dQC provides the QC-free based signal correction for 
#' large scale mass spectrometry-based omics data.
#' @param samPeno The file with the meta information including the sample name,
#'  batches, class and order (denoting other covariates besides batch). 
#' @param samFile The file with the expression information. 
#' @param Frule Modified n precent rule function. A variable will be kept if it has a non-zero value
#' for at least n precent of samples in any one group. 
#' (Default: 0.8)  
#' @param MLmethod 'ComBat' allows users to adjust for batch effects in datasets where the batch covariate is known, using methodology
#' described in Johnson et al. 2007. It uses either parametric or non-parametric empirical Bayes frameworks for adjusting data for
#' batch effects.  Users are returned an expression matrix that has been corrected for batch effects.The function was revised accroding to 'sva' package (version = "3.8").
#' @param imputeM The parameter for imputation method i.e., nearest neighbor 
#' averaging, 'KNN'; minimum values, 'min'; Half of minimum values, 'minHalf'; 
#' median values, 'median'.
#' @param par.prior TRUE indicates parametric adjustments will be used, FALSE indicates non-parametric adjustments will be used
#' @param prior.plots (Optional) TRUE give prior plots.
#' @param mod.covariates TRUE indicates model matrix for outcome of interest and other covariates besides batch (Column 'order' denotes covariates the  in samPeno file).
#' @param batch.Num (Optional) NULL If given, will use the selected batch as a reference for batch adjustment.
#' @return the shiftCor files. See the details at https://stattarget.github.io
#' @examples 
#' datpath <- system.file('extdata',package = 'statTarget')
#' samPeno <- paste(datpath,'MTBLS79_dQC_sampleList.csv', sep='/')
#' samFile <- paste(datpath,'MTBLS79.csv', sep='/')
#' shiftCor_dQC(samPeno,samFile, Frule = 0.8, MLmethod = "Combat",mod.covariates = FALSE)
#' shiftCor_dQC(samPeno,samFile, Frule = 0.8, MLmethod = "Combat",mod.covariates = TRUE,batch.Num = 1)
#' @keywords Quality Controls,Correction
#' @export 
shiftCor_dQC <- function(samPeno, samFile, Frule = 0.8, imputeM = "KNN",
                         MLmethod = "Combat",par.prior = TRUE,prior.plots = FALSE,
                         mod.covariates =FALSE, batch.Num = NULL) {
    cat("\n")
    
    cat("statTarget: Signal Correction Start... Time:", date(), "\n\n")
    cat("* Step 1: Data File Checking Start..., Time: ", date(), "\n")
    
    cat("\n", "Data Link", "\n")
    cat(" metaFile:", samPeno, "\n")
    cat(" profileFile:", samFile, "\n")
    
    samPeno <- read.csv(samPeno, header = TRUE, check.names = FALSE, stringsAsFactors = FALSE)
    samPeno <- as.data.frame(samPeno)
    #samPeno <- plyr::arrange(samPeno, order)
    samFile <- read.csv(samFile, header = FALSE, check.names = FALSE, stringsAsFactors = FALSE)
    samFile <- t(samFile)
    colnames(samFile) <- samFile[1, ]
    samFile <- as.data.frame(samFile[-1, ])
    rownames(samFile) <- samFile$name
    
    ############## Checking the input file#############
    
    
    ## .............................................
    
    ## Data Matching
    
    ## .............................................
    
    cat("\n", " ", dim(samPeno)[1], " Meta Samples vs ", dim(samFile)[1], " Profile samples", sep = "")
    cat("\n", "The Meta samples list (*NA, missing data from the Profile File)")
    cat("\n\n")
    mcdat <- samFile[, 1][match(samPeno[, 1], samFile[, 1])]
    print(as.vector(mcdat))
    
    ## .............................................
    if (any(is.na(mcdat))) {
        stop("\n", "Missing data from the Profile File! Check your data please!!")
    }
    
    if (dim(samFile)[1] - dim(samPeno)[1] > 0) {
        cat("\n", "Warning: The sample size in Profile File is larger than Meta File! ")
        cat("\n")
    } else if (dim(samFile)[1] - dim(samPeno)[1] < 0) {
        stop("\n", "The sample size in Profile File should be no less than Meta File!", " Check your data please!!")
    }
    
    samFP <- samFile[samPeno$sample, ]
    
    if (sum(is.na(samPeno$class)) <= 0) {
        warning("\n", "There were no QC samples in your data!")
    }
    
    samPeno_stat <- samPeno
    rownames(samPeno_stat) <- samPeno[, 1]
    #qc_seq <- rownames(samPeno_stat)
    #qc_seq_tmp <- grep("QC", qc_seq)
    
    
    
    #sam_seq_tmp <- samPeno_stat[-c(qc_seq_tmp), ]
    
    if (sum(is.na(samPeno_stat$batch)) > 0) {
        stop("\n", "There were missing values in batch! Check your data please!\n")
    }
    if (sum(is.na(samPeno_stat$class)) > 0) {
        stop("\n", "There were missing values (Unclassified data) in sample class! 
         Check your data please!\n")
    }
    
    
    #samPeno_stat$class[is.na(samPeno_stat$class)] <- "QC"
    data_stat = aggregate(samPeno_stat$class, by = list(Category = samPeno_stat$class), FUN = length)
    batch_stat = aggregate(samPeno_stat$class, by = list(Category = samPeno_stat$batch), FUN = length)
    colnames(data_stat) <- c("Class", "No.")
    colnames(batch_stat) <- c("Batch", "No.")
    cat("\n", "Meta-information:", "\n\n")
    print(data_stat)
    print(batch_stat)
    num_sam <- dim(samFile[, 2:ncol(samFile)])
    num_sam <- data.frame(num_sam)
    colnames(num_sam) <- c("no.")
    rownames(num_sam) <- c("Samples", "Metabolites")
    cat("\n", "Metabolic profile information:", "\n\n")
    print(num_sam)
    
    
    ################# data NA########
    samFP_temp <- as.matrix(samFP[, 2:ncol(samFP)])
    
    if (length(samFP_temp[samFP_temp < 0L]) > 0) {
        samFP_temp[samFP_temp < 0L] <- 0L
    }
    samFP_temp[samFP_temp == 0L] <- NA
    
    cat("\n")
    cat("* Step 2: Evaluation of Missing Value...", "\n")
    cat("\n", "The number of missing value before signal correction: ", sum(is.na(samFP_temp)))
    imsamFP <- samFP_temp
    ############# Filter miss value###################
    
    FilterMV = function(m, degree) {
        dx <- c()
        for (i in 1:ncol(m)) {
            freq <- as.vector(tapply(m[, i], degree, function(x) {
                sum(is.na(x))/length(x)
            }))
            if (sum(freq > 1 - Frule) > 0) 
                dx <- c(dx, i)
        }
        if (length(dx) > 0) 
            m <- m[, -dx]
        return(m)
    }
    classF <- as.factor(samPeno$class)
    #classF = addNA(classF)
    imsamFPF = FilterMV(imsamFP, classF)
    Frule_warning = paste("The number of filtered variables using the modified ", Frule * 100, "% rule :", 
        sep = " ")
    cat("\n", Frule_warning, " ", dim(imsamFP)[2] - dim(imsamFPF)[2], "\n")
    imsamFP = as.matrix(imsamFPF)
    ############## impute missing value#################
    cat("\n")
    cat("* Step 3: Imputation start...", "\n")
    
    
    if (imputeM == "KNN") {
        # require(impute)
        cat("\n", "The imputation method was set at 'KNN'")
        mvd <- impute::impute.knn(imsamFP, rowmax = 0.99, colmax = 0.99, maxp = 15000)
        inputedData <- mvd$data
    } else if (imputeM == "min") {
        cat("\n", "The imputation method was set at 'min'")
        
        minValue <- function(x, group) {
            group = as.factor(as.numeric(group))
            for (i in 1:dim(x)[1]) {
                for (j in 1:dim(x)[2]) {
                  if (is.na(x[i, j]) == TRUE) {
                    x[i, j] <- tapply(as.numeric(x[, j]), group, min, na.rm = TRUE)[group[i]]
                  }
                }
            }
            return(x)
        }
        inputedData = minValue(imsamFP, classF)
        # inputedData = inputedData[,-1]
        
    } else if (imputeM == "minHalf") {
        cat("\n", "The imputation method was set at 'minHalf'")
        
        minHalfValue <- function(x, group) {
            group = as.factor(as.numeric(group))
            for (i in 1:dim(x)[1]) {
                for (j in 1:dim(x)[2]) {
                  if (is.na(x[i, j]) == TRUE) {
                    x[i, j] <- tapply(as.numeric(x[, j]), group, min, na.rm = TRUE)[group[i]]/2
                  }
                }
            }
            return(x)
        }
        inputedData = minHalfValue(imsamFP, classF)
        # inputedData = inputedData[,-1]
        
    } else if (imputeM == "median") {
        
        medianvalue <- function(x, group) {
            group = as.factor(as.numeric(group))
            for (i in 1:dim(x)[1]) {
                for (j in 1:dim(x)[2]) {
                  if (is.na(x[i, j]) == TRUE) {
                    x[i, j] <- tapply(as.numeric(x[, j]), group, median, na.rm = TRUE)[group[i]]
                  }
                }
            }
            return(x)
        }
        
        cat("\n", "The imputation method was set at 'median'")
        
        inputedData = medianvalue(imsamFP, classF)
        # inputedData = inputedData[,-1]
    }
    cat("\n", "The number of missing value after imputation: ", sum(is.na(inputedData)))
    
    
    if (sum(is.na(inputedData)) > 0) {
        inputedData = minHalfValue(inputedData, classF)
        cat("\n", "The number of missing value after the second imputation: ", sum(is.na(inputedData)))
    }
    
    
    cat("\n", "Imputation Finished!", "\n", "\n")
    
    cat("* Step 4: Signal Correction Start... Time: ", date(), "\n")
    
    dat <- as.matrix(t(inputedData))
    #numX <- 1:dim(dat)[2]
    
    
    if (MLmethod == "Combat") {
        # require(randomForest)
        cat("\n", "The Signal Correction method was set at Combat", "\n")
        
       #suprr <- capture.output(
        if(mod.covariates) {
          batchinfo <- samPeno_stat$batch
          covar <- as.factor(samPeno_stat[,4])
          if(length(levels(covar)) - length(covar) == 0 ) stop("No. of covariates levels should be not equal to No. of samples")
          mod = model.matrix(~ covar, data=samPeno_stat)
          comba <- ComBat_qcFree(dat=dat, batch=batchinfo, mod=mod, 
                                 par.prior=par.prior, prior.plots=prior.plots,
                                 ref.batch = batch.Num)
        } else {
          batchinfo <- samPeno_stat$batch
        comba <- ComBat_qcFree(dat=dat, batch=batchinfo, mod=NULL, 
                              par.prior=par.prior, prior.plots=prior.plots,
                              ref.batch = batch.Num)
        }
       #)
       }
    
    combOutput <- as.data.frame(cbind(samPeno_stat$sample,samPeno_stat$class,t(comba)),
                               stringsAsFactors =FALSE)
    colnames(combOutput)[1] <- "sample";colnames(combOutput)[2] <- "class";
    
    combraw <- as.data.frame(cbind(samPeno_stat$sample,samPeno_stat$class,t(dat)),
                                stringsAsFactors =FALSE)
    colnames(combraw)[1] <- "sample";colnames(combraw)[2] <- "class";
    
    
    ############### dataCheck

    dirout.uni = paste(getwd(), "/statTarget/", sep = "")
    dirsc.ID = getwd()
    dir.create(dirout.uni)
    dirout.w = paste(getwd(), "/statTarget/shiftCor", sep = "")
    dir.create(dirout.w)
    dirout.Bs = paste(getwd(), "/statTarget/shiftCor/Before_shiftCor", sep = "")
    dir.create(dirout.Bs)
    dirout.As = paste(getwd(), "/statTarget/shiftCor/After_shiftCor", sep = "")
    dir.create(dirout.As)

    
    combafter = paste("shift_cor_Combat", ".csv", sep = "")
    write.csv(combOutput, paste(dirout.As, combafter, sep = "/"), row.names = FALSE)
    
    combabefore = paste("shift_raw_Combat", ".csv", sep = "")
    write.csv(combraw, paste(dirout.Bs, combabefore, sep = "/"), row.names = FALSE)
    
    
    cat("", "Output Link:", getwd(), "\n")
    cat("\n", "Correction Finished! Time: ", date(), "\n")
    cat("\n", "####################################", "\n")
    cat(" # Software Version: statTarget 2.0 #", "\n")
    cat(" ####################################", "\n")
    
    ################## Loess Plot########################
    setwd(dirsc.ID)
    
    # parameter output
    scPam1 <- c("Frule", "MLmethod", "imputeM", "par.prior","mod.covariates","batch.Num")
    scPam2 <- c(Frule, MLmethod, imputeM, par.prior,mod.covariates, paste("No.", batch.Num,sep = ""))
    scpam <- data.frame(scPam1, scPam2)
    colnames(scpam) <- c("parameter", "value")
    par_sh = paste("statTarget/ParameterShiftCor", ".log", sep = "")
    write.table(scpam, paste(getwd(), par_sh, sep = "/"), row.names = FALSE)
    
    tmpfilesc = paste(getwd(), "/tmp", sep = "")
    unlink(tmpfilesc, recursive = TRUE)
}




rowVars <- function(x, ...) 
{
  sqr = function(x) x * x
  n = rowSums(!is.na(x))
  n[n <= 1] = NA
  return(rowSums(sqr(x - rowMeans(x, ...)), ...)/(n - 1))
}

aprior <- function(gamma.hat){m=mean(gamma.hat); s2=var(gamma.hat); (2*s2+m^2)/s2}
bprior <- function(gamma.hat){m=mean(gamma.hat); s2=var(gamma.hat); (m*s2+m^3)/s2}

it.sol  <- function(sdat,g.hat,d.hat,g.bar,t2,a,b,conv=.0001){
  n <- apply(!is.na(sdat),1,sum)
  g.old <- g.hat
  d.old <- d.hat
  change <- 1
  count <- 0
  while(change>conv){
    g.new <- postmean(g.hat,g.bar,n,d.old,t2)
    sum2 <- apply((sdat-g.new%*%t(rep(1,ncol(sdat))))^2, 1, sum,na.rm=T)
    d.new <- postvar(sum2,n,a,b)
    change <- max(abs(g.new-g.old)/g.old,abs(d.new-d.old)/d.old)
    g.old <- g.new
    d.old <- d.new
    count <- count+1
  }
  #cat("This batch took", count, "iterations until convergence\n")
  adjust <- rbind(g.new, d.new)
  rownames(adjust) <- c("g.star","d.star")
  adjust
}

postmean <- function(g.hat,g.bar,n,d.star,t2){(t2*n*g.hat+d.star*g.bar)/(t2*n+d.star)}
postvar <- function(sum2,n,a,b){(.5*sum2+b)/(n/2+a-1)}

int.eprior <- function(sdat,g.hat,d.hat){
  g.star <- d.star <- NULL
  r <- nrow(sdat)
  for(i in 1:r){
    g <- g.hat[-i]
    d <- d.hat[-i]		
    x <- sdat[i,!is.na(sdat[i,])]
    n <- length(x)
    j <- numeric(n)+1
    dat <- matrix(as.numeric(x),length(g),n,byrow=T)
    resid2 <- (dat-g)^2
    sum2 <- resid2%*%j
    LH <- 1/(2*pi*d)^(n/2)*exp(-sum2/(2*d))
    LH[LH=="NaN"]=0
    g.star <- c(g.star,sum(g*LH)/sum(LH))
    d.star <- c(d.star,sum(d*LH)/sum(LH))
    #if(i%%1000==0){cat(i,'\n')}
  }
  adjust <- rbind(g.star,d.star)
  rownames(adjust) <- c("g.star","d.star")
  adjust	
} 

### Loplot, a visible figure of signal correction loplot provide the visible figure of QC-based
### correction.  x The file before QC-based correction.  z The file after QC-based correction.  i
### The name of variable.  The plot of RSD distribution and data table.
loplot <- function(x, z, i) {
    # x is the Corrected Data
    cn <- colnames(x)
    qcid <- grep("QC", cn)
    meantem <- mean(z[i, qcid])
    sdtem <- sd(z[i, qcid])
    rsdcutoffU <- 0.3
    rsdcutoffL <- 0.15
    # rsdcutoffX <- 1.0
    sdLineU <- rsdcutoffU * meantem
    sdLineL <- rsdcutoffL * meantem
    cuttemU <- meantem + sdLineU
    cuttemD <- meantem - sdLineU
    cuttemUL <- meantem + sdLineL
    cuttemDL <- meantem - sdLineL
    
    meantemR <- mean(x[i, qcid])
    sdtemR <- sd(x[i, qcid])
    rsdcutoffRU <- 0.3
    rsdcutoffRL <- 0.15
    # rsdcutoffRX <- 1.0
    sdLineRU <- rsdcutoffRU * meantemR
    sdLineRL <- rsdcutoffRL * meantemR
    cuttemRU <- meantemR + sdLineRU
    cuttemRD <- meantemR - sdLineRU
    cuttemRUL <- meantemR + sdLineRL
    cuttemRDL <- meantemR - sdLineRL
    
    
    RSD30_CV = paste(rownames(x)[i], "_", i, ".pdf", sep = "")
    dirout.loplot <- paste(getwd(), "/statTarget/shiftCor/After_shiftCor/loplot", sep = "")
    if (!file.exists(dirout.loplot)){
      dir.create(dirout.loplot,showWarnings = FALSE)
    }
    
    pdf(paste(dirout.loplot, RSD30_CV, sep = "/"), width = 6, height = 6)
    graphics::layout(matrix(1:2, nrow = 2))
    
    numY <- 1:dim(x)[2]
    graphics::plot(numY, x[i, ], pch = 19, col = "yellow", ylab = c("Intensity"), xlab = c("Injection Order"), 
        main = "Raw Peak")
    points(qcid, x[i, qcid], pch = 19, col = "blue")
    abline(h = cuttemRU, col = "#D55E00", lwd = 1.2, lty = 1)
    abline(h = cuttemRD, col = "#D55E00", lwd = 1.2, lty = 1)
    abline(h = cuttemRUL, col = rgb(0, 0, 0, 0.3), lwd = 1.2, lty = 2)
    abline(h = cuttemRDL, col = rgb(0, 0, 0, 0.3), lwd = 1.2, lty = 2)
    mtext("+SD@30%CV", side = 4, line = 0.3, at = cuttemRU, col = "#D55E00", cex = 0.3, las = 1)
    mtext("-SD@30%CV", side = 4, line = 0.3, at = cuttemRD, col = "#D55E00", cex = 0.3, las = 1)
    mtext("+SD@15%CV", side = 4, line = 0.3, at = cuttemRUL, col = "#D55E00", cex = 0.3, las = 1)
    mtext("-SD@15%CV", side = 4, line = 0.3, at = cuttemRDL, col = "#D55E00", cex = 0.3, las = 1)
    
    legend("top", c("Sample", "QC"), col = c("yellow", "blue"), lty = 1, pch = 19, bty = "n", cex = 0.75, 
        horiz = TRUE)
    # lines(qcid,x[i,qcid],col=rgb(0,0,0,0.3),lwd=4) loe <- loess(x[i,qcid]~qcid)
    # points(numY,predict(loe,numY),type='l',col=rgb(0,0,0,0.3),lwd=4)
    
    graphics::plot(numY, z[i, ], pch = 19, col = "yellow", ylab = c("Intensity"), xlab = c("Injection Order"), 
        main = "Corrected Peak")
    points(qcid, z[i, qcid], pch = 19, col = "blue")
    abline(h = cuttemUL, col = rgb(0, 0, 0, 0.3), lwd = 1.2, lty = 2)
    abline(h = cuttemDL, col = rgb(0, 0, 0, 0.3), lwd = 1.2, lty = 2)
    abline(h = cuttemU, col = "#D55E00", lwd = 1.2, lty = 1)
    abline(h = cuttemD, col = "#D55E00", lwd = 1.2, lty = 1)
    mtext("+SD@30%CV", side = 4, line = 0.3, at = cuttemU, col = "#D55E00", cex = 0.3, las = 1)
    mtext("-SD@30%CV", side = 4, line = 0.3, at = cuttemD, col = "#D55E00", cex = 0.3, las = 1)
    mtext("+SD@15%CV", side = 4, line = 0.3, at = cuttemUL, col = "#D55E00", cex = 0.3, las = 1)
    mtext("-SD@15%CV", side = 4, line = 0.3, at = cuttemDL, col = "#D55E00", cex = 0.3, las = 1)
    
    # lines(qcid,z[i,qcid],col=rgb(0,0,0,0.3),lwd=4) loe_n <- loess(z[i,qcid]~qcid)
    # points(numY,predict(loe_n,numY),type='l',col=rgb(0,0,0,0.3),lwd=4)
    legend("top", c("Sample", "QC"), col = c("yellow", "blue"), lty = 1, pch = 19, bty = "n", cex = 0.75, 
        horiz = TRUE)
    dev.off()
}

sT_univariate <- function(file, FDR = FDR, plot.volcano, upper.lim, lower.lim, sig.lim, save.boxplot) {
    dirout.uni = paste(getwd(), "/Univariate/", sep = "")
    dir.create(dirout.uni)
    comp = read.csv(file, sep = ",", header = TRUE)
    comp.x = comp[, 3:ncol(comp)]
    comp.x = cbind(comp[, 2], comp[, 1], comp.x)
    pwdfile = paste(getwd(), "/Univariate/DataTable.csv", sep = "")
    write.csv(comp.x, pwdfile, row.names = FALSE)
    
    # cal min no. of levels
    checkNum <- min(summary(as.factor(comp.x[,1])))
    
    cat("\n", "P-value Calculating...")
    
    if (FDR) {
        cat("\n", "P-value was adjusted using Benjamini-Hochberg Method")
    }
    
    shapiro(pwdfile)
    welch(pwdfile)
    wmw(pwdfile)
    cat("\n")
    cat("\n", "Odd.Ratio Calculating...", "\n")
    oddRatio(pwdfile)
    
    if(checkNum <= 5) {
      cat("\n", "*ROC analysis skipped", "\n")
    } else {
    cat("\n", "ROC Calculating...", "\n")
    aucROC(pwdfile)
    }
    # cat('\n','RandomForest Calculating...','\n')
    
    # RandomF(file,nvarRF)
    
    
    if (FDR) {
        pvalues(pwdfile, fdr = TRUE)
    } else {
        pvalues(pwdfile, fdr = FALSE)
    }
    col_pvalues(pwdfile)
    if (plot.volcano) {
        cat("\n", "Volcano Plot Output...", "\n")
        volcano(pwdfile, upper.lim = upper.lim, lower.lim = lower.lim, sig.lim = sig.lim)
    }
    if (save.boxplot) {
        cat("\n", "Box Plot Output...", "\n")
        boxPlot(pwdfile)
    }
}

---
title: 'QC-free approach with Combat method'
author: "Hemi Luan"
date: "Modified: 5 Dec 2017. Compiled: `r format(Sys.Date(), '%d %b %Y')`"
output: 
  BiocStyle::html_document:
    toc: true
       
vignette: >
  %\VignetteIndexEntry{QC_free approach with Combat method}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}  
---

```{r style, echo = FALSE, results = 'asis'}
BiocStyle::markdown()
```

```{r, echo = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

### Combat
'ComBat' allows users to adjust for batch effects in datasets where the batch covariate is known, using methodology described in Johnson et al. 2007. It uses either parametric or non-parametric empirical Bayes frameworks for adjusting data for batch effects.  Users are returned an expression matrix that has been corrected for batch effects.The function was revised accroding to 'sva' package (version = "3.8").


### statTargetGUI 

```
## Examples Code for graphical user interface 

library(statTarget)

statTargetGUI()

#For mac PC,  the GUI function 'statTargetGUI()' need the XQuartz instead of X11 support. Download it from https://www.xquartz.org. R 3.3.0 and RGtk2 2.20.31 are recommended for RGtk2 installation.

```


`statTargetGUI`



Download the [Reference manual](https://github.com/13479776/Picture/blob/master/statTarget-manual.pdf) and [example data](https://github.com/13479776/Picture/raw/master/Data_example.zip) .




### Running Signal Correction (the shiftCor function) from the GUI


`Meta File`
  
  
  Meta information includes the Sample name, class, batch and order. 
  Do not change the name of each column
  
  (1) Class: The group of samples. QC samples is not required, but the class should be available (no 'NA').
 
  (2) Order: The concent can be replaced by covariates if the mod.covariates (See shiftCor_dQC function) is TRUE.
  
  (3) Batch: The analysis blocks or batches
  
  (4) Sample name should be consistent in Meta file and Profile file
  
  (See the example data)


`Profile File`


Expression data includes the sample name and expression
data.(See the example data)


`NA.Filter`


Modified n precent rule function. A variable will be kept if it has a non-zero value for at least n precent of samples in any one group. (Default: 0.8)  
 

`MLmethod`
 

 The QC-based signal correction (i.e. QC-RFSC, QC-RLSC.) or QC-free methods (Combat) 
 

`Ntree`
	
	
Number of trees to grow for QC-RFSC (Default: 500) .


`QCspan`


The smoothing parameter for QCRLSC which controls the bias-variance tradeoff. 
The common range of QCspan value is from 0.5 to 0.75. If you choose
a span that is too small then there will be a large variance. 
If the span is too large, a large bias will be produced. 
The default value of QCspan is set at '0', the generalised 
cross-validation will be performed for choosing a good value, 
avoiding overfitting of the observed data. (Default: 0) 



`Imputation`


 The parameter for imputation method.(i.e., 
 nearest neighbor averaging, "KNN"; minimum values, "min"; Half of minimum values 
"minHalf" median values, "median". (Default: KNN) 



```
## Examples Code

library(statTarget)

datpath <- system.file('extdata',package = 'statTarget')
samPeno <- paste(datpath,'MTBLS79_sampleList.csv', sep='/')
samFile <- paste(datpath,'MTBLS79.csv', sep='/')
# Combat for QC-free datasets
samPeno2 <- paste(datpath,'MTBLS79_dQC_sampleList.csv', sep='/')
shiftCor_dQC(samPeno2,samFile, Frule = 0.8, MLmethod = "Combat")

See ?shiftCor_dQC for off-line help
```


 
## References

Luan H., Ji F., Chen Y., Cai Z. (2018) statTarget: A streamlined tool for signal drift correction and interpretations of quantitative mass spectrometry-based omics data. Analytica Chimica Acta. dio: https://doi.org/10.1016/j.aca.2018.08.002
 
Luan H., Ji F., Chen Y., Cai Z. (2018) Quality control-based signal drift correction and interpretations of metabolomics/proteomics data using random forest regression. bioRxiv 253583; doi: https://doi.org/10.1101/253583

Dunn, W.B., et al.,Procedures for large-scale metabolic profiling of
serum and plasma using gas chromatography and liquid chromatography coupled 
to mass spectrometry. Nature Protocols 2011, 6, 1060.

Luan H., LC-MS-Based Urinary Metabolite Signatures in Idiopathic 
Parkinson's Disease. J Proteome Res., 2015, 14,467.

Luan H., Non-targeted metabolomics and lipidomics LC-MS data 
from maternal plasma of 180 healthy pregnant women. GigaScience 2015 4:16

---
title: 'pathway analysis (mummichog)'
author: "Hemi Luan"
date: "Modified: 2019. Compiled: `r format(Sys.Date(), '%d %b %Y')`"
output: 
  BiocStyle::html_document:
    toc: true
       
vignette: >
  %\VignetteIndexEntry{statTarget2 for pathway analysis }
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}  
---

```{r style, echo = FALSE, results = 'asis'}
BiocStyle::markdown()
```

```{r, echo = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```


 
# statTarget result for pathway analysis using mummichog approach.

## Pathways analysis by using MS peaks with accurate mass 


1. to get the p.value from the file in /statTarget/statAnalysis/Univariate/Pvalues
  

2. to get the fold.change from the file in /statTarget/statAnalysis/Univariate/Fold_Changes
  

3. to get the m.z from the .tsv file from the xcms output.
  

4. to use the EXCEL for generation the ".txt" format with `column named m.z, p.value and t.score (fold.change value)`
  

submit the table to MS to pathway analysis panel in metaboanalyst web: https://www.metaboanalyst.ca/MetaboAnalyst/faces/upload/PeakUploadView.xhtml
  

---
title: 'statTarget'
author: "Hemi Luan"
date: "Modified: 5 Jan 2022. Compiled: `r format(Sys.Date(), '%d %b %Y')`"
output: 
  BiocStyle::html_document:
    toc: true
       
vignette: >
  %\VignetteIndexEntry{statTarget2 On using the Graphical User Interface}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}  
---

```{r style, echo = FALSE, results = 'asis'}
BiocStyle::markdown()
```

```{r, echo = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```



# Welcome to statTarget ! 

`Quality Control (QC)` has been considered as an essential step in the
MS-based omics platform for high reproducibility and accuracy of data. 
The repetitive use of the same QC samples is more and more accepted for 
evaluation and correction of the signal drift during the sequence of MS run order, 
especially beneficial to improve the quality of data in multi-blocks 
experiments.


   We have released a new version named [statTargetAPP](https://github.com/statTarget/statTarget2/releases/tag/statTarget) with GUI interface. If interested, you can contact via email (Prof. Hemi Luan <hm-luan at msn.com>)


##  Package overview

A streamlined tool provides graphical user interface for quality control based signal correction, integration of metabolomics and proteomics data from multi-batch experiments, and the comprehensive statistic analysis. 
(URL: https://stattarget.github.io)

##  Citation

 Please cite the following article when using statTarget or QC-RFSC algorithm:
 
 Luan H., Ji F., Chen Y., Cai Z. (2018) statTarget: A streamlined tool for signal drift correction and interpretations of quantitative mass spectrometry-based omics data. Analytica Chimica Acta. dio: https://doi.org/10.1016/j.aca.2018.08.002
 
 Luan H., Ji F., Chen Y., Cai Z. (2018) Quality control-based signal drift correction and interpretations of metabolomics/proteomics data using random forest regression. bioRxiv 253583; doi: https://doi.org/10.1101/253583


## What does statTarget offer statistically

The statTarget has two basic sections. The first section 
is Signal Correction (See the shiftCor function). It includes 'Ensemble Learning' for QC based signal correction. For example, QC-based random forest correction (QC-RFSC). In addition, Combat is also provided for QC free datasets. The second section is Statistical Analysis (See the statAnalysis function). It provides comprehensively computational and statistical methods that are commonly applied to analyze Omics data, and offers multiple results for biomarker discovery.


`Section 1 - Signal Correction` provide `ensemble learning method` for QC based signal correction. i.e. QC based random forest signal correction (QC-RFSC) that fit the QC data, 
and each metabolites in the true sample will be normalized to the QC sample. 

`Section 2 - Statistical Analysis` provide features including Data preprocessing, Data descriptions, Multivariate statistics analysis and Univariate analysis.


Data preprocessing : 80-precent rule, sum normalization (SUM) and probabilistic quotient normalization (PQN), glog transformation, KNN imputation, median imputation, and minimum imputation.


Data descriptions :  Mean value, Median value, sum, Quartile, Standard 
derivatives, etc.


Multivariate statistics analysis : PCA, PLSDA, VIP, Random forest, Permutation-based feature selection.


Univariate analysis : Welch's T-test, Shapiro-Wilk normality test and Mann-Whitney test.


Biomarkers analysis: ROC, Odd ratio, Adjusted P-value, Box plot and Volcano plot.



### Running Signal Correction (the shiftCor function) from the GUI


`Meta File`
  
  
  Meta information includes the Sample name, class, batch and order. 
  Do not change the name of each column
  
  (1) Class: The QC should be labeled as NA
 
  (2) Order : Injection sequence. 
  
  (3) Batch: The analysis blocks or batches
  
  (4) Sample: Sample name should be consistent in Meta file and Profile file. *The QC sample name should be tagged with "QC".
  
  (See the example data)


`Profile File`


Expression data includes the sample name and expression
data.(See the example data)


`NA.Filter`


Modified n precent rule function. A variable will be kept if it has a non-zero value for at least n precent of samples in any one group. (Default: 0.8)  
 

`MLmethod`
 

 The QC-based signal correction (i.e. QC-RFSC.) or QC-free methods (Combat) 
 

`Ntree`
	
	
Number of trees to grow for QC-RFSC (Default: 500) .



`Imputation`


 The parameter for imputation method.(i.e., 
 nearest neighbor averaging, "KNN"; minimum values, "min"; Half of minimum values 
"minHalf" median values, "median". (Default: KNN) 



```
## Examples Code

library(statTarget)

datpath <- system.file('extdata',package = 'statTarget')
samPeno <- paste(datpath,'MTBLS79_sampleList.csv', sep='/')
samFile <- paste(datpath,'MTBLS79.csv', sep='/')
shiftCor(samPeno,samFile, Frule = 0.8, MLmethod = "QCRFSC", imputeM = "KNN")
# Combat for QC-free datasets
samPeno2 <- paste(datpath,'MTBLS79_dQC_sampleList.csv', sep='/')
shiftCor_dQC(samPeno2,samFile, Frule = 0.8, MLmethod = "Combat")

See ?shiftCor for off-line help
```


### Running Statistical Analysis (the statAnalysis function) from the GUI


`Stat File`


Expression data includes the sample name, group, and expression data with long format.


`NA.Filter`


Modified n precent rule function. A variable will be kept if it has a non-zero value for at least n precent of samples in any one group. (Default: 0.8)  



`Imputation`


The parameter for imputation method.(i.e., nearest neighbor averaging, "KNN"; 
minimum values, "min"; Half of minimum values "minHalf"; median values, 
"median". (Default: KNN) 


`Normalization`


The parameter for normalization method (i.e probabilistic quotient 
normalization, "PQN"; integral normalization , "SUM", and "none").



`Glog`


Generalised logarithm (glog) transformation for Variance stabilization  
(Default: TRUE)


`Scaling Method`

Scaling method before statistic analysis i.e. PCA or PLS(DA). 
Center can be used for specifying the Center scaling. 
Pareto can be used for specifying the Pareto scaling. 
Auto can be used for specifying the Auto scaling (or unit variance scaling). 
Vast can be used for specifying the vast scaling. 
Range can be used for specifying the Range scaling. (Default: Pareto) 


`Permutation times`


The number of permutation times for cross-validation of PLS-DA model, and variable importance of randomforest model



`PCs`


PCs in the Xaxis or Yaxis: Principal components in 
PCA-PLS model for the x or y-axis (Default: 1 and 2) 


`nvarRF`


The number of variables with top permutation importance in randomforest model. 
(Default: 20) 


`Labels`


To show the name of sample or groups in the Score plot. (Default: TRUE) 


`Multiple testing`


This multiple testing correction via false discovery rate (FDR) 
estimation with Benjamini-Hochberg method. The false discovery rate 
for conceptualizing the rate of type I errors in null hypothesis 
testing when conducting multiple comparisons. (Default: TRUE) 



`Volcano FC`


The up or down -regulated metabolites using Fold Changes cut off 
values in the Volcano plot. (Default:  > 2 or < 0.5) 


`Volcano Pvalue`


The significance level for metabolites in the Volcano plot.(Default: 0.05) 


```
## Examples Code

library(statTarget)

datpath <- system.file('extdata',package = 'statTarget')
file <- paste(datpath,'data_example.csv', sep='/')
statAnalysis(file,Frule = 0.8, normM = "NONE", imputeM = "KNN", glog = TRUE,scaling = "Pareto")

```




### Generation of input file (the transX function)

The transX function is to generate statTarget input file formats from Mass Spectrometry Data softwares, such as XCMS, MZmine2, SIEVE and SKYLINE. '?transX' for off-line help.

```
## Examples Code

library(statTarget)

datpath <- system.file('extdata',package = 'statTarget')
dataXcms <- paste(datpath,'xcmsOutput.tsv', sep='/')
dataSkyline <- paste(datpath,'skylineDemo.csv', sep='/')
transX(dataXcms,'xcms')
transX(dataSkyline,'skyline')

See ?transX for off-line help
```

### Random Forest classfication and variable importance measures

rForest provides the Breiman’s random forest algorithm for classification and permutation-based variable importance measures (PIMP-algorithm).

```
## Examples Code

library(statTarget)

datpath <- system.file('extdata',package = 'statTarget')
statFile <- paste(datpath,'data_example.csv', sep='/')
getFile <- read.csv(statFile,header=TRUE)

# Random Forest classfication 
rFtest <- rForest(getFile,ntree = 10,times = 5)

# Prediction of test data using random forest in statTarget.
predictOutput <- predict_RF(rFtest, getFile[1:19,3:8])

# Multi-dimensional scaling plot of proximity matrix from randomForest.
mdsPlot(rFtest$randomForest,rFtest$pimpTest)

# Create plots for Gini importance and permutation-based variable Gini importance measures.
pvimPlot(rFtest$randomForest,rFtest$pimpTest)

graphics.off()

See ?rForest for off-line help
```

Once data files have been analysed it is time to investigate them. 
Please get this info. through the GitHub page.
(URL: https://stattarget.github.io)

#### Results of Signal Correction (ShiftCor)


- __The output file: __

```
statTarget -- shiftCor 
-- After_shiftCor             # The fold for intergrated and corrected data 
   -- shift_all_cor.csv         # The corrected data of samples and QCs
   -- shift_QC_cor.csv          # The corrected data of QCs only
   -- shift_sample_cor.csv      # The corrected data of samples only
   -- loplot                    # The fold for quality control images
      -- *.pdf                  # The quality control images for each features

-- Before_shiftCor            # The fold for raw data
   -- shift_QC_raw.csv          # The raw data of QCs
   -- shift_sam_raw.csv         # The raw data of samples
   
-- RSDresult                  # The fold for variation analysis and quality assessment
   -- RSD_all.csv               # The RSD values of each feature
   -- RSDdist_QC_stat.csv       # The RSD distribution of QCs in each batch and all batches
   -- RSD distribution.pdf      # The RSD distribution plot in samples and QCs of all batches
   -- RSD variation.pdf         # The RSD variation plot for pre- and post- signal correction

```

 
## References

Luan H., Ji F., Chen Y., Cai Z. (2018) statTarget: A streamlined tool for signal drift correction and interpretations of quantitative mass spectrometry-based omics data. Analytica Chimica Acta. dio: https://doi.org/10.1016/j.aca.2018.08.002
 
Luan H., Ji F., Chen Y., Cai Z. (2018) Quality control-based signal drift correction and interpretations of metabolomics/proteomics data using random forest regression. bioRxiv 253583; doi: https://doi.org/10.1101/253583

Dunn, W.B., et al.,Procedures for large-scale metabolic profiling of
serum and plasma using gas chromatography and liquid chromatography coupled 
to mass spectrometry. Nature Protocols 2011, 6, 1060.

Luan H., LC-MS-Based Urinary Metabolite Signatures in Idiopathic 
Parkinson's Disease. J Proteome Res., 2015, 14,467.

Luan H., Non-targeted metabolomics and lipidomics LC-MS data 
from maternal plasma of 180 healthy pregnant women. GigaScience 2015 4:16

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shiftCor_v2.R
\name{shiftCor}
\alias{shiftCor}
\title{shiftCor}
\usage{
shiftCor(
  samPeno,
  samFile,
  Frule = 0.8,
  MLmethod = "QCRFSC",
  ntree = 500,
  imputeM = "KNN",
  coCV = 30,
  plot = FALSE
)
}
\arguments{
\item{samPeno}{File path. The file with the meta information including the sample name,
batches, class and order.}

\item{samFile}{File path. The file with the expression information.}

\item{Frule}{Modified n precent rule function. A variable will be kept if it has a non-zero value
for at least n precent of samples in any one group. 
(Default: 0.8)}

\item{MLmethod}{The machine learning method for QC based signal correction, such as QC based random forest signal correction (QC-RFSC). QC-RLSC was deprecated .}

\item{ntree}{Number of trees to grow in random forest model.}

\item{imputeM}{The parameter for imputation method i.e., nearest neighbor 
averaging, 'KNN'; minimum values, 'min'; Half of minimum values, 'minHalf'; 
median values, 'median'.}

\item{coCV}{Define the cutoff value (0-100) of CV for controlling the number of features.}

\item{plot}{Defines if images of feature quality should be generated (TRUE) or not (FALSE). 
Defaults to FALSE.}
}
\value{
the shiftCor files. See the details at https://stattarget.github.io
}
\description{
shiftCor provides the QC based signal correction for 
large scale metabolomics and targeted proteomics.
}
\examples{
datpath <- system.file('extdata',package = 'statTarget')
samPeno <- paste(datpath,'MTBLS79_sampleList.csv', sep='/')
samFile <- paste(datpath,'MTBLS79.csv', sep='/')
samPeno
samFile
shiftCor(samPeno,samFile, MLmethod = 'QCRFSC', imputeM = 'KNN',coCV = 30)
}
\keyword{Controls,Correction}
\keyword{Quality}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statAnalysis.R
\name{statAnalysis}
\alias{statAnalysis}
\title{statAnalysis for statistical analysis for omics data or others.}
\usage{
statAnalysis(
  file,
  Frule = 0.8,
  normM = "NONE",
  imputeM = "KNN",
  glog = TRUE,
  FDR = TRUE,
  ntree = 500,
  nvarRF = 5,
  scaling = "Pareto",
  plot.volcano = TRUE,
  save.boxplot = FALSE,
  silt = 20,
  pcax = 1,
  pcay = 2,
  Labels = TRUE,
  upper.lim = 2,
  lower.lim = 0.5,
  sig.lim = 0.05
)
}
\arguments{
\item{file}{The file with the expression information with long format}

\item{Frule}{Modified n precent rule function. A variable will be kept if it has a non-zero value
for at least n precent of samples in any one group. (Default: 0.8)}

\item{normM}{The parameter for normalization method (i.e median quotient 
normalization, 'PQN'; integral normalization , 'SUM', and 'NONE').}

\item{imputeM}{The parameter for imputation method i.e., nearest neighbor 
averaging, 'KNN'; minimum values, 'min'; Half of minimum values, 'minHalf'; 
median values, 'median'.}

\item{glog}{Generalised logarithm (glog) transformation, with the default 
value TRUE. The glog is a better behaved log transformation when some data 
values are zero or just near zero.}

\item{FDR}{The false discovery rate for conceptualizing the rate of type I 
errors in null hypothesis testing when conducting multiple comparisons.}

\item{ntree}{Number of trees to grow for randomForest model. This should not 
be set to too small a number, to ensure that every input row gets predicted 
at least a few times.}

\item{nvarRF}{The number of the variables with top importance in randomforest
model}

\item{scaling}{Scaling method before statistic analysis (PCA or PLS-DA). 
'pareto', 'Pareto', 'p' or 'P' can be used for specifying the Pareto scaling.
 'auto', 'Auto', 'auto', 'a' or 'A' can be used for specifying the Auto 
 scaling (or unit variance scaling). 'vast', 'Vast', 'v' or 'V' can be used
  for specifying the vast scaling. 'range', 'Range', 'r' or 'R' can be used 
  for specifying the Range scaling.}

\item{plot.volcano}{if TRUE, the volcano plot is performed}

\item{save.boxplot}{if TRUE, the box plot is performed}

\item{silt}{The number of permutation for PLS-DA model and variable importance of randomForest.}

\item{pcax}{Principal components in PCA model for the x-axis.}

\item{pcay}{Principal components in PCA model for the y-axis.}

\item{Labels}{Name labels for score plot of multiple statistical analysis}

\item{upper.lim}{The up-regulated metabolites using Fold Changes cut off 
values in the Volcano plot. Fold change values will be calculated 
before normalization step.}

\item{lower.lim}{The down-regulated metabolites using Fold Changes cut off
values in the Volcano plot. Fold change values will be calculated before 
normalization step.}

\item{sig.lim}{The significance level for metabolites in the Pvalues file in 
the Volcano plot.}
}
\value{
The statAnalsis output files. See the details at https://stattarget.github.io
}
\description{
statAnalysis provides the statistical analysis for metabolomics
 data or others.
}
\examples{
datpath <- system.file('extdata',package = 'statTarget')
file <- paste(datpath,'data_example_long.csv', sep='/')
statAnalysis(file,Frule = 0.8, normM = 'NONE', imputeM = 'KNN', glog = TRUE,scaling = 'Pareto')
}
\author{
Hemi Luan, hemi.luan@gmail.com
}
\keyword{P-value}
\keyword{PCA}
\keyword{PLSDA}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statTarget_rforest.R
\name{mdsPlot}
\alias{mdsPlot}
\title{MDSplot in statTarget}
\usage{
mdsPlot(rForest,pimpModel,Labels = TRUE,slink = FALSE, 
slinkDat, ...)
}
\arguments{
\item{rForest}{An object of class randomForest that contains the proximity 
component from statTarget_rForest function.}

\item{pimpModel}{An object of permutation-based variable Gini importance 
measures (PIMP-algorithm) from statTarget_rForest function.}

\item{Labels}{Labels is TRUE for visible the sample name in the figure else 
with the index for class.}

\item{slink}{Logical indicating if slinkDat is active for extenal classID.}

\item{slinkDat}{A data frame for the extenal classID.}

\item{...}{A generic MDSplot function in randomForest package}
}
\value{
The output of cmdscale on 1 - rf$proximity is returned invisibly.
}
\description{
Multi-dimensional scaling plot of proximity matrix from 
randomForest.
}
\seealso{
MDSplot
}
\author{
Hemi Luan, hemi.luan@gmail.com
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shiftCor_combat.R
\name{shiftCor_dQC (deprecated)}
\alias{shiftCor_dQC (deprecated)}
\alias{shiftCor_dQC}
\title{QC-free based signal correction}
\usage{
shiftCor_dQC(
  samPeno,
  samFile,
  Frule = 0.8,
  imputeM = "KNN",
  MLmethod = "Combat",
  par.prior = TRUE,
  prior.plots = FALSE,
  mod.covariates = FALSE,
  batch.Num = NULL
)
}
\arguments{
\item{samPeno}{The file with the meta information including the sample name,
batches, class and order (denoting other covariates besides batch).}

\item{samFile}{The file with the expression information.}

\item{Frule}{Modified n precent rule function. A variable will be kept if it has a non-zero value
for at least n precent of samples in any one group. 
(Default: 0.8)}

\item{imputeM}{The parameter for imputation method i.e., nearest neighbor 
averaging, 'KNN'; minimum values, 'min'; Half of minimum values, 'minHalf'; 
median values, 'median'.}

\item{MLmethod}{'ComBat' allows users to adjust for batch effects in datasets where the batch covariate is known, using methodology
described in Johnson et al. 2007. It uses either parametric or non-parametric empirical Bayes frameworks for adjusting data for
batch effects.  Users are returned an expression matrix that has been corrected for batch effects.The function was revised accroding to 'sva' package (version = "3.8").}

\item{par.prior}{TRUE indicates parametric adjustments will be used, FALSE indicates non-parametric adjustments will be used}

\item{prior.plots}{(Optional) TRUE give prior plots.}

\item{mod.covariates}{TRUE indicates model matrix for outcome of interest and other covariates besides batch (Column 'order' denotes covariates the  in samPeno file).}

\item{batch.Num}{(Optional) NULL If given, will use the selected batch as a reference for batch adjustment.}
}
\value{
the shiftCor files. See the details at https://stattarget.github.io
}
\description{
shiftCor_dQC provides the QC-free based signal correction for 
large scale mass spectrometry-based omics data.
}
\examples{
datpath <- system.file('extdata',package = 'statTarget')
samPeno <- paste(datpath,'MTBLS79_dQC_sampleList.csv', sep='/')
samFile <- paste(datpath,'MTBLS79.csv', sep='/')
shiftCor_dQC(samPeno,samFile, Frule = 0.8, MLmethod = "Combat",mod.covariates = FALSE)
shiftCor_dQC(samPeno,samFile, Frule = 0.8, MLmethod = "Combat",mod.covariates = TRUE,batch.Num = 1)
}
\keyword{Controls,Correction}
\keyword{Quality}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/randomForest_ptest.R
\name{pvimPlot}
\alias{pvimPlot}
\title{Gini importance and permutation-based variable 
importance measures plots}
\usage{
pvimPlot(rForest,pimpModel,nvarRF = 6,border= NA,
space = 0.3,...)
}
\arguments{
\item{rForest}{an object of class randomForest that contains the proximity 
component from statTarget rForest function.}

\item{pimpModel}{an object of permutation-based variable Gini importance 
measures (PIMP-algorithm) from statTarget rForest function.}

\item{nvarRF}{The number of variables in importance plot of randomForest.}

\item{border}{The color to be used for the border of the bars. 
Use border = NA to omit borders. see also barplot.}

\item{space}{The amount of space (as a fraction of the average bar width) 
left before each bar. May be given as a single number or one number per bar.
see also barplot}

\item{...}{A generic barplot function from graphics package.}
}
\value{
The output of the name of selected variable importance.
}
\description{
Create plots for Gini importance and permutation-based variable 
Gini importance measures.
}
\examples{
datpath <- system.file('extdata',package = 'statTarget')
statFile <- paste(datpath,'data_example_long.csv', sep='/')
getFile <- read.csv(statFile,header=TRUE)
rFtest <- rForest(getFile,ntree = 10,times = 5)
pvimPlot(rFtest$randomForest,rFtest$pimpTest)
}
\author{
Hemi Luan, hemi.luan@gmail.com
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statTarget-package.R
\docType{package}
\name{statTarget-package}
\alias{statTarget-package}
\alias{statTarget}
\title{Statistical Analysis of Molecular Profiles}
\usage{
statTarget()
}
\value{
A description of statTarget. See the details at https://stattarget.github.io
}
\description{
An streamlined tool provides graphical user interface 
for quality control based signal correction, integration of 
MS-based data from multiple batches, and the 
comprehensive statistical analysis for omics studies.
}
\details{
Package: statTarget

Type: package

License: LGPL (>= 3)
}
\author{
Hemi Luan

Maintainer: Hemi Luan hemi.luan@gmail.com
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict_RF.R
\name{predict_RF}
\alias{predict_RF}
\title{preidict function for random forest objects in statTarget}
\usage{
predict_RF(object, newdata, type='response',...)
}
\arguments{
\item{object}{An object created by the function statTarget_rForest.}

\item{newdata}{A data frame or matrix containing new data. (Note: If not 
given, the out-of-bag prediction in object is returned. see randomForest 
package.}

\item{type}{One of response, prob. or votes, indicating the type of output: 
predicted values, matrix of class probabilities, or matrix of vote counts. 
class is allowed, but automatically converted to 'response', for backward 
compatibility.}

\item{...}{A generic predict function from randomForest package.}
}
\value{
A class of predicted values is returned. 
Object type is classification, for detail see randomForest package.
}
\description{
Prediction of test data using random forest in statTarget.
}
\seealso{
randomForest
}
\author{
Hemi Luan, hemi.luan@gmail.com
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statTarget_rforest.R
\name{rForest}
\alias{rForest}
\title{Random Forest classfication in statTarget}
\usage{
rForest(file,ntree = 100,times = 100, gDist = TRUE,
seed = 123,...)
}
\arguments{
\item{file}{An data frame or 'Stat File' from statTarget software.}

\item{ntree}{Number of trees to grow. This should not be set to too small a 
number, to ensure that every input row gets predicted at least a few times.}

\item{times}{The number of permutations for 
permutation-based variable importance measures.}

\item{gDist}{If gDist is TRUE the null importance distributions are 
approximated with Gaussian distributions else with empirical cumulative 
distributions.}

\item{seed}{For the same set of random variables and reproducible results.}

\item{...}{A generic function in randomForest package}
}
\value{
Objects  Two objects from statTarget_rForest (1. randomForest,rfModel; 
2. PIMPresult, pimpModel)

VarImp  The original Gini importance

PerVarImp  A matrix, where the permuted VarImp measures for the
predictor variable.

p-value  The probability of observing the original VarImp or a larger
value, given the fitted null importance distribution.

p.ks.test The p-values of the Kolmogorov-Smirnov Tests for 
each row PerVarImp.
}
\description{
rForest provides the Breiman's random 
forest algorithm for classification and permutation-based variable 
importance measures (PIMP-algorithm).
}
\examples{
datpath <- system.file('extdata',package = 'statTarget')
statFile <- paste(datpath,'data_example_long.csv', sep='/')
getFile <- read.csv(statFile,header=TRUE)
rFtest <- rForest(getFile,ntree = 10,times = 5)
}
\references{
Altmann A.,Tolosi L.,Sander O. and Lengauer T. (2010) Permutation importance: 
a corrected feature importance measure, Bioinformatics 26 (10), 1340-1347.

Ender Celik. (2015) vita: Variable Importance Testing Approaches. R package 
version 1.0.0 https://CRAN.R-project.org/package=vita
}
\author{
Hemi Luan, hemi.luan@gmail.com
}
